@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,12 +12,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
-#ifndef MIR_COMPOSITOR_BUFFER_H_
-#define MIR_COMPOSITOR_BUFFER_H_
+#ifndef MIR_GRAPHICS_BUFFER_H_
+#define MIR_GRAPHICS_BUFFER_H_
 
 #include "mir/graphics/native_buffer.h"
 #include "mir/graphics/buffer_id.h"
@@ -25,27 +23,32 @@
 #include "mir_toolkit/common.h"
 
 #include <memory>
+#include <functional>
 
 namespace mir
 {
 namespace graphics
 {
 
+class NativeBufferBase
+{
+protected:
+    NativeBufferBase() = default;
+    virtual ~NativeBufferBase() = default;
+    NativeBufferBase(NativeBuffer const&) = delete;
+    NativeBufferBase operator=(NativeBuffer const&) = delete;
+};
+
 class Buffer
 {
 public:
     virtual ~Buffer() {}
 
-    virtual std::shared_ptr<NativeBuffer> native_buffer_handle() const = 0;
     virtual BufferID id() const = 0;
     virtual geometry::Size size() const = 0;
-    virtual geometry::Stride stride() const = 0;
     virtual MirPixelFormat pixel_format() const = 0;
-    virtual void gl_bind_to_texture() = 0;
-    /* TODO: remove this function, as it is specific to the mesa platform */
-    virtual bool can_bypass() const = 0;
-    
-    virtual void write(unsigned char const* pixels, size_t size) = 0;
+
+    virtual NativeBufferBase* native_buffer_base() = 0;
 
 protected:
     Buffer() = default;
@@ -53,4 +56,4 @@ protected:
 
 }
 }
-#endif // MIR_COMPOSITOR_BUFFER_H_
+#endif // MIR_GRAPHICS_BUFFER_H_
