@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
 #ifndef MIR_GRAPHICS_DISPLAY_CONFIGURATION_H_
@@ -23,19 +21,32 @@
 #include "mir/geometry/size.h"
 #include "mir/geometry/rectangle.h"
 #include "mir/geometry/point.h"
+#include "mir/graphics/gamma_curves.h"
+#include "mir/optional_value.h"
 #include "mir_toolkit/common.h"
 
+#include <glm/glm.hpp>
+
 #include <functional>
+#include <map>
+#include <memory>
+#include <optional>
 #include <vector>
 
 namespace mir
 {
 namespace graphics
 {
-namespace detail { struct GraphicsConfCardIdTag; struct GraphicsConfOutputIdTag; }
+namespace detail
+{
+struct GraphicsConfCardIdTag;
+struct GraphicsConfOutputIdTag;
+struct GraphicsConfLogicalGroupId;
+}
 
 typedef IntWrapper<detail::GraphicsConfCardIdTag> DisplayConfigurationCardId;
 typedef IntWrapper<detail::GraphicsConfOutputIdTag> DisplayConfigurationOutputId;
+typedef IntWrapper<detail::GraphicsConfLogicalGroupId> DisplayConfigurationLogicalGroupId;
 
 /**
  * Configuration information for a display card.
@@ -51,21 +62,24 @@ struct DisplayConfigurationCard
  */
 enum class DisplayConfigurationOutputType
 {
-    unknown,
-    vga,
-    dvii,
-    dvid,
-    dvia,
-    composite,
-    svideo,
-    lvds,
-    component,
-    ninepindin,
-    displayport,
-    hdmia,
-    hdmib,
-    tv,
-    edp
+    unknown     = mir_output_type_unknown,
+    vga         = mir_output_type_vga,
+    dvii        = mir_output_type_dvii,
+    dvid        = mir_output_type_dvid,
+    dvia        = mir_output_type_dvia,
+    composite   = mir_output_type_composite,
+    svideo      = mir_output_type_svideo,
+    lvds        = mir_output_type_lvds,
+    component   = mir_output_type_component,
+    ninepindin  = mir_output_type_ninepindin,
+    displayport = mir_output_type_displayport,
+    hdmia       = mir_output_type_hdmia,
+    hdmib       = mir_output_type_hdmib,
+    tv          = mir_output_type_tv,
+    edp         = mir_output_type_edp,
+    virt        = mir_output_type_virtual,
+    dsi         = mir_output_type_dsi,
+    dpi         = mir_output_type_dpi,
 };
 
 /**
@@ -86,6 +100,8 @@ struct DisplayConfigurationOutput
     DisplayConfigurationOutputId id;
     /** The id of the card the output is connected to. */
     DisplayConfigurationCardId card_id;
+    /** The id of the logical output group (aka display wall) this output belongs to, or 0 for none. */
+    DisplayConfigurationLogicalGroupId logical_group_id;
     /** The type of the output. */
     DisplayConfigurationOutputType type;
     /** The pixel formats supported by the output */
@@ -111,14 +127,43 @@ struct DisplayConfigurationOutput
     MirOrientation orientation;
 
     /** Requested scale factor for this output, for HiDPI support */
-    float scale;
+    float scale{1.0f};
     /** Form factor of this output; phone display, tablet, monitor, TV, projector... */
     MirFormFactor form_factor;
 
+    /** Subpixel arrangement of this output */
+    MirSubpixelArrangement subpixel_arrangement;
+
+    /** The current gamma for the display */
+    GammaCurves gamma;
+    MirOutputGammaSupported gamma_supported;
+
+    /** EDID of the display, if non-empty */
+    std::vector<uint8_t> edid;
+
+    mir::optional_value<geometry::Size> custom_logical_size;
+
+    /** The output's displayable name. */
+    // TODO: Better output names that are consistant between sessions
+    std::string name = "OUT-" + std::to_string(id.as_value());
+
+    /// Custom attributes (typically set via the .display configuration file
+    std::map<std::string const, std::optional<std::string>> custom_attribute = {};
+
     /** The logical rectangle occupied by the output, based on its position,
         current mode and orientation (rotation) */
     geometry::Rectangle extents() const;
     bool valid() const;
+
+    /**
+     * The additional transformation (if any) that this output must undergo
+     * to appear correctly on screen, including rotation (orientation),
+     * inversion and skew.
+     *   Note that scaling and translation are not part of this transformation
+     * matrix because those are already built in to the extents() rectangle
+     * for the renderer to use in compositing.
+     */
+    glm::mat2 transformation() const;
 };
 
 /**
@@ -129,6 +174,7 @@ struct UserDisplayConfigurationOutput
 {
     DisplayConfigurationOutputId const& id;
     DisplayConfigurationCardId const& card_id;
+    DisplayConfigurationLogicalGroupId& logical_group_id;
     DisplayConfigurationOutputType const& type;
     std::vector<MirPixelFormat> const& pixel_formats;
     std::vector<DisplayConfigurationMode> const& modes;
@@ -143,8 +189,16 @@ struct UserDisplayConfigurationOutput
     MirOrientation& orientation;
     float& scale;
     MirFormFactor& form_factor;
+    MirSubpixelArrangement& subpixel_arrangement;
+    GammaCurves& gamma;
+    MirOutputGammaSupported const& gamma_supported;
+    std::vector<uint8_t const> const& edid;
+    mir::optional_value<geometry::Size>& custom_logical_size;
+    std::string const& name;
+    /// Custom attributes (typically set by the .display configuration file
+    std::map<std::string const, std::optional<std::string>>& custom_attribute;
 
-    UserDisplayConfigurationOutput(DisplayConfigurationOutput& master);
+    UserDisplayConfigurationOutput(DisplayConfigurationOutput& main);
     geometry::Rectangle extents() const;
 };
 
@@ -166,13 +220,14 @@ bool operator!=(DisplayConfigurationOutput const& val1, DisplayConfigurationOutp
 class DisplayConfiguration
 {
 public:
-    virtual ~DisplayConfiguration() {}
+    virtual ~DisplayConfiguration() = default;
 
-    /** Executes a function object for each card in the configuration. */
-    virtual void for_each_card(std::function<void(DisplayConfigurationCard const&)> f) const = 0;
     /** Executes a function object for each output in the configuration. */
     virtual void for_each_output(std::function<void(DisplayConfigurationOutput const&)> f) const = 0;
     virtual void for_each_output(std::function<void(UserDisplayConfigurationOutput&)> f) = 0;
+
+    virtual std::unique_ptr<DisplayConfiguration> clone() const = 0;
+
     virtual bool valid() const;
 
 protected:
@@ -181,6 +236,9 @@ protected:
     DisplayConfiguration& operator=(DisplayConfiguration const& c) = delete;
 };
 
+bool operator==(DisplayConfiguration const& lhs, DisplayConfiguration const& rhs);
+bool operator!=(DisplayConfiguration const& lhs, DisplayConfiguration const& rhs);
+
 std::ostream& operator<<(std::ostream& out, DisplayConfiguration const& val);
 
 }
