@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,15 +12,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 
 #ifndef MIR_OPTIONS_PROGRAM_OPTION_H_
 #define MIR_OPTIONS_PROGRAM_OPTION_H_
 
-#include "option.h"
+#include "mir/options/option.h"
 
 #include <boost/program_options/variables_map.hpp>
 #include <boost/program_options/options_description.hpp>
@@ -54,8 +52,11 @@ public:
     boost::any const& get(char const* name) const override;
     using Option::get;
 
+    std::vector<std::string> unparsed_command_line() const;
+
 private:
     boost::program_options::variables_map options;
+    std::vector<std::string> unparsed_tokens;
 };
 }
 }
