@@ -55,39 +55,3 @@ void MirWindowEvent::set_value(int value)
 {
     value_ = value;
 }
-
-void MirWindowEvent::set_dnd_handle(std::vector<uint8_t> const& handle)
-{
-    dnd_handle_ = handle;
-}
-
-namespace
-{
-struct MyMirBlob : MirBlob
-{
-
-    size_t size() const override { return data_.size(); }
-    virtual void const* data() const override { return data_.data(); }
-
-    std::vector<uint8_t> data_;
-};
-}
-
-MirBlob* MirWindowEvent::dnd_handle() const
-{
-    if (!dnd_handle_)
-        return nullptr;
-
-    auto blob = std::make_unique<MyMirBlob>();
-
-    auto reader = *dnd_handle_;
-
-    blob->data_.reserve(reader.size());
-
-    // Can't use std::copy() as the CapnP iterators don't provide an iterator category
-    for (auto p = reader.begin(); p != reader.end(); ++p)
-        blob->data_.push_back(*p);
-
-    return blob.release();
-}
-
