@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Kevin DuBois <kevin.dubois@canonical.com>
  */
 
 #ifndef MIR_FD_SOCKET_TRANSMISSION_H_
@@ -37,7 +35,7 @@ struct socket_disconnected_error : std::system_error
 
 struct fd_reception_error : std::runtime_error
 {
-    fd_reception_error();
+    fd_reception_error(std::string const& message);
 };
 
 bool socket_error_is_transient(int error_code);
