@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Christopher James Halse Rogers <christopher.halse.rogers@canonical.com>
  */
 
 #ifndef MIR_SHARED_LIBRARY_PROBER_H_
@@ -30,6 +28,14 @@ namespace mir
 class SharedLibrary;
 
 std::vector<std::shared_ptr<SharedLibrary>> libraries_for_path(std::string const& path, SharedLibraryProberReport& report);
+
+// The selector can tell select_libraries_for_path() to persist or quit after each library
+enum class Selection { persist, quit };
+
+void select_libraries_for_path(
+    std::string const& path,
+    std::function<Selection(std::shared_ptr<SharedLibrary> const&)> const& selector,
+    SharedLibraryProberReport& report);
 }
 
 
