@@ -57,10 +57,12 @@ class CompositorReport;
 namespace frontend
 {
 class Connector;
-class SessionAuthorizer;
-class EventSink;
 class DisplayChanger;
+class DragIconController;
+class EventSink;
 class InputConfigurationChanger;
+class PointerInputDispatcher;
+class SessionAuthorizer;
 class SurfaceStack;
 }
 
@@ -172,6 +174,7 @@ public:
     std::shared_ptr<MainLoop>               the_main_loop() override;
     std::shared_ptr<ServerStatusListener>   the_server_status_listener() override;
     std::shared_ptr<DisplayChanger>         the_display_changer() override;
+    std::shared_ptr<ConsoleServices>        the_console_services() override;
     std::vector<std::shared_ptr<graphics::DisplayPlatform>> const& the_display_platforms() override;
     auto the_rendering_platforms() -> std::vector<std::shared_ptr<graphics::RenderingPlatform>> const& override;
     std::shared_ptr<input::InputDispatcher> the_input_dispatcher() override;
@@ -251,6 +254,8 @@ public:
      *  @{ */
     virtual std::shared_ptr<frontend::SessionAuthorizer>      the_session_authorizer();
     virtual std::shared_ptr<frontend::DisplayChanger>         the_frontend_display_changer();
+    virtual std::shared_ptr<frontend::DragIconController>     the_drag_icon_controller();
+    virtual std::shared_ptr<frontend::PointerInputDispatcher> the_pointer_input_dispatcher();
     /** @name frontend configuration - internal dependencies
      * internal dependencies of frontend
      *  @{ */
@@ -335,7 +340,6 @@ public:
     virtual std::shared_ptr<ServerActionQueue> the_server_action_queue();
     virtual std::shared_ptr<SharedLibraryProberReport>  the_shared_library_prober_report();
 
-    virtual std::shared_ptr<ConsoleServices> the_console_services();
     auto default_reports() -> std::shared_ptr<void>;
 
 protected:
@@ -363,6 +367,7 @@ protected:
     CachedPtr<frontend::Connector>   connector;
     CachedPtr<frontend::Connector>   wayland_connector;
     CachedPtr<frontend::Connector>   xwayland_connector;
+    CachedPtr<frontend::DragIconController> drag_icon_controller;
 
     CachedPtr<input::InputReport> input_report;
     CachedPtr<input::EventFilterChainDispatcher> event_filter_chain_dispatcher;
