@@ -40,9 +40,6 @@ public:
     virtual void set_focus(std::shared_ptr<input::Surface> const& focus_surface) = 0;
     virtual void clear_focus() = 0;
 
-    virtual void set_drag_and_drop_handle(std::vector<uint8_t> const& handle) = 0;
-    virtual void clear_drag_and_drop_handle() = 0;
-
 protected:
     InputTargeter() = default;
     InputTargeter(InputTargeter const&) = delete;
