@@ -108,9 +108,6 @@ miral::BasicWindowManager::BasicWindowManager(
 miral::BasicWindowManager::~BasicWindowManager()
 {
     display_config_monitor->delete_listener(this);
-
-    if (last_input_event)
-        mir_event_unref(last_input_event);
 }
 void miral::BasicWindowManager::add_session(std::shared_ptr<scene::Session> const& session)
 {
@@ -391,40 +388,26 @@ void miral::BasicWindowManager::handle_raise_surface(
         policy->handle_raise_window(info_for(surface));
 }
 
-void miral::BasicWindowManager::handle_request_drag_and_drop(
-    std::shared_ptr<mir::scene::Session> const& /*session*/,
-    std::shared_ptr<mir::scene::Surface> const& surface,
-    uint64_t timestamp)
-{
-    Locker lock{this};
-
-    if (!surface_known(surface, "drag-and-drop"))
-        return;
-
-    if (timestamp >= last_input_event_timestamp)
-        policy->handle_request_drag_and_drop(info_for(surface));
-}
-
 void miral::BasicWindowManager::handle_request_move(
     std::shared_ptr<mir::scene::Session> const& /*session*/,
     std::shared_ptr<mir::scene::Surface> const& surface,
-    uint64_t timestamp)
+    MirInputEvent const* event)
 {
     std::lock_guard lock(mutex);
 
     if (!surface_known(surface, "move"))
         return;
 
-    if (timestamp >= last_input_event_timestamp && last_input_event)
+    if (mir_input_event_get_event_time(event) >= static_cast<int64_t>(last_input_event_timestamp))
     {
-        policy->handle_request_move(info_for(surface), mir_event_get_input_event(last_input_event));
+        policy->handle_request_move(info_for(surface), event);
     }
 }
 
 void miral::BasicWindowManager::handle_request_resize(
     std::shared_ptr<mir::scene::Session> const& /*session*/,
     std::shared_ptr<mir::scene::Surface> const& surface,
-    uint64_t timestamp,
+    MirInputEvent const* event,
     MirResizeEdge edge)
 {
     std::lock_guard lock(mutex);
@@ -432,9 +415,9 @@ void miral::BasicWindowManager::handle_request_resize(
     if (!surface_known(surface, "resize"))
         return;
 
-    if (timestamp >= last_input_event_timestamp && last_input_event)
+    if (mir_input_event_get_event_time(event) >= static_cast<int64_t>(last_input_event_timestamp))
     {
-        policy->handle_request_resize(info_for(surface), mir_event_get_input_event(last_input_event), edge);
+        policy->handle_request_resize(info_for(surface), event, edge);
     }
 }
 
@@ -941,17 +924,6 @@ void miral::BasicWindowManager::raise_tree(Window const& root)
     focus_controller->raise({begin(windows), end(windows)});
 }
 
-void miral::BasicWindowManager::start_drag_and_drop(WindowInfo& window_info, std::vector<uint8_t> const& handle)
-{
-    std::shared_ptr<scene::Surface>(window_info.window())->start_drag_and_drop(handle);
-    focus_controller->set_drag_and_drop_handle(handle);
-}
-
-void miral::BasicWindowManager::end_drag_and_drop()
-{
-    focus_controller->clear_drag_and_drop_handle();
-}
-
 void miral::BasicWindowManager::move_tree(miral::WindowInfo& root, mir::geometry::Displacement movement)
 {
     if (movement == mir::geometry::Displacement{})
@@ -1011,6 +983,7 @@ void miral::BasicWindowManager::modify_window(WindowInfo& window_info, WindowSpe
     COPY_IF_SET(exclusive_rect);
     COPY_IF_SET(application_id);
     COPY_IF_SET(focus_mode);
+    COPY_IF_SET(visible_on_lock_screen);
 
 #undef COPY_IF_SET
 
@@ -1024,6 +997,15 @@ void miral::BasicWindowManager::modify_window(WindowInfo& window_info, WindowSpe
     {
         if (auto const& parent_window = modifications.parent().value().lock())
         {
+            Window ancestor{info_for(parent_window).window()};
+            while (ancestor)
+            {
+                if (window_info.window() == ancestor)
+                {
+                    BOOST_THROW_EXCEPTION(std::runtime_error("cyclic parent-child relationship detected"));
+                }
+                ancestor = info_for(ancestor).parent();
+            }
             window_info_tmp.parent(info_for(parent_window).window());
         }
         else
@@ -1597,10 +1579,6 @@ void miral::BasicWindowManager::update_event_timestamp(MirTouchEvent const* tev)
 void miral::BasicWindowManager::update_event_timestamp(MirInputEvent const* iev)
 {
     last_input_event_timestamp = mir_input_event_get_event_time(iev);
-
-    if (last_input_event)
-        mir_event_unref(last_input_event);
-    last_input_event = mir_event_ref(mir_input_event_get_event(iev));
 }
 
 void miral::BasicWindowManager::invoke_under_lock(std::function<void()> const& callback)
