@@ -585,24 +585,6 @@ try {
 }
 MIRAL_TRACE_EXCEPTION
 
-void miral::WindowManagementTrace::start_drag_and_drop(miral::WindowInfo& window_info, std::vector<uint8_t> const& handle)
-try {
-    log_input();
-    mir::log_info("%s window_info=%s", __func__, dump_of(window_info).c_str());
-    trace_count++;
-    wrapped.start_drag_and_drop(window_info, handle);
-}
-MIRAL_TRACE_EXCEPTION
-
-void miral::WindowManagementTrace::end_drag_and_drop()
-try {
-    log_input();
-    mir::log_info("%s", __func__);
-    trace_count++;
-    wrapped.end_drag_and_drop();
-}
-MIRAL_TRACE_EXCEPTION
-
 void miral::WindowManagementTrace::modify_window(
     miral::WindowInfo& window_info, miral::WindowSpecification const& modifications)
 try {
@@ -835,12 +817,15 @@ try {
 }
 MIRAL_TRACE_EXCEPTION
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 void miral::WindowManagementTrace::handle_request_drag_and_drop(miral::WindowInfo& window_info)
 try {
     mir::log_info("%s window_info=%s", __func__, dump_of(window_info).c_str());
     policy->handle_request_drag_and_drop(window_info);
 }
 MIRAL_TRACE_EXCEPTION
+#pragma GCC diagnostic pop
 
 void miral::WindowManagementTrace::handle_request_move(miral::WindowInfo& window_info, MirInputEvent const* input_event)
 try {
