@@ -173,14 +173,12 @@ void log_drm_details(std::vector<std::shared_ptr<mgg::helpers::DRMHelper>> const
 
 mgg::Display::Display(std::vector<std::shared_ptr<helpers::DRMHelper>> const& drm,
                       std::shared_ptr<helpers::GBMHelper> const& gbm,
-                      std::shared_ptr<ConsoleServices> const& vt,
                       mgg::BypassOption bypass_option,
                       std::shared_ptr<DisplayConfigurationPolicy> const& initial_conf_policy,
                       std::shared_ptr<GLConfig> const& gl_config,
                       std::shared_ptr<DisplayReport> const& listener)
     : drm{drm},
       gbm(gbm),
-      vt(vt),
       listener(listener),
       monitor(mir::udev::Context()),
       shared_egl{*gl_config},
@@ -242,9 +240,8 @@ std::unique_ptr<mg::DisplayConfiguration> mgg::Display::configuration() const
         current_display_configuration.update();
         dirty_configuration = false;
     }
-    return std::unique_ptr<mg::DisplayConfiguration>(
-        new mgg::RealKMSDisplayConfiguration(current_display_configuration)
-        );
+
+    return std::make_unique<mgg::RealKMSDisplayConfiguration>(current_display_configuration);
 }
 
 void mgg::Display::configure(mg::DisplayConfiguration const& conf)
@@ -282,14 +279,6 @@ void mgg::Display::register_configuration_change_handler(
             }));
 }
 
-void mgg::Display::register_pause_resume_handlers(
-    EventHandlerRegister& handlers,
-    DisplayPauseHandler const& pause_handler,
-    DisplayResumeHandler const& resume_handler)
-{
-    vt->register_switch_handlers(handlers, pause_handler, resume_handler);
-}
-
 void mgg::Display::pause()
 {
     if (auto c = cursor.lock()) c->suspend();
@@ -403,13 +392,6 @@ bool mgg::Display::apply_if_configuration_preserves_display_buffers(
     return result;
 }
 
-mg::Frame mgg::Display::last_frame_on(unsigned output_id) const
-{
-    auto output = current_display_configuration.get_output_for(
-        DisplayConfigurationOutputId{static_cast<int>(output_id)});
-    return output->last_frame();
-}
-
 namespace
 {
 /*
