@@ -247,13 +247,7 @@ void mgg::RealKMSOutput::wait_for_page_flip()
         fatal_error("Output %s has no associated CRTC to wait on",
                    mgk::connector_name(connector).c_str());
     }
-
-    last_frame_.store(page_flipper->wait_for_flip(current_crtc->crtc_id));
-}
-
-mg::Frame mgg::RealKMSOutput::last_frame() const
-{
-    return last_frame_.load();
+    page_flipper->wait_for_flip(current_crtc->crtc_id);
 }
 
 bool mgg::RealKMSOutput::set_cursor(gbm_bo* buffer)
