@@ -129,13 +129,6 @@ void mgw::Display::register_configuration_change_handler(
     config_change_handlers.push_back(conf_change_handler);
 }
 
-void mgw::Display::register_pause_resume_handlers(
-    EventHandlerRegister& /*handlers*/,
-    DisplayPauseHandler const& /*pause_handler*/,
-    DisplayResumeHandler const& /*resume_handler*/)
-{
-}
-
 void mgw::Display::pause()
 {
    BOOST_THROW_EXCEPTION(std::runtime_error("'Display::pause()' not yet supported on wayland platform"));
@@ -157,12 +150,6 @@ bool mgw::Display::apply_if_configuration_preserves_display_buffers(DisplayConfi
     return false;
 }
 
-auto mgw::Display::last_frame_on(unsigned) const -> Frame
-{
-    fatal_error(__PRETTY_FUNCTION__);
-    return {};
-}
-
 auto mgw::Display::create_gl_context() const -> std::unique_ptr<mrg::Context>
 {
     EGLint const static context_attr[] = {
@@ -270,6 +257,26 @@ try
             }
         }
 
+        if (wl_display_flush(display) < 0)
+        {
+            // wl_display_flush may fail with errno == EAGAIN if the write buffer is full
+            if (errno == EAGAIN)
+            {
+                // In this case, we should also wake if the display_fd is writable, so we can
+                // finish flushing.
+                fds[display_fd].events |= POLLOUT;
+            }
+            else
+            {
+                BOOST_THROW_EXCEPTION((std::system_error{errno, std::system_category(), "Failed to flush Wayland events"}));
+            }
+        }
+        else
+        {
+            // We've fully flushed our pending events to the server; we don't need to wake-on-writable
+            fds[display_fd].events &= ~POLLOUT;
+        }
+
         if (poll(fds, indices, -1) == -1)
         {
             wl_display_cancel_read(display);
@@ -302,7 +309,6 @@ try
                 lock.lock();
             }
             lock.unlock();
-            wl_display_flush(display);
         }
     }
 }
