@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,17 +12,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by:
- *  Alan Griffiths <alan@octopull.co.uk>
- *  Thomas Voss <thomas.voss@canonical.com>
  */
 
 #ifndef MIR_COMPOSITOR_BUFFER_STREAM_FACTORY_H_
 #define MIR_COMPOSITOR_BUFFER_STREAM_FACTORY_H_
 
 #include "mir/scene/buffer_stream_factory.h"
-#include "mir/compositor/frame_dropping_policy_factory.h"
 
 #include <memory>
 
@@ -38,17 +33,15 @@ namespace compositor
 class BufferStreamFactory : public scene::BufferStreamFactory
 {
 public:
-    BufferStreamFactory(std::shared_ptr<graphics::GraphicBufferAllocator> const& gralloc,
-                        std::shared_ptr<FrameDroppingPolicyFactory> const& policy_factory);
+    BufferStreamFactory();
 
     virtual ~BufferStreamFactory() {}
 
+    virtual std::shared_ptr<compositor::BufferStream> create_buffer_stream(
+        int nbuffers,
+        graphics::BufferProperties const& buffer_properties) override;
     virtual std::shared_ptr<BufferStream> create_buffer_stream(
-        graphics::BufferProperties const& buffer_properties);
-
-private:
-    std::shared_ptr<graphics::GraphicBufferAllocator> gralloc;
-    std::shared_ptr<FrameDroppingPolicyFactory> const policy_factory;
+        graphics::BufferProperties const&) override;
 };
 
 }
