@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
 #ifndef MIR_DEFAULT_EMERGENCY_CLEANUP_H_
@@ -32,12 +30,13 @@ class DefaultEmergencyCleanup : public EmergencyCleanup
 {
 public:
     void add(EmergencyCleanupHandler const& handler) override;
+    void add(ModuleEmergencyCleanupHandler handler) override;
     void operator()() const override;
 
 private:
     struct ListItem
     {
-        EmergencyCleanupHandler handler;
+        std::shared_ptr<EmergencyCleanupHandler> handler;
         std::unique_ptr<ListItem> next;
     };
 
