@@ -191,6 +191,11 @@ private:
     void handle_close_request() override;
     void surface_destroyed() override;
 
+    void destroy_role() const override
+    {
+        wl_resource_destroy(resource);
+    }
+
     DoubleBuffered<uint32_t> exclusive_zone{0};
     DoubleBuffered<Anchors> anchors;
     DoubleBuffered<Margin> margin;
@@ -665,15 +670,13 @@ void mf::LayerSurfaceV1::handle_close_request()
 
 void mf::LayerSurfaceV1::surface_destroyed()
 {
-    // Squeekboard (and possibly other purism apps) violate the protocol by destroying the surface before the role.
-    // Until it gets fixed we ignore this error for layer shell specifically.
-    // See: https://gitlab.gnome.org/World/Phosh/squeekboard/-/issues/285
-    try
+    if (!Resource::client->is_being_destroyed())
     {
-        WindowWlSurfaceRole::surface_destroyed();
-    }
-    catch (std::exception const& err)
-    {
-        log_warning("Ignoring layer shell protocol violation: %s", err.what());
+        // Squeekboard (and possibly other purism apps) violate the protocol by destroying the surface before the role.
+        // Until it gets fixed we ignore this error for layer shell specifically.
+        // See: https://gitlab.gnome.org/World/Phosh/squeekboard/-/issues/285
+        log_warning(
+            "Ignoring layer shell protocol violation: wl_surface destroyed before associated zwlr_layer_surface_v1@%d",
+            wl_resource_get_id(resource));
     }
 }
