@@ -40,7 +40,7 @@ namespace mir
 namespace frontend
 {
 
-class XdgSurfaceStable : mw::XdgSurface
+class XdgSurfaceStable : public mw::XdgSurface
 {
 public:
     static XdgSurfaceStable* from(wl_resource* surface);
@@ -67,7 +67,7 @@ private:
     void set_window_role(WindowWlSurfaceRole* role);
 
     mw::Weak<WindowWlSurfaceRole> window_role_;
-    WlSurface* const surface;
+    mw::Weak<WlSurface> const surface;
 
 public:
     XdgShellStable const& xdg_shell;
@@ -104,8 +104,9 @@ public:
 private:
     static XdgToplevelStable* from(wl_resource* surface);
     void send_toplevel_configure();
+    void destroy_role() const override;
 
-    XdgSurfaceStable* const xdg_surface;
+    mw::Weak<XdgSurfaceStable> const xdg_surface;
 };
 
 class XdgPositionerStable : public mw::XdgPositioner, public shell::SurfaceSpecification
@@ -113,6 +114,10 @@ class XdgPositionerStable : public mw::XdgPositioner, public shell::SurfaceSpeci
 public:
     XdgPositionerStable(wl_resource* new_resource);
 
+    void ensure_complete() const;
+
+    bool reactive{false};
+
 private:
     void set_size(int32_t width, int32_t height) override;
     void set_anchor_rect(int32_t x, int32_t y, int32_t width, int32_t height) override;
@@ -120,6 +125,9 @@ private:
     void set_gravity(uint32_t gravity) override;
     void set_constraint_adjustment(uint32_t constraint_adjustment) override;
     void set_offset(int32_t x, int32_t y) override;
+    void set_reactive() override;
+    void set_parent_size(int32_t parent_width, int32_t parent_height) override;
+    void set_parent_configure(uint32_t serial) override;
 };
 
 }
@@ -147,7 +155,7 @@ mf::XdgShellStable::XdgShellStable(
     std::shared_ptr<msh::Shell> shell,
     WlSeat& seat,
     OutputManager* output_manager)
-    : Global(display, Version<1>()),
+    : Global(display, Version<5>()),
       wayland_executor{wayland_executor},
       shell{shell},
       seat{seat},
@@ -161,7 +169,7 @@ void mf::XdgShellStable::bind(wl_resource* new_resource)
 }
 
 mf::XdgShellStable::Instance::Instance(wl_resource* new_resource, mf::XdgShellStable* shell)
-    : XdgWmBase{new_resource, Version<1>()},
+    : XdgWmBase{new_resource, Version<5>()},
       shell{shell}
 {
 }
@@ -191,7 +199,7 @@ mf::XdgSurfaceStable* mf::XdgSurfaceStable::from(wl_resource* surface)
 }
 
 mf::XdgSurfaceStable::XdgSurfaceStable(wl_resource* new_resource, WlSurface* surface, XdgShellStable const& xdg_shell)
-    : mw::XdgSurface(new_resource, Version<1>()),
+    : mw::XdgSurface(new_resource, Version<5>()),
       surface{surface},
       xdg_shell{xdg_shell}
 {
@@ -199,7 +207,14 @@ mf::XdgSurfaceStable::XdgSurfaceStable(wl_resource* new_resource, WlSurface* sur
 
 void mf::XdgSurfaceStable::get_toplevel(wl_resource* new_toplevel)
 {
-    auto toplevel = new XdgToplevelStable{new_toplevel, this, surface};
+    if (!surface)
+    {
+        BOOST_THROW_EXCEPTION(mw::ProtocolError(
+            resource,
+            mw::generic_error_code,
+            "Tried to create toplevel after destroying surface"));
+    }
+    auto toplevel = new XdgToplevelStable{new_toplevel, this, &surface.value()};
     set_window_role(toplevel);
 }
 
@@ -223,7 +238,19 @@ void mf::XdgSurfaceStable::get_popup(
         }
     }
 
-    auto popup = new XdgPopupStable{new_popup, this, parent_role, positioner, surface};
+    auto xdg_positioner = static_cast<XdgPositionerStable*>(
+        static_cast<mw::XdgPositioner*>(
+            wl_resource_get_user_data(positioner)));
+
+    if (!surface)
+    {
+        BOOST_THROW_EXCEPTION(mw::ProtocolError(
+            resource,
+            mw::generic_error_code,
+            "Tried to create popup after destroying surface"));
+    }
+
+    auto popup = new XdgPopupStable{new_popup, this, parent_role, *xdg_positioner, &surface.value()};
     set_window_role(popup);
 }
 
@@ -277,9 +304,9 @@ mf::XdgPopupStable::XdgPopupStable(
     wl_resource* new_resource,
     XdgSurfaceStable* xdg_surface,
     std::optional<WlSurfaceRole*> parent_role,
-    struct wl_resource* positioner,
+    XdgPositionerStable& positioner,
     WlSurface* surface)
-    : mw::XdgPopup(new_resource, Version<1>()),
+    : mw::XdgPopup(new_resource, Version<5>()),
       WindowWlSurfaceRole(
           xdg_surface->xdg_shell.wayland_executor,
           &xdg_surface->xdg_shell.seat,
@@ -287,30 +314,22 @@ mf::XdgPopupStable::XdgPopupStable(
           surface,
           xdg_surface->xdg_shell.shell,
           xdg_surface->xdg_shell.output_manager),
+      reactive{positioner.reactive},
+      aux_rect{positioner.aux_rect ? positioner.aux_rect.value() : geom::Rectangle{}},
       shell{xdg_surface->xdg_shell.shell},
       xdg_surface{xdg_surface}
 {
-    auto specification = static_cast<mir::shell::SurfaceSpecification*>(
-                                static_cast<XdgPositionerStable*>(
-                                    static_cast<mw::XdgPositioner*>(
-                                        wl_resource_get_user_data(positioner))));
-
-    if (specification->aux_rect)
-    {
-        aux_rect = specification->aux_rect.value();
-    }
-
-    specification->type = mir_window_type_gloss;
-    specification->placement_hints = mir_placement_hints_slide_any;
+    positioner.ensure_complete();
+    positioner.type = mir_window_type_gloss;
     if (parent_role)
     {
         if (auto scene_surface = parent_role.value()->scene_surface())
         {
-            specification->parent = scene_surface.value();
+            positioner.parent = scene_surface.value();
         }
     }
 
-    apply_spec(*specification);
+    apply_spec(positioner);
 }
 
 void mf::XdgPopupStable::set_aux_rect_offset_now(geom::Displacement const& new_aux_rect_offset)
@@ -346,30 +365,60 @@ void mf::XdgPopupStable::grab(struct wl_resource* seat, uint32_t serial)
     set_type(mir_window_type_menu);
 }
 
+void mf::XdgPopupStable::reposition(wl_resource* positioner_resource, uint32_t token)
+{
+    auto const& positioner = *static_cast<XdgPositionerStable*>(
+        static_cast<mw::XdgPositioner*>(
+            wl_resource_get_user_data(positioner_resource)));
+    positioner.ensure_complete();
+
+    reactive = positioner.reactive;
+    aux_rect = positioner.aux_rect ? positioner.aux_rect.value() : geom::Rectangle{};
+    reposition_token = token;
+
+    auto const scene_surface_{scene_surface()};
+    if (scene_surface_)
+    {
+        shell->modify_surface(
+            scene_surface_.value()->session().lock(),
+            scene_surface_.value(),
+            positioner);
+    }
+    else
+    {
+        apply_spec(positioner);
+    }
+}
+
 void mf::XdgPopupStable::handle_resize(
-    std::optional<geometry::Point> const& new_top_left_,
+    std::optional<geometry::Point> const& new_top_left,
     geometry::Size const& new_size)
 {
-    auto new_top_left{new_top_left_};
+    auto const prev_rect = popup_rect();
     if (new_top_left)
     {
-        new_top_left.value() -= aux_rect_offset;
+        cached_top_left = new_top_left.value() - aux_rect_offset;
     }
-
-    bool const needs_configure = (new_top_left != cached_top_left) || (new_size != cached_size);
-
-    if (new_top_left)
-        cached_top_left = new_top_left;
-
     cached_size = new_size;
+    auto const new_rect = popup_rect();
 
-    if (needs_configure && cached_top_left && cached_size)
+    bool const configure_appropriate =
+        reposition_token ||
+        initial_configure_pending ||
+        (reactive && new_rect != prev_rect);
+    if (new_rect && configure_appropriate)
     {
+        if (reposition_token)
+        {
+            send_repositioned_event(reposition_token.value());
+            reposition_token = std::nullopt;
+        }
         send_configure_event(cached_top_left.value().x.as_int(),
                              cached_top_left.value().y.as_int(),
                              cached_size.value().width.as_int(),
                              cached_size.value().height.as_int());
-        xdg_surface->send_configure();
+        if (xdg_surface) xdg_surface.value().send_configure();
+        initial_configure_pending = false;
     }
 }
 
@@ -386,10 +435,27 @@ auto mf::XdgPopupStable::from(wl_resource* resource) -> XdgPopupStable*
     return popup;
 }
 
+auto mf::XdgPopupStable::popup_rect() const -> std::optional<geometry::Rectangle>
+{
+    if (cached_top_left && cached_size)
+    {
+        return geometry::Rectangle{cached_top_left.value(), cached_size.value()};
+    }
+    else
+    {
+        return std::nullopt;
+    }
+}
+
+void mf::XdgPopupStable::destroy_role() const
+{
+    wl_resource_destroy(resource);
+}
+
 // XdgToplevelStable
 
 mf::XdgToplevelStable::XdgToplevelStable(wl_resource* new_resource, XdgSurfaceStable* xdg_surface, WlSurface* surface)
-    : mw::XdgToplevel(new_resource, Version<1>()),
+    : mw::XdgToplevel(new_resource, Version<5>()),
       WindowWlSurfaceRole(
           xdg_surface->xdg_shell.wayland_executor,
           &xdg_surface->xdg_shell.seat,
@@ -399,6 +465,26 @@ mf::XdgToplevelStable::XdgToplevelStable(wl_resource* new_resource, XdgSurfaceSt
           xdg_surface->xdg_shell.output_manager),
       xdg_surface{xdg_surface}
 {
+    if (version_supports_wm_capabilities())
+    {
+        std::vector<uint32_t> capabilities{
+            WmCapabilities::maximize,
+            WmCapabilities::minimize,
+            WmCapabilities::fullscreen,
+        };
+        wl_array capability_array;
+        wl_array_init(&capability_array);
+        for (auto& capability : capabilities)
+        {
+            if (uint32_t *item = static_cast<decltype(item)>(wl_array_add(&capability_array, sizeof *item)))
+            {
+                *item = capability;
+            }
+        }
+        send_wm_capabilities_event(&capability_array);
+        wl_array_release(&capability_array);
+    }
+
     wl_array states;
     wl_array_init(&states);
     send_configure_event(0, 0, &states);
@@ -477,7 +563,15 @@ void mf::XdgToplevelStable::resize(struct wl_resource* /*seat*/, uint32_t serial
         edge = mir_resize_edge_southeast;
         break;
 
-    default:;
+    case ResizeEdge::none:
+        edge = mir_resize_edge_none;
+        break;
+
+    default:
+        BOOST_THROW_EXCEPTION(mw::ProtocolError(
+            resource,
+            Error::invalid_resize_edge,
+            "Invalid resize edge %d", edges));
     }
 
     initiate_interactive_resize(edge, serial);
@@ -583,7 +677,7 @@ void mf::XdgToplevelStable::send_toplevel_configure()
     send_configure_event(size.width.as_int(), size.height.as_int(), &states);
     wl_array_release(&states);
 
-    xdg_surface->send_configure();
+    if (xdg_surface) xdg_surface.value().send_configure();
 }
 
 mf::XdgToplevelStable* mf::XdgToplevelStable::from(wl_resource* surface)
@@ -592,24 +686,55 @@ mf::XdgToplevelStable* mf::XdgToplevelStable::from(wl_resource* surface)
     return static_cast<XdgToplevelStable*>(static_cast<mw::XdgToplevel*>(tmp));
 }
 
+void mf::XdgToplevelStable::destroy_role() const
+{
+    wl_resource_destroy(resource);
+}
+
 // XdgPositionerStable
 
 mf::XdgPositionerStable::XdgPositionerStable(wl_resource* new_resource)
-    : mw::XdgPositioner(new_resource, Version<1>())
+    : mw::XdgPositioner(new_resource, Version<5>())
 {
     // specifying gravity is not required by the xdg shell protocol, but is by Mir window managers
     surface_placement_gravity = mir_placement_gravity_center;
     aux_rect_placement_gravity = mir_placement_gravity_center;
+    placement_hints = static_cast<MirPlacementHints>(0);
+}
+
+void mf::XdgPositionerStable::ensure_complete() const
+{
+    if (!width || !height || !aux_rect)
+    {
+        BOOST_THROW_EXCEPTION(mw::ProtocolError(
+            resource,
+            mw::XdgWmBase::Error::invalid_positioner,
+            "Incomplete positioner"));
+    }
 }
 
 void mf::XdgPositionerStable::set_size(int32_t width, int32_t height)
 {
+    if (width <= 0 || height <= 0)
+    {
+        BOOST_THROW_EXCEPTION(mw::ProtocolError(
+            resource,
+            mw::XdgPositioner::Error::invalid_input,
+            "Invalid popup positioner size: %dx%d", width, height));
+    }
     this->width = geom::Width{width};
     this->height = geom::Height{height};
 }
 
 void mf::XdgPositionerStable::set_anchor_rect(int32_t x, int32_t y, int32_t width, int32_t height)
 {
+    if (width < 0 || height < 0)
+    {
+        BOOST_THROW_EXCEPTION(mw::ProtocolError(
+            resource,
+            mw::XdgPositioner::Error::invalid_input,
+            "Invalid popup anchor rect size: %dx%d", width, height));
+    }
     aux_rect = geom::Rectangle{{x, y}, {width, height}};
 }
 
@@ -696,8 +821,15 @@ void mf::XdgPositionerStable::set_gravity(uint32_t gravity)
             placement = mir_placement_gravity_northwest;
             break;
 
-        default:
+        case Gravity::none:
             placement = mir_placement_gravity_center;
+            break;
+
+        default:
+            BOOST_THROW_EXCEPTION(mw::ProtocolError(
+                resource,
+                mw::XdgPositioner::Error::invalid_input,
+                "Invalid gravity value %d", gravity));
     }
 
     surface_placement_gravity = placement;
@@ -705,8 +837,32 @@ void mf::XdgPositionerStable::set_gravity(uint32_t gravity)
 
 void mf::XdgPositionerStable::set_constraint_adjustment(uint32_t constraint_adjustment)
 {
-    (void)constraint_adjustment;
-    // TODO
+    uint32_t new_placement_hints{0};
+    if (constraint_adjustment & ConstraintAdjustment::slide_x)
+    {
+        new_placement_hints |= mir_placement_hints_slide_x;
+    }
+    if (constraint_adjustment & ConstraintAdjustment::slide_y)
+    {
+        new_placement_hints |= mir_placement_hints_slide_y;
+    }
+    if (constraint_adjustment & ConstraintAdjustment::flip_x)
+    {
+        new_placement_hints |= mir_placement_hints_flip_x;
+    }
+    if (constraint_adjustment & ConstraintAdjustment::flip_x)
+    {
+        new_placement_hints |= mir_placement_hints_flip_y;
+    }
+    if (constraint_adjustment & ConstraintAdjustment::resize_x)
+    {
+        new_placement_hints |= mir_placement_hints_resize_x;
+    }
+    if (constraint_adjustment & ConstraintAdjustment::resize_y)
+    {
+        new_placement_hints |= mir_placement_hints_resize_y;
+    }
+    placement_hints = static_cast<MirPlacementHints>(new_placement_hints);
 }
 
 void mf::XdgPositionerStable::set_offset(int32_t x, int32_t y)
@@ -715,6 +871,26 @@ void mf::XdgPositionerStable::set_offset(int32_t x, int32_t y)
     aux_rect_placement_offset_y = y;
 }
 
+void mf::XdgPositionerStable::set_reactive()
+{
+    reactive = true;
+}
+
+void mf::XdgPositionerStable::set_parent_size(int32_t parent_width, int32_t parent_height)
+{
+    (void)parent_width;
+    (void)parent_height;
+    // TODO
+    log_warning("xdg_positioner.set_parent_size not implemented");
+}
+
+void mf::XdgPositionerStable::set_parent_configure(uint32_t serial)
+{
+    (void)serial;
+    // TODO
+    log_warning("xdg_positioner.set_parent_configure not implemented");
+}
+
 auto mf::XdgShellStable::get_window(wl_resource* surface) -> std::shared_ptr<scene::Surface>
 {
     namespace mw = mir::wayland;
