@@ -38,7 +38,7 @@ namespace mir
 namespace frontend
 {
 
-class XdgSurfaceV6 : mw::XdgSurfaceV6
+class XdgSurfaceV6 : public mw::XdgSurfaceV6
 {
 public:
     static XdgSurfaceV6* from(wl_resource* surface);
@@ -62,7 +62,7 @@ private:
     void set_window_role(WindowWlSurfaceRole* role);
 
     mw::Weak<WindowWlSurfaceRole> window_role_;
-    WlSurface* const surface;
+    mw::Weak<WlSurface> const surface;
 
 public:
     XdgShellV6 const& xdg_shell;
@@ -92,7 +92,9 @@ private:
     std::optional<geom::Point> cached_top_left;
     std::optional<geom::Size> cached_size;
 
-    XdgSurfaceV6* const xdg_surface;
+    mw::Weak<XdgSurfaceV6> const xdg_surface;
+
+    void destroy_role() const override;
 };
 
 class XdgToplevelV6 : mw::XdgToplevelV6, public WindowWlSurfaceRole
@@ -125,8 +127,9 @@ public:
 private:
     static XdgToplevelV6* from(wl_resource* surface);
     void send_toplevel_configure();
+    void destroy_role() const override;
 
-    XdgSurfaceV6* const xdg_surface;
+    mw::Weak<XdgSurfaceV6> const xdg_surface;
 };
 
 class XdgPositionerV6 : public mw::XdgPositionerV6, public shell::SurfaceSpecification
@@ -221,13 +224,27 @@ mf::XdgSurfaceV6::XdgSurfaceV6(wl_resource* new_resource, WlSurface* surface,
 
 void mf::XdgSurfaceV6::get_toplevel(wl_resource* new_toplevel)
 {
-    auto toplevel = new XdgToplevelV6{new_toplevel, this, surface};
+    if (!surface)
+    {
+        BOOST_THROW_EXCEPTION(mw::ProtocolError(
+            resource,
+            mw::generic_error_code,
+            "Tried to create toplevel v6 after destroying surface"));
+    }
+    auto toplevel = new XdgToplevelV6{new_toplevel, this, &surface.value()};
     set_window_role(toplevel);
 }
 
 void mf::XdgSurfaceV6::get_popup(wl_resource* new_popup, struct wl_resource* parent_surface, struct wl_resource* positioner)
 {
-    auto popup = new XdgPopupV6{new_popup, this, XdgSurfaceV6::from(parent_surface), positioner, surface};
+    if (!surface)
+    {
+        BOOST_THROW_EXCEPTION(mw::ProtocolError(
+            resource,
+            mw::generic_error_code,
+            "Tried to create popup v6 after destroying surface"));
+    }
+    auto popup = new XdgPopupV6{new_popup, this, XdgSurfaceV6::from(parent_surface), positioner, &surface.value()};
     set_window_role(popup);
 }
 
@@ -302,7 +319,6 @@ mf::XdgPopupV6::XdgPopupV6(
     auto parent_scene_surface = parent_role ? parent_role.value().scene_surface() : std::nullopt;
 
     specification->type = mir_window_type_gloss;
-    specification->placement_hints = mir_placement_hints_slide_any;
     if (parent_scene_surface)
         specification->parent = parent_scene_surface.value();
     else
@@ -334,7 +350,7 @@ void mf::XdgPopupV6::handle_resize(
                              cached_top_left.value().y.as_int(),
                              cached_size.value().width.as_int(),
                              cached_size.value().height.as_int());
-        xdg_surface->send_configure();
+        if (xdg_surface) xdg_surface.value().send_configure();
     }
 }
 
@@ -343,6 +359,11 @@ void mf::XdgPopupV6::handle_close_request()
     send_popup_done_event();
 }
 
+void mf::XdgPopupV6::destroy_role() const
+{
+    wl_resource_destroy(resource);
+}
+
 // XdgToplevelV6
 
 mf::XdgToplevelV6::XdgToplevelV6(struct wl_resource* new_resource, XdgSurfaceV6* xdg_surface, WlSurface* surface)
@@ -539,7 +560,7 @@ void mf::XdgToplevelV6::send_toplevel_configure()
     send_configure_event(size.width.as_int(), size.height.as_int(), &states);
     wl_array_release(&states);
 
-    xdg_surface->send_configure();
+    if (xdg_surface) xdg_surface.value().send_configure();
 }
 
 mf::XdgToplevelV6* mf::XdgToplevelV6::from(wl_resource* surface)
@@ -548,6 +569,11 @@ mf::XdgToplevelV6* mf::XdgToplevelV6::from(wl_resource* surface)
     return static_cast<XdgToplevelV6*>(static_cast<mw::XdgToplevelV6*>(tmp));
 }
 
+void mf::XdgToplevelV6::destroy_role() const
+{
+    wl_resource_destroy(resource);
+}
+
 // XdgPositionerV6
 
 mf::XdgPositionerV6::XdgPositionerV6(wl_resource* new_resource)
@@ -556,6 +582,7 @@ mf::XdgPositionerV6::XdgPositionerV6(wl_resource* new_resource)
     // specifying gravity is not required by the xdg shell protocol, but is by Mir window managers
     surface_placement_gravity = mir_placement_gravity_center;
     aux_rect_placement_gravity = mir_placement_gravity_center;
+    placement_hints = static_cast<MirPlacementHints>(0);
 }
 
 void mf::XdgPositionerV6::set_size(int32_t width, int32_t height)
@@ -609,8 +636,32 @@ void mf::XdgPositionerV6::set_gravity(uint32_t gravity)
 
 void mf::XdgPositionerV6::set_constraint_adjustment(uint32_t constraint_adjustment)
 {
-    (void)constraint_adjustment;
-    // TODO
+    uint32_t new_placement_hints{0};
+    if (constraint_adjustment & ConstraintAdjustment::slide_x)
+    {
+        new_placement_hints |= mir_placement_hints_slide_x;
+    }
+    if (constraint_adjustment & ConstraintAdjustment::slide_y)
+    {
+        new_placement_hints |= mir_placement_hints_slide_y;
+    }
+    if (constraint_adjustment & ConstraintAdjustment::flip_x)
+    {
+        new_placement_hints |= mir_placement_hints_flip_x;
+    }
+    if (constraint_adjustment & ConstraintAdjustment::flip_x)
+    {
+        new_placement_hints |= mir_placement_hints_flip_y;
+    }
+    if (constraint_adjustment & ConstraintAdjustment::resize_x)
+    {
+        new_placement_hints |= mir_placement_hints_resize_x;
+    }
+    if (constraint_adjustment & ConstraintAdjustment::resize_y)
+    {
+        new_placement_hints |= mir_placement_hints_resize_y;
+    }
+    placement_hints = static_cast<MirPlacementHints>(new_placement_hints);
 }
 
 void mf::XdgPositionerV6::set_offset(int32_t x, int32_t y)
