@@ -688,12 +688,18 @@ void mf::XWaylandSurface::attach_wl_surface(WlSurface* wl_surface)
 
     std::shared_ptr<XWaylandClientManager::Session> local_client_session;
     std::shared_ptr<ms::Session> session;
+    bool rejected = false;
     reply_functions.push_back(connection->read_property(
         window, connection->_NET_WM_PID,
         XCBConnection::Handler<uint32_t>{
             [&](uint32_t pid)
             {
                 local_client_session = client_manager->session_for_client(pid);
+                if (!local_client_session)
+                {
+                    rejected = true;
+                    return;
+                }
                 session = local_client_session->session();
             },
             [&](std::string const&)
@@ -709,6 +715,14 @@ void mf::XWaylandSurface::attach_wl_surface(WlSurface* wl_surface)
         reply_function();
     }
 
+    if (rejected)
+    {
+        xcb_kill_client(*connection, window);
+        close();
+        connection->flush();
+        return;
+    }
+
     if (!session)
     {
         fatal_error("Property handlers did not set a valid session");
@@ -772,11 +786,11 @@ void mf::XWaylandSurface::move_resize(uint32_t detail)
     auto const action = static_cast<NetWmMoveresize>(detail);
     if (action == NetWmMoveresize::MOVE)
     {
-        shell->request_move(scene_surface->session().lock(), scene_surface, event->event_time().count());
+        shell->request_move(scene_surface->session().lock(), scene_surface, event.get());
     }
     else if (auto const edge = wm_resize_edge_to_mir_resize_edge(action))
     {
-        shell->request_resize(scene_surface->session().lock(), scene_surface, event->event_time().count(), edge.value());
+        shell->request_resize(scene_surface->session().lock(), scene_surface, event.get(), edge.value());
     }
     else
     {
