@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014-2015 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,15 +12,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
- *              Alberto Aguirre <alberto.aguirre@canonical.com>
  */
 
 #include "mir/glib_main_loop_sources.h"
-#include "mir/recursive_read_write_mutex.h"
-#include "mir/thread_safe_list.h"
+#include "mir/lockable_callback.h"
 #include "mir/raii.h"
+#include <mir/log.h>
 
 #include <algorithm>
 #include <atomic>
@@ -28,6 +25,7 @@
 #include <sstream>
 
 #include <csignal>
+#include <cstring>
 #include <unistd.h>
 #include <fcntl.h>
 
@@ -67,7 +65,6 @@ void destroy_idler(gpointer user_data)
     g_source_unref(gsource);
 }
 #endif
-
 }
 
 /*****************
@@ -81,24 +78,24 @@ md::GSourceHandle::GSourceHandle()
 
 md::GSourceHandle::GSourceHandle(
     GSource* gsource,
-    std::function<void(GSource*)> const& pre_destruction_hook)
-    : gsource{gsource},
-      pre_destruction_hook{pre_destruction_hook}
+    std::function<void(GSource*)> const& terminate_dispatch)
+    : gsource(gsource),
+      terminate_dispatch(terminate_dispatch)
 {
 }
 
 md::GSourceHandle::GSourceHandle(GSourceHandle&& other)
-    : gsource{std::move(other.gsource)},
-      pre_destruction_hook{std::move(other.pre_destruction_hook)}
+    : gsource(std::move(other.gsource)),
+      terminate_dispatch(std::move(other.terminate_dispatch))
 {
     other.gsource = nullptr;
-    other.pre_destruction_hook = [](GSource*){};
+    other.terminate_dispatch = [](GSource*){};
 }
 
 md::GSourceHandle& md::GSourceHandle::operator=(GSourceHandle other)
 {
     std::swap(other.gsource, gsource);
-    std::swap(other.pre_destruction_hook, pre_destruction_hook);
+    std::swap(other.terminate_dispatch, terminate_dispatch);
     return *this;
 }
 
@@ -106,10 +103,8 @@ md::GSourceHandle::~GSourceHandle()
 {
     if (gsource)
     {
-        pre_destruction_hook(gsource);
-        g_source_destroy(gsource);
-
 #ifdef GLIB_HAS_FIXED_LP_1401488
+        g_source_destroy(gsource);
         g_source_unref(gsource);
 #else
         /*
@@ -119,17 +114,34 @@ md::GSourceHandle::~GSourceHandle()
          * and so the main loop might be mid-iteration of the same source
          * making callbacks. And glib lacks protection to prevent sources
          * getting fully free()'d mid-callback (TODO: fix glib?). So we defer
-         * the final unref of the source to a point in the loop when it's safe:
+         * the final unref of the source to a point in the loop when it's safe.
          */
-        auto main_context = g_source_get_context(gsource);
-        auto idler = g_idle_source_new();
-        g_source_set_callback(idler, idle_callback, gsource, destroy_idler);
-        g_source_attach(idler, main_context);
-        g_source_unref(idler);
+        if (!g_source_is_destroyed(gsource))
+        {
+            auto main_context = g_source_get_context(gsource);
+            g_source_destroy(gsource);
+            auto idler = g_idle_source_new();
+            g_source_set_callback(idler, idle_callback, gsource, destroy_idler);
+            g_source_attach(idler, main_context);
+            g_source_unref(idler);
+        }
+        else
+        {
+            // The source is already destroyed so it's safe to unref now
+            g_source_unref(gsource);
+        }
 #endif
     }
 }
 
+void md::GSourceHandle::ensure_no_further_dispatch()
+{
+    if (gsource)
+    {
+        terminate_dispatch(gsource);
+    }
+}
+
 md::GSourceHandle::operator GSource*() const
 {
     return gsource;
@@ -176,6 +188,13 @@ void md::add_server_action_gsource(
         void const* const owner;
         std::function<void(void)> const action;
         std::function<bool(void const*)> const should_dispatch;
+
+        // If we come to finalize() before dispatch() we have already
+        // torn down most of Mir and even unloaded some shared libraries.
+        // That means the action could refer to  stuff that is no longer
+        // in the address space.
+        // We will just leak any resources instead of crashing.
+        bool mutable dispatched{false};
     };
 
     struct ServerActionGSource
@@ -201,13 +220,14 @@ void md::add_server_action_gsource(
         {
             auto const& ctx = reinterpret_cast<ServerActionGSource*>(source)->ctx;
             ctx.action();
+            ctx.dispatched = true;
             return FALSE;
         }
 
         static void finalize(GSource* source)
         {
             auto const sa_gsource = reinterpret_cast<ServerActionGSource*>(source);
-            if (sa_gsource->ctx_constructed)
+            if (sa_gsource->ctx_constructed && sa_gsource->ctx.dispatched)
                 sa_gsource->ctx.~ServerActionContext();
         }
     };
@@ -234,29 +254,26 @@ void md::add_server_action_gsource(
 md::GSourceHandle md::add_timer_gsource(
     GMainContext* main_context,
     std::shared_ptr<time::Clock> const& clock,
-    std::function<void()> const& handler,
-    std::function<void()> const& lock,
-    std::function<void()> const& unlock,
+    std::shared_ptr<LockableCallback> const& handler,
+    std::function<void()> const& exception_handler,
     time::Timestamp target_time)
 {
     struct TimerContext
     {
         TimerContext(std::shared_ptr<time::Clock> const& clock,
-                     std::function<void()> const& handler,
-                     std::function<void()> const& lock,
-                     std::function<void()> const& unlock,
+                     std::shared_ptr<LockableCallback> const& handler,
+                     std::function<void()> const& exception_handler,
                      time::Timestamp target_time)
-            : clock{clock}, handler{handler}, lock{lock}, unlock{unlock},
+            : clock{clock}, handler{handler}, exception_handler{exception_handler},
               target_time{target_time}, enabled{true}
         {
         }
         std::shared_ptr<time::Clock> clock;
-        std::function<void()> handler;
-        std::function<void()> lock;
-        std::function<void()> unlock;
+        std::shared_ptr<LockableCallback> handler;
+        std::function<void()> exception_handler;
         time::Timestamp target_time;
+        std::recursive_mutex mutex;
         bool enabled;
-        mir::RecursiveReadWriteMutex mutex;
     };
 
     struct TimerGSource
@@ -292,13 +309,20 @@ md::GSourceHandle md::add_timer_gsource(
         static gboolean dispatch(GSource* source, GSourceFunc, gpointer)
         {
             auto& ctx = reinterpret_cast<TimerGSource*>(source)->ctx;
-
-            // Caller may pass std::function objects to preserve locking
-            // order during callback dispatching, so aquire them first.
-            auto caller_lock = mir::raii::paired_calls(std::ref(ctx.lock), std::ref(ctx.unlock));
-            RecursiveReadLock lock{ctx.mutex};
-            if (ctx.enabled)
-                ctx.handler();
+            try
+            {
+                // Attempt to preserve locking order during callback dispatching
+                // so we acquire the caller's lock before our own.
+                auto& handler = *ctx.handler;
+                std::lock_guard handler_lock{handler};
+                std::lock_guard lock{ctx.mutex};
+                if (ctx.enabled)
+                    handler();
+            }
+            catch(...)
+            {
+                ctx.exception_handler();
+            }
 
             return FALSE;
         }
@@ -313,7 +337,7 @@ md::GSourceHandle md::add_timer_gsource(
         static void disable(GSource* source)
         {
             auto& ctx = reinterpret_cast<TimerGSource*>(source)->ctx;
-            RecursiveWriteLock lock{ctx.mutex};
+            std::lock_guard lock{ctx.mutex};
             ctx.enabled = false;
         }
     };
@@ -333,7 +357,7 @@ md::GSourceHandle md::add_timer_gsource(
     auto const timer_gsource = reinterpret_cast<TimerGSource*>(static_cast<GSource*>(gsource));
 
     timer_gsource->ctx_constructed = false;
-    new (&timer_gsource->ctx) TimerContext{clock, handler, lock, unlock, target_time};
+    new (&timer_gsource->ctx) TimerContext{clock, handler, exception_handler, target_time};
     timer_gsource->ctx_constructed = true;
 
     g_source_attach(gsource, main_context);
@@ -354,13 +378,13 @@ struct md::FdSources::FdContext
 
     void disable_callback()
     {
-        RecursiveWriteLock lock{mutex};
+        std::lock_guard lock{mutex};
         enabled = false;
     }
 
     static gboolean static_call(int fd, GIOCondition, FdContext* ctx)
     {
-        RecursiveReadLock lock{ctx->mutex};
+        std::lock_guard lock{ctx->mutex};
 
         if (ctx->enabled)
             ctx->handler(fd);
@@ -376,11 +400,16 @@ struct md::FdSources::FdContext
 private:
     std::function<void(int)> handler;
     bool enabled;
-    mir::RecursiveReadWriteMutex mutex;
+    std::recursive_mutex mutex;
 };
 
 struct md::FdSources::FdSource
 {
+    ~FdSource()
+    {
+        gsource.ensure_no_further_dispatch();
+    }
+
     GSourceHandle gsource;
     void const* const owner;
 };
@@ -408,11 +437,11 @@ void md::FdSources::add(
 
     g_source_set_callback(
         gsource,
-        reinterpret_cast<GSourceFunc>(&FdContext::static_call),
+        reinterpret_cast<GSourceFunc>(reinterpret_cast<void*>(&FdContext::static_call)),
         fd_context,
-        reinterpret_cast<GDestroyNotify>(&FdContext::static_destroy));
+        reinterpret_cast<GDestroyNotify>(reinterpret_cast<void*>(&FdContext::static_destroy)));
 
-    std::lock_guard<std::mutex> lock{sources_mutex};
+    std::lock_guard lock{sources_mutex};
 
     sources.emplace_back(new FdSource{std::move(gsource), owner});
     g_source_attach(sources.back()->gsource, main_context);
@@ -420,7 +449,7 @@ void md::FdSources::add(
 
 void md::FdSources::remove_all_owned_by(void const* owner)
 {
-    std::lock_guard<std::mutex> lock{sources_mutex};
+    std::lock_guard lock{sources_mutex};
 
     auto const new_end = std::remove_if(
         sources.begin(), sources.end(),
@@ -451,15 +480,17 @@ public:
         remove_write_fd();
     }
 
-    static void notify_sources_of_signal(int sig)
+    static void notify_sources_of_signal(int sig, siginfo_t* info, void*)
     {
+        SignalPid const sigpid{sig, info->si_pid};
+
         for (auto const& write_fd : write_fds)
         {
             // There is a benign race here: write_fd may have changed
             // between checking and using it. This doesn't matter
             // since in the worst case we will call write() with an invalid
             // fd (-1) which is harmless.
-            if (write_fd >= 0 && write(write_fd, &sig, sizeof(sig))) {}
+            if (write_fd >= 0 && write(write_fd, &sigpid, sizeof(sigpid))) {}
         }
     }
 
@@ -507,7 +538,7 @@ private:
 std::array<std::atomic<int>,10> md::SignalSources::SourceRegistration::write_fds;
 
 md::SignalSources::SignalSources(md::FdSources& fd_sources)
-    : fd_sources{fd_sources}
+    : fd_sources(fd_sources)
 {
     int pipefd[2];
 
@@ -542,13 +573,16 @@ md::SignalSources::~SignalSources()
 void md::SignalSources::dispatch_pending_signal()
 {
     auto const sig = read_pending_signal();
-    if (sig != -1)
-        dispatch_signal(sig);
+    if (sig.sig != -1)
+    {
+        mir::log_debug("Handling %s from pid=%d", strsignal(sig.sig), sig.pid);
+        dispatch_signal(sig.sig);
+    }
 }
 
-int md::SignalSources::read_pending_signal()
+auto md::SignalSources::read_pending_signal() -> SignalPid
 {
-    int sig = -1;
+    SignalPid sig{-1, 0};
     size_t total = 0;
 
     do
@@ -561,7 +595,7 @@ int md::SignalSources::read_pending_signal()
         if (nread < 0)
         {
             if (errno != EINTR)
-                return -1;
+                return SignalPid{-1, 0};
         }
         else
         {
@@ -583,18 +617,17 @@ void md::SignalSources::add(
 
 void md::SignalSources::ensure_signal_is_handled(int sig)
 {
-    std::lock_guard<std::mutex> lock{handled_signals_mutex};
+    std::lock_guard lock{handled_signals_mutex};
 
     if (handled_signals.find(sig) != handled_signals.end())
         return;
 
-    static int const no_flags{0};
     struct sigaction old_action;
     struct sigaction new_action;
 
-    new_action.sa_handler = SourceRegistration::notify_sources_of_signal;
+    new_action.sa_sigaction = SourceRegistration::notify_sources_of_signal;
     sigfillset(&new_action.sa_mask);
-    new_action.sa_flags = no_flags;
+    new_action.sa_flags = SA_SIGINFO;
 
     if (sigaction(sig, &new_action, &old_action) == -1)
     {
