@@ -176,19 +176,6 @@ private:
     std::map<ms::Surface*, std::shared_ptr<ms::SurfaceObserver>> surface_observers;
 };
 
-std::shared_ptr<mi::Surface> topmost_surface_containing_point(
-    std::shared_ptr<mi::Scene> const& targets, geom::Point const& point)
-{
-    std::shared_ptr<mi::Surface> top_surface_at_point;
-    targets->for_each([&top_surface_at_point, &point]
-        (std::shared_ptr<mi::Surface> const& surface) 
-        {
-            if (surface->input_area_contains(point))
-                top_surface_at_point = surface;
-        });
-    return top_surface_at_point;
-}
-
 bool is_empty(std::shared_ptr<mg::CursorImage> const& image)
 {
     auto const size = image->size();
@@ -248,8 +235,7 @@ void mi::CursorController::set_cursor_image_locked(std::unique_lock<std::mutex>&
 
 void mi::CursorController::update_cursor_image_locked(std::unique_lock<std::mutex>& lock)
 {
-    auto surface = topmost_surface_containing_point(input_targets, cursor_location);
-    if (surface)
+    if (auto const surface = input_targets->input_surface_at(cursor_location))
     {
         set_cursor_image_locked(lock, surface->cursor_image());
     }
@@ -269,15 +255,21 @@ void mi::CursorController::cursor_moved_to(float abs_x, float abs_y)
 {
     auto const new_location = geom::Point{geom::X{abs_x}, geom::Y{abs_y}};
 
+    std::shared_ptr<ms::Surface> di;
     {
         std::unique_lock lock(cursor_state_guard);
 
         cursor_location = new_location;
 
         update_cursor_image_locked(lock);
+        di = drag_icon.lock();
     }
 
     cursor->move_to(new_location);
+    if (di)
+    {
+        di->move_to(new_location);
+    }
 }
 
 void mir::input::CursorController::pointer_usable()
@@ -285,26 +277,58 @@ void mir::input::CursorController::pointer_usable()
     std::lock_guard lock(serialize_pointer_usable_unusable);
     bool became_usable = false;
     std::shared_ptr<mg::CursorImage> image;
+    std::shared_ptr<ms::Surface> di;
+
     {
         std::lock_guard lock(cursor_state_guard);
         became_usable = !usable;
         image = current_cursor;
         usable = true;
+        di = drag_icon.lock();
     }
 
     if (became_usable)
     {
         if (image && !is_empty(image))
             cursor->show(*image);
+        if (di)
+        {
+            di->show();
+        }
     }
 }
 
 void mir::input::CursorController::pointer_unusable()
 {
     std::lock_guard lock(serialize_pointer_usable_unusable);
+    std::shared_ptr<ms::Surface> di;
+
     {
         std::lock_guard lock(cursor_state_guard);
         usable = false;
+        di = drag_icon.lock();
     }
+
     cursor->hide();
+    if (di)
+    {
+        di->hide();
+    }
+}
+
+void mir::input::CursorController::set_drag_icon(std::weak_ptr<scene::Surface> icon)
+{
+    std::shared_ptr<ms::Surface> di;
+
+    {
+        std::lock_guard lock{cursor_state_guard};
+        drag_icon = std::move(icon);
+
+        di = drag_icon.lock();
+    }
+
+    if (di)
+    {
+        di->move_to(cursor_location);
+    }
 }
