@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Andreas Pokorny <andreas.pokorny@canonical.com>
  */
 
 #ifndef MIR_INPUT_NULL_INPUT_TARGETER_H_
@@ -25,18 +23,17 @@ namespace mir
 {
 namespace input
 {
-class InputChannel;
 
 struct NullInputTargeter : public shell::InputTargeter
 {
     NullInputTargeter() = default;
-    virtual ~NullInputTargeter() noexcept(true) = default;
+    virtual ~NullInputTargeter() noexcept = default;
 
-    void focus_changed(std::shared_ptr<InputChannel const> const&) override
+    void set_focus(std::shared_ptr<Surface> const&) override
     {
     }
 
-    void focus_cleared() override
+    void clear_focus() override
     {
     }
 };
