@@ -21,7 +21,7 @@
 #include "mir/scene/null_observer.h"
 #include "mir/scene/surface.h"
 #include "mir/scene/null_surface_observer.h"
-#include "mir/events/event_builders.h"
+#include "mir/events/event_helpers.h"
 #include "mir/events/pointer_event.h"
 #include "mir_toolkit/mir_cookie.h"
 
@@ -44,10 +44,12 @@ struct InputDispatcherSceneObserver :
     InputDispatcherSceneObserver(
         std::function<void(std::shared_ptr<ms::Surface>)> const& on_removed,
         std::function<void(ms::Surface const*)> const& on_surface_moved,
-        std::function<void()> const& on_surface_resized)
+        std::function<void()> const& on_surface_resized,
+        std::function<void()> const& on_scene_changed)
         : on_removed(on_removed),
           on_surface_moved{on_surface_moved},
-          on_surface_resized{on_surface_resized}
+          on_surface_resized{on_surface_resized},
+          on_scene_changed{on_scene_changed}
     {
     }
 
@@ -61,6 +63,11 @@ struct InputDispatcherSceneObserver :
         on_removed(surface);
     }
 
+    void scene_changed() override
+    {
+        on_scene_changed();
+    }
+
     void surface_exists(std::shared_ptr<ms::Surface> const& surface) override
     {
         surface->register_interest(shared_from_this());
@@ -89,12 +96,21 @@ struct InputDispatcherSceneObserver :
     std::function<void(std::shared_ptr<ms::Surface>)> const on_removed;
     std::function<void(ms::Surface const*)> const on_surface_moved;
     std::function<void()> const on_surface_resized;
+    std::function<void()> const on_scene_changed;
 };
 
-void deliver_without_relative_motion(
-    std::shared_ptr<mi::Surface> const& surface,
-    MirEvent const* ev,
-    std::vector<uint8_t> const& drag_and_drop_handle)
+void set_local_positions_based_on_surface_input_bounds(
+    MirEvent& event,
+    mir::geometry::Rectangle const& input_bounds)
+{
+    mev::map_positions(event, [&](auto global, auto local)
+        {
+            local = global - geom::DisplacementF{as_displacement(input_bounds.top_left)};
+            return std::make_pair(global, local);
+        });
+}
+
+void deliver_without_relative_motion(std::shared_ptr<mi::Surface> const& surface, MirEvent const* ev)
 {
     auto const* input_ev = mir_event_get_input_event(ev);
     auto const* pev = mir_input_event_get_pointer_event(input_ev);
@@ -122,24 +138,16 @@ void deliver_without_relative_motion(
         0.0f,
         0.0f);
 
-    mev::transform_positions(*to_deliver, geom::Displacement{bounds.top_left.x.as_int(), bounds.top_left.y.as_int()});
-    if (!drag_and_drop_handle.empty())
-        mev::set_drag_and_drop_handle(*to_deliver, drag_and_drop_handle);
+    set_local_positions_based_on_surface_input_bounds(*to_deliver, bounds);
     surface->consume(std::move(to_deliver));
 }
 
-void deliver(
-    std::shared_ptr<mi::Surface> const& surface,
-    MirEvent const* ev,
-    std::vector<uint8_t> const& drag_and_drop_handle)
+void deliver(std::shared_ptr<mi::Surface> const& surface, MirEvent const* ev)
 {
     auto to_deliver = mev::clone_event(*ev);
 
-    if (!drag_and_drop_handle.empty())
-        mev::set_drag_and_drop_handle(*to_deliver, drag_and_drop_handle);
-
     auto const& bounds = surface->input_bounds();
-    mev::transform_positions(*to_deliver, geom::Displacement{bounds.top_left.x.as_int(), bounds.top_left.y.as_int()});
+    set_local_positions_based_on_surface_input_bounds(*to_deliver, bounds);
     surface->consume(std::move(to_deliver));
 }
 
@@ -147,12 +155,13 @@ void deliver(
 
 mi::SurfaceInputDispatcher::SurfaceInputDispatcher(std::shared_ptr<mi::Scene> const& scene)
     : scene(scene),
-      started(false)
+      screen_is_locked(false)
 {
     scene_observer = std::make_shared<InputDispatcherSceneObserver>(
         [this](std::shared_ptr<ms::Surface> const& s) { surface_removed(s); },
         [this](scene::Surface const* s) { surface_moved(s); },
-        [this] { surface_resized(); });
+        [this] { surface_resized(); },
+        [this] { scene_changed(); });
     scene->add_observer(scene_observer);
 }
 
@@ -181,14 +190,10 @@ void mi::SurfaceInputDispatcher::surface_removed(std::shared_ptr<ms::Surface> su
         set_focus_locked(lg, nullptr);
     }
 
-    for (auto& kv : pointer_state_by_id)
-    {
-        auto& state = kv.second;
-        if (compare_surfaces(state.current_target, surface.get()))
-            state.current_target.reset();
-        if (compare_surfaces(state.gesture_owner, surface.get()))
-            state.gesture_owner.reset();
-    }
+    if (compare_surfaces(current_target, surface.get()))
+        current_target.reset();
+    if (compare_surfaces(gesture_owner, surface.get()))
+        gesture_owner.reset();
 
     for (auto& kv : touch_state_by_id)
     {
@@ -210,7 +215,7 @@ struct SceneChangeContext
 
 SceneChangeContext context_for_event(
     MirEvent const* last_pointer_event,
-    std::function<std::shared_ptr<mi::Surface>*(MirInputDeviceId)> const& get_current_target,
+    std::shared_ptr<mi::Surface>& current_target,
     std::function<std::shared_ptr<mi::Surface>(geom::Point const&)> const& surface_under_point)
 {
     auto const iev = mir_event_get_input_event(last_pointer_event);
@@ -223,7 +228,7 @@ SceneChangeContext context_for_event(
     return SceneChangeContext{
         iev,
         pev,
-        *get_current_target(mir_input_event_get_device_id(iev)),
+        current_target,
         surface_under_point(event_x_y)
     };
 }
@@ -330,11 +335,11 @@ void mi::SurfaceInputDispatcher::surface_moved(ms::Surface const* moved_surface)
 
     auto ctx = context_for_event(
         last_pointer_event.get(),
-        [this](auto id) { return &this->ensure_pointer_state(id).current_target; },
-        [this](auto point) { return this->find_target_surface(point); });
+        current_target,
+        [this](auto point) { return scene->input_surface_at(point); });
 
     // If we're in a move/resize gesture we don't need to synthesize an event
-    if (ensure_pointer_state(mir_input_event_get_device_id(ctx.iev)).gesture_owner)
+    if (gesture_owner)
         return;
 
     auto const entered_surface_changed = dispatch_scene_change_enter_exit_events(
@@ -348,7 +353,7 @@ void mi::SurfaceInputDispatcher::surface_moved(ms::Surface const* moved_surface)
         send_motion_event_to_moved_surface(
             ctx,
             moved_surface,
-            [this](auto surf, auto ev) { deliver_without_relative_motion(surf, ev, drag_and_drop_handle); });
+            [](auto surf, auto ev) { deliver_without_relative_motion(surf, ev); });
     }
 }
 
@@ -361,8 +366,8 @@ void mi::SurfaceInputDispatcher::surface_resized()
 
     auto ctx = context_for_event(
         last_pointer_event.get(),
-        [this](auto id) { return &this->ensure_pointer_state(id).current_target; },
-        [this](auto point) { return this->find_target_surface(point); });
+        current_target,
+        [this](auto point) { return scene->input_surface_at(point); });
 
     auto const entered_surface_changed = dispatch_scene_change_enter_exit_events(
         ctx,
@@ -374,20 +379,19 @@ void mi::SurfaceInputDispatcher::surface_resized()
     }
 }
 
-void mi::SurfaceInputDispatcher::device_reset(MirInputDeviceId reset_device_id, std::chrono::nanoseconds /* when */)
+void mi::SurfaceInputDispatcher::scene_changed()
 {
-    std::lock_guard lg(dispatcher_mutex);
-
-    if (!started)
-        return;
-
-    auto pointer_it = pointer_state_by_id.find(reset_device_id);
-    if (pointer_it != pointer_state_by_id.end())
-        pointer_state_by_id.erase(pointer_it);
-    
-    auto touch_it = touch_state_by_id.find(reset_device_id);
-    if (touch_it != touch_state_by_id.end())
-        touch_state_by_id.erase(touch_it);
+    std::lock_guard lock{dispatcher_mutex};
+    bool const screen_is_locked_new = scene->screen_is_locked();
+    if (screen_is_locked != screen_is_locked_new && screen_is_locked_new)
+    {
+        auto const focused = focus_surface.lock();
+        if (focused && !focused->visible_on_lock_screen())
+        {
+            set_focus_locked(lock, nullptr);
+        }
+    }
+    screen_is_locked = screen_is_locked_new;
 }
 
 bool mi::SurfaceInputDispatcher::dispatch_key(std::shared_ptr<MirEvent const> const& ev)
@@ -419,40 +423,20 @@ bool is_gesture_terminator(MirPointerEvent const* pev)
 }
 }
 
-std::shared_ptr<mi::Surface> mi::SurfaceInputDispatcher::find_target_surface(geom::Point const& point)
-{
-    std::shared_ptr<mi::Surface> top_target = nullptr;
-    scene->for_each([&top_target, &point](std::shared_ptr<mi::Surface> const& target) {
-            if (target->input_area_contains(point))
-                top_target = target;
-    });
-    return top_target;
-}
-
 void mi::SurfaceInputDispatcher::send_enter_exit_event(std::shared_ptr<mi::Surface> const& surface,
                                                        MirPointerEvent const* pev,
                                                        MirPointerAction action)
 {
-    geom::DisplacementF const surface_displacement{as_displacement(surface->input_bounds().top_left)};
     auto const* input_ev = mir_pointer_event_input_event(pev);
     auto event = mev::clone_event(*mir_input_event_get_event(input_ev));
     auto const pointer_ev = static_cast<MirPointerEvent*>(event.get());
     pointer_ev->set_action(action);
     if (pointer_ev->position())
     {
-        pointer_ev->set_position(pointer_ev->position().value() - surface_displacement);
+        set_local_positions_based_on_surface_input_bounds(*event, surface->input_bounds());
     }
-    if (!drag_and_drop_handle.empty())
-    {
-        mev::set_drag_and_drop_handle(*event, drag_and_drop_handle);
-    }
-    surface->consume(std::move(event));
-}
 
-mi::SurfaceInputDispatcher::PointerInputState& mi::SurfaceInputDispatcher::ensure_pointer_state(MirInputDeviceId id)
-{
-    pointer_state_by_id.insert(std::make_pair(id, PointerInputState()));
-    return pointer_state_by_id[id];
+    surface->consume(std::move(event));
 }
 
 mi::SurfaceInputDispatcher::TouchInputState& mi::SurfaceInputDispatcher::ensure_touch_state(MirInputDeviceId id)
@@ -461,90 +445,88 @@ mi::SurfaceInputDispatcher::TouchInputState& mi::SurfaceInputDispatcher::ensure_
     return touch_state_by_id[id];
 }
 
-bool mi::SurfaceInputDispatcher::dispatch_pointer(MirInputDeviceId id, std::shared_ptr<MirEvent const> const& event)
+bool mi::SurfaceInputDispatcher::dispatch_pointer(MirInputDeviceId /*id*/, std::shared_ptr<MirEvent const> const& event)
 {
     auto const ev = event.get();
-    std::lock_guard lg(dispatcher_mutex);
+    std::unique_lock lg(dispatcher_mutex);
     last_pointer_event = event;
     auto const* input_ev = mir_event_get_input_event(ev);
     auto const* pev = mir_input_event_get_pointer_event(input_ev);
     auto action = mir_pointer_event_action(pev);
-    auto& pointer_state = ensure_pointer_state(id);
     geom::Point event_x_y = { mir_pointer_event_axis_value(pev,mir_pointer_axis_x),
                               mir_pointer_event_axis_value(pev,mir_pointer_axis_y) };
 
-    if (pointer_state.gesture_owner)
+    if (gesture_owner && dispatch_to_gesture_owner)
     {
-        deliver(pointer_state.gesture_owner, ev, drag_and_drop_handle);
+        deliver(gesture_owner, ev);
 
-        auto const gesture_terminated = is_gesture_terminator(pev);
-
-        if (gesture_terminated)
+        if (is_gesture_terminator(pev))
         {
-            pointer_state.gesture_owner.reset();
-        }
+            gesture_owner.reset();
 
-        if (gesture_terminated || !drag_and_drop_handle.empty())
-        {
-            auto target = find_target_surface(event_x_y);
+            auto target = scene->input_surface_at(event_x_y);
 
-            if (pointer_state.current_target != target)
+            if (current_target != target)
             {
-                if (pointer_state.current_target)
-                    send_enter_exit_event(pointer_state.current_target, pev, mir_pointer_action_leave);
+                if (current_target)
+                    send_enter_exit_event(current_target, pev, mir_pointer_action_leave);
 
-                pointer_state.current_target = target;
+                current_target = target;
                 if (target)
                     send_enter_exit_event(target, pev, mir_pointer_action_enter);
-
-                if (!gesture_terminated)
-                    pointer_state.gesture_owner = target;
             }
         }
 
         return true;
     }
-    else if (action == mir_pointer_action_button_up)
-    {
-        // If we have an up but no gesture owner
-        // then we never delivered the corresponding
-        // down to anyone, so we drop this event.
-        return false;
-    }
     else
     {
-        auto target = find_target_surface(event_x_y);
+        auto const target = scene->input_surface_at(event_x_y);
         bool sent_ev = false;
-        if (pointer_state.current_target != target)
+        if (current_target != target)
         {
-            if (pointer_state.current_target)
-                send_enter_exit_event(pointer_state.current_target, pev, mir_pointer_action_leave);
+            if (current_target)
+                send_enter_exit_event(current_target, pev, mir_pointer_action_leave);
 
-            pointer_state.current_target = target;
+            current_target = target;
             if (target)
                 send_enter_exit_event(target, pev, mir_pointer_action_enter);
 
             sent_ev = true;
         }
-        if (!target)
-            return sent_ev;
-        if (action == mir_pointer_action_button_down)
+
+        if (target)
         {
-            pointer_state.gesture_owner = target;
+            if (action == mir_pointer_action_button_down)
+            {
+                gesture_owner = target;
+            }
+
+            if (sent_ev)
+            {
+                if (action != mir_pointer_action_motion)
+                    deliver_without_relative_motion(target, ev);
+            }
+            else
+            {
+                deliver(target, ev);
+            }
+            sent_ev = true;
         }
 
-        if (sent_ev)
+        if (is_gesture_terminator(pev))
         {
-            if (action != mir_pointer_action_motion)
-                deliver_without_relative_motion(target, ev, drag_and_drop_handle);
+            gesture_owner.reset();
+            dispatch_to_gesture_owner = true;
+            auto end_gesture = on_end_gesture;
+            on_end_gesture = []{};
+            lg.unlock();
+
+            end_gesture();
         }
-        else
-        {
-            deliver(target, ev, drag_and_drop_handle);
-        }
-        return true;
+
+        return sent_ev;
     }
-    return false;
 }
 
 namespace
@@ -591,12 +573,12 @@ bool mi::SurfaceInputDispatcher::dispatch_touch(MirInputDeviceId id, MirEvent co
         geom::Point event_x_y = { mir_touch_event_axis_value(tev, 0, mir_touch_axis_x),
                                   mir_touch_event_axis_value(tev, 0, mir_touch_axis_y) };
 
-        gesture_owner = find_target_surface(event_x_y);
+        gesture_owner = scene->input_surface_at(event_x_y);
     }
 
     if (gesture_owner)
     {
-        deliver(gesture_owner, ev, drag_and_drop_handle);
+        deliver(gesture_owner, ev);
 
         if (is_gesture_end(tev))
             gesture_owner.reset();
@@ -626,30 +608,28 @@ bool mi::SurfaceInputDispatcher::dispatch(std::shared_ptr<MirEvent const> const&
     default:
         BOOST_THROW_EXCEPTION(std::logic_error("InputDispatcher got an input event of unknown type"));
     }
-    
-    return true;
 }
 
 void mi::SurfaceInputDispatcher::start()
 {
-    std::lock_guard lg(dispatcher_mutex);
-
-    started = true;
 }
 
 void mi::SurfaceInputDispatcher::stop()
 {
     std::lock_guard lg(dispatcher_mutex);
 
-    pointer_state_by_id.clear();
+    gesture_owner.reset();
+    current_target.reset();
     touch_state_by_id.clear();
     last_pointer_event.reset();
-    
-    started = false;
 }
 
 void mi::SurfaceInputDispatcher::set_focus_locked(std::lock_guard<std::mutex> const&, std::shared_ptr<mi::Surface> const& target)
 {
+    if (screen_is_locked && target && !target->visible_on_lock_screen())
+    {
+        return;
+    }
     focus_surface = target;
     keyboard_multiplexer.keyboard_focus_set(target);
 }
@@ -666,18 +646,6 @@ void mi::SurfaceInputDispatcher::clear_focus()
     set_focus_locked(lg, nullptr);
 }
 
-void mir::input::SurfaceInputDispatcher::set_drag_and_drop_handle(std::vector<uint8_t> const& handle)
-{
-    std::lock_guard lg(dispatcher_mutex);
-    drag_and_drop_handle = handle;
-}
-
-void mir::input::SurfaceInputDispatcher::clear_drag_and_drop_handle()
-{
-    std::lock_guard lg(dispatcher_mutex);
-    drag_and_drop_handle.clear();
-}
-
 void mir::input::SurfaceInputDispatcher::register_interest(std::weak_ptr<KeyboardObserver> const& observer)
 {
     keyboard_multiplexer.register_interest(observer);
@@ -694,3 +662,10 @@ void mir::input::SurfaceInputDispatcher::unregister_interest(KeyboardObserver co
 {
     keyboard_multiplexer.unregister_interest(observer);
 }
+
+void mir::input::SurfaceInputDispatcher::disable_dispatch_to_gesture_owner(std::function<void()> on_end_gesture)
+{
+    std::lock_guard lg(dispatcher_mutex);
+    dispatch_to_gesture_owner = false;
+    this->on_end_gesture = std::move(on_end_gesture);
+}
