@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,13 +12,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Andreas Pokorny <andreas.pokorny@canonical.com>
  */
 
 #include "mir/default_server_configuration.h"
 #include "mir/options/configuration.h"
 
+#include "reports.h"
 #include "lttng_report_factory.h"
 #include "logging_report_factory.h"
 #include "null_report_factory.h"
@@ -37,15 +36,15 @@ std::unique_ptr<mir::report::ReportFactory> mir::DefaultServerConfiguration::rep
 
     if (opt == options::log_opt_value)
     {
-        return std::unique_ptr<mir::report::ReportFactory>(new report::LoggingReportFactory(the_logger(), the_clock()));
+        return std::make_unique<report::LoggingReportFactory>(the_logger(), the_clock());
     }
     else if (opt == options::lttng_opt_value)
     {
-        return std::unique_ptr<mir::report::ReportFactory>(new report::LttngReportFactory());
+        return std::make_unique<report::LttngReportFactory>();
     }
     else if (opt == options::off_opt_value)
     {
-        return std::unique_ptr<mir::report::ReportFactory>(new report::NullReportFactory());
+        return std::make_unique<report::NullReportFactory>();
     }
     else
     {
@@ -55,6 +54,11 @@ std::unique_ptr<mir::report::ReportFactory> mir::DefaultServerConfiguration::rep
     }
 }
 
+std::shared_ptr<void> mir::DefaultServerConfiguration::default_reports()
+{
+    return std::make_unique<report::Reports>(*this, *the_options());
+}
+
 auto mir::DefaultServerConfiguration::the_compositor_report() -> std::shared_ptr<mc::CompositorReport>
 {
     return compositor_report(
@@ -64,33 +68,6 @@ auto mir::DefaultServerConfiguration::the_compositor_report() -> std::shared_ptr
         });
 }
 
-auto mir::DefaultServerConfiguration::the_connector_report() -> std::shared_ptr<mf::ConnectorReport>
-{
-    return connector_report(
-        [this]()->std::shared_ptr<mf::ConnectorReport>
-        {
-            return report_factory(options::connector_report_opt)->create_connector_report();
-        });
-}
-
-auto mir::DefaultServerConfiguration::the_session_mediator_report() -> std::shared_ptr<mf::SessionMediatorReport>
-{
-    return session_mediator_report(
-        [this]()->std::shared_ptr<mf::SessionMediatorReport>
-        {
-            return report_factory(options::session_mediator_report_opt)->create_session_mediator_report();
-        });
-}
-
-auto mir::DefaultServerConfiguration::the_message_processor_report() -> std::shared_ptr<mf::MessageProcessorReport>
-{
-    return message_processor_report(
-        [this]()->std::shared_ptr<mf::MessageProcessorReport>
-        {
-            return report_factory(options::msg_processor_report_opt)->create_message_processor_report();
-        });
-}
-
 auto mir::DefaultServerConfiguration::the_display_report() -> std::shared_ptr<mg::DisplayReport>
 {
     return display_report(
@@ -126,3 +103,13 @@ auto mir::DefaultServerConfiguration::the_shared_library_prober_report() -> std:
             return report_factory(options::shared_library_prober_report_opt)->create_shared_library_prober_report();
         });
 }
+
+auto mir::DefaultServerConfiguration::the_shell_report() -> std::shared_ptr<shell::ShellReport>
+{
+    return shell_report(
+        [this]()->std::shared_ptr<shell::ShellReport>
+        {
+            return report_factory(options::shell_report_opt)->create_shell_report();
+        });
+}
+
