@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,12 +12,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Andreas Pokorny <andreas.pokorny@canonical.com>
  */
 
 #include "compositor_report.h"
 
+#include "mir/graphics/buffer.h"
 #include "mir/report/lttng/mir_tracepoint.h"
 
 #define TRACEPOINT_DEFINE
@@ -42,6 +41,16 @@ void mir::report::lttng::CompositorReport::began_frame(SubCompositorId id)
     mir_tracepoint(mir_server_compositor, began_frame, id);
 }
 
+void mir::report::lttng::CompositorReport::renderables_in_frame(
+    SubCompositorId id, graphics::RenderableList const& list)
+{
+    std::vector<uint32_t> ids(list.size());
+    auto it = list.begin();
+    for(auto& id : ids)
+        id = (*it++)->buffer()->id().as_value();
+    mir_tracepoint(mir_server_compositor, buffers_in_frame, id, ids.data(), ids.size());
+}
+
 void mir::report::lttng::CompositorReport::rendered_frame(SubCompositorId id)
 {
     mir_tracepoint(mir_server_compositor, rendered_frame, id);
@@ -51,4 +60,3 @@ void mir::report::lttng::CompositorReport::finished_frame(SubCompositorId id)
 {
     mir_tracepoint(mir_server_compositor, finished_frame, id);
 }
-
