@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "mir/report/lttng/mir_tracepoint.h"
@@ -39,11 +37,6 @@ void mir::report::lttng::InputReport::published_motion_event(int dest_fd, uint32
     mir_tracepoint(mir_server_input, published_motion_event, dest_fd, seq_id, event_time);
 }
 
-void mir::report::lttng::InputReport::received_event_finished_signal(int src_fd, uint32_t seq_id)
-{
-    mir_tracepoint(mir_server_input, received_event_finished_signal, src_fd, seq_id);
-}
-
 void mir::report::lttng::InputReport::opened_input_device(char const* name, char const* platform)
 {
     mir_tracepoint(mir_server_input, opened_input_device, name, platform);
