@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
 #undef TRACEPOINT_PROVIDER
@@ -39,56 +37,53 @@ TRACEPOINT_EVENT(
      )
 )
 
-TRACEPOINT_EVENT(
+TRACEPOINT_EVENT_CLASS(
     mir_server_input,
+    published_event,
+    TP_ARGS(int, dest_fd, uint32_t, seq_id, int64_t, event_time),
+    TP_FIELDS(
+        ctf_integer(int, dest_fd, dest_fd)
+        ctf_integer(uint32_t, seq_id, seq_id)
+        ctf_integer(int64_t, event_time, event_time)
+    )
+)
+
+TRACEPOINT_EVENT_INSTANCE(
+    mir_server_input,
+    published_event,
     published_key_event,
-    TP_ARGS(int, dest_fd, uint32_t, seq_id, int64_t, event_time),
-    TP_FIELDS(
-        ctf_integer(int, dest_fd, dest_fd)
-        ctf_integer(uint32_t, seq_id, seq_id)
-        ctf_integer(int64_t, event_time, event_time)
-     )
+    TP_ARGS(int, dest_fd, uint32_t, seq_id, int64_t, event_time)
 )
 
-TRACEPOINT_EVENT(
+TRACEPOINT_EVENT_INSTANCE(
     mir_server_input,
+    published_event,
     published_motion_event,
-    TP_ARGS(int, dest_fd, uint32_t, seq_id, int64_t, event_time),
-    TP_FIELDS(
-        ctf_integer(int, dest_fd, dest_fd)
-        ctf_integer(uint32_t, seq_id, seq_id)
-        ctf_integer(int64_t, event_time, event_time)
-     )
+    TP_ARGS(int, dest_fd, uint32_t, seq_id, int64_t, event_time)
 )
 
-TRACEPOINT_EVENT(
+TRACEPOINT_EVENT_CLASS(
     mir_server_input,
-    received_event_finished_signal,
-    TP_ARGS(int, src_fd, uint32_t, seq_id),
+    device_event,
+    TP_ARGS(const char*, device, const char*, platform),
     TP_FIELDS(
-        ctf_integer(int, src_fd, src_fd)
-        ctf_integer(uint32_t, seq_id, seq_id)
+        ctf_string(device, device)
+        ctf_string(platform, platform)
     )
 )
 
-TRACEPOINT_EVENT(
+TRACEPOINT_EVENT_INSTANCE(
     mir_server_input,
+    device_event,
     opened_input_device,
-    TP_ARGS(const char*, device, const char*, platform),
-    TP_FIELDS(
-        ctf_string(device, device)
-        ctf_string(platform, platform)
-    )
+    TP_ARGS(const char*, device, const char*, platform)
 )
 
-TRACEPOINT_EVENT(
+TRACEPOINT_EVENT_INSTANCE(
     mir_server_input,
+    device_event,
     failed_to_open_input_device,
-    TP_ARGS(const char*, device, const char*, platform),
-    TP_FIELDS(
-        ctf_string(device, device)
-        ctf_string(platform, platform)
-    )
+    TP_ARGS(const char*, device, const char*, platform)
 )
 
 #endif /* MIR_LTTNG_DISPLAY_REPORT_TP_H_ */
