@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Daniel van Vugt <daniel.van.vugt@canonical.com>
  */
 
 #include "compositor_report.h"
@@ -28,6 +26,10 @@ void mrn::CompositorReport::began_frame(SubCompositorId)
 {
 }
 
+void mrn::CompositorReport::renderables_in_frame(SubCompositorId, mir::graphics::RenderableList const&)
+{
+}
+
 void mrn::CompositorReport::rendered_frame(SubCompositorId)
 {
 }
