@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Daniel van Vugt <daniel.van.vugt@canonical.com>
  */
 
 #ifndef MIR_REPORT_NULL_COMPOSITOR_REPORT_H_
@@ -33,6 +31,7 @@ class CompositorReport : public compositor::CompositorReport
 public:
     void added_display(int width, int height, int x, int y, SubCompositorId id) override;
     void began_frame(SubCompositorId id) override;
+    void renderables_in_frame(SubCompositorId id, graphics::RenderableList const& renderables) override;
     void rendered_frame(SubCompositorId id) override;
     void finished_frame(SubCompositorId id) override;
     void started() override;
