@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013,2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 
@@ -33,14 +31,15 @@ class InputReport : public input::InputReport
 {
 public:
     InputReport() = default;
-    virtual ~InputReport() noexcept(true) = default;
+    virtual ~InputReport() noexcept = default;
 
-    void received_event_from_kernel(int64_t when, int type, int code, int value);
+    void received_event_from_kernel(int64_t when, int type, int code, int value) override;
 
-    void published_key_event(int dest_fd, uint32_t seq_id, int64_t event_time);
-    void published_motion_event(int dest_fd, uint32_t seq_id, int64_t event_time);
+    void published_key_event(int dest_fd, uint32_t seq_id, int64_t event_time) override;
+    void published_motion_event(int dest_fd, uint32_t seq_id, int64_t event_time) override;
 
-    void received_event_finished_signal(int src_fd, uint32_t seq_id);
+    void opened_input_device(char const* device_name, char const* input_platform) override;
+    void failed_to_open_input_device(char const* device_name, char const* input_platform) override;
 };
 
 }
