@@ -34,7 +34,6 @@
 
 namespace mc = mir::compositor;
 namespace ms = mir::scene;
-namespace msh = mir::shell;
 namespace mg = mir::graphics;
 namespace mi = mir::input;
 namespace mf = mir::frontend;
@@ -130,11 +129,6 @@ public:
         for_each_observer(&SurfaceObserver::input_consumed, surf, event);
     }
 
-    void start_drag_and_drop(Surface const* surf, std::vector<uint8_t> const& handle) override
-    {
-        for_each_observer(&SurfaceObserver::start_drag_and_drop, surf, handle);
-    }
-
     void depth_layer_set_to(Surface const* surf, MirDepthLayer depth_layer) override
     {
         for_each_observer(&SurfaceObserver::depth_layer_set_to, surf, depth_layer);
@@ -822,11 +816,6 @@ void ms::BasicSurface::placed_relative(geometry::Rectangle const& placement)
     observers->placed_relative(this, placement);
 }
 
-void mir::scene::BasicSurface::start_drag_and_drop(std::vector<uint8_t> const& handle)
-{
-    observers->start_drag_and_drop(this, handle);
-}
-
 auto mir::scene::BasicSurface::depth_layer() const -> MirDepthLayer
 {
     return synchronised_state.lock()->depth_layer;
@@ -838,6 +827,16 @@ void mir::scene::BasicSurface::set_depth_layer(MirDepthLayer depth_layer)
     observers->depth_layer_set_to(this, depth_layer);
 }
 
+auto mir::scene::BasicSurface::visible_on_lock_screen() const -> bool
+{
+    return synchronised_state.lock()->visible_on_lock_screen;
+}
+
+void mir::scene::BasicSurface::set_visible_on_lock_screen(bool visible)
+{
+    synchronised_state.lock()->visible_on_lock_screen = visible;
+}
+
 std::optional<geom::Rectangle> mir::scene::BasicSurface::clip_area() const
 {
     return synchronised_state.lock()->clip_area;
