@@ -36,6 +36,7 @@ namespace ms = mir::scene;
 namespace mc = mir::compositor;
 namespace mg = mir::graphics;
 namespace mi = mir::input;
+namespace mf = mir::frontend;
 namespace geom = mir::geometry;
 
 namespace
@@ -157,7 +158,7 @@ mc::SceneElementSequence ms::SurfaceStack::scene_elements_for(mc::CompositorID i
     {
         for (auto const& surface : layer)
         {
-            if (surface->visible())
+            if (surface_can_be_shown(surface) && surface->visible())
             {
                 for (auto& renderable : surface->generate_renderables(id))
                 {
@@ -187,7 +188,7 @@ int ms::SurfaceStack::frames_pending(mc::CompositorID id) const
     {
         for (auto const& surface : layer)
         {
-            if (surface->visible())
+            if (surface_can_be_shown(surface) && surface->visible())
             {
                 auto const tracker = rendering_trackers.find(surface.get());
                 if (tracker != rendering_trackers.end() && tracker->second->is_exposed_in(id))
@@ -331,7 +332,7 @@ auto ms::SurfaceStack::surface_at(geometry::Point cursor) const
             // TODO be maintained and whether this test will detect clicks on
             // TODO decorations (it should) as these may be outside the area
             // TODO known to the client.  But it works for now.
-            if (surface->input_area_contains(cursor))
+            if (surface_can_be_shown(surface) && surface->input_area_contains(cursor))
                     return surface;
         }
     }
@@ -339,16 +340,9 @@ auto ms::SurfaceStack::surface_at(geometry::Point cursor) const
     return {};
 }
 
-void ms::SurfaceStack::for_each(std::function<void(std::shared_ptr<mi::Surface> const&)> const& callback)
+auto ms::SurfaceStack::input_surface_at(geometry::Point point) const -> std::shared_ptr<input::Surface>
 {
-    RecursiveReadLock lg(guard);
-    for (auto const& layer : surface_layers)
-    {
-        for (auto const& surface : layer)
-        {
-            callback(surface);
-        }
-    }
+    return surface_at(point);
 }
 
 void ms::SurfaceStack::raise(Surface const* surface)
@@ -456,6 +450,11 @@ void ms::SurfaceStack::insert_surface_at_top_of_depth_layer(std::shared_ptr<Surf
     surface_layers[depth_index].push_back(surface);
 }
 
+auto ms::SurfaceStack::surface_can_be_shown(std::shared_ptr<Surface> const& surface) const -> bool
+{
+    return screen_lock_handle.expired() || surface->visible_on_lock_screen();
+}
+
 void ms::SurfaceStack::add_observer(std::shared_ptr<ms::Observer> const& observer)
 {
     observers.add(observer);
@@ -500,6 +499,80 @@ auto ms::SurfaceStack::stacking_order_of(SurfaceSet const& surfaces) const -> Su
     return result;
 }
 
+struct ms::SurfaceStack::SharedScreenLock
+{
+    SharedScreenLock(std::weak_ptr<ms::SurfaceStack> surface_stack) : surface_stack{std::move(surface_stack)}
+    {
+    }
+
+    ~SharedScreenLock()
+    {
+        if (auto const shared = surface_stack.lock())
+        {
+            // shared->screen_is_locked() should already be false
+            shared->emit_scene_changed();
+        }
+    }
+
+private:
+    std::weak_ptr<ms::SurfaceStack> surface_stack;
+};
+
+struct ms::SurfaceStack::BasicScreenLockHandle : mf::ScreenLockHandle
+{
+    BasicScreenLockHandle(std::shared_ptr<SharedScreenLock> lock)
+        : lock{new std::shared_ptr<SharedScreenLock>{std::move(lock)}}
+    {
+    }
+
+    ~BasicScreenLockHandle()
+    {
+        if (allowed_to_be_dropped)
+        {
+            delete lock;
+        }
+        else
+        {
+            fatal_error("The screen was not properly unlocked");
+        }
+    }
+
+    void allow_to_be_dropped() override
+    {
+        allowed_to_be_dropped = true;
+    }
+
+private:
+    std::shared_ptr<SharedScreenLock> const* const lock;
+    bool allowed_to_be_dropped{false};
+};
+
+auto ms::SurfaceStack::lock_screen() -> std::unique_ptr<mf::ScreenLockHandle>
+{
+    std::shared_ptr<SharedScreenLock> shared;
+    bool is_new{false};
+    {
+        RecursiveWriteLock lk(guard);
+        shared = screen_lock_handle.lock();
+        if (!shared)
+        {
+            screen_lock_handle = shared = std::make_shared<SharedScreenLock>(shared_from_this());
+            is_new = true;
+        }
+    }
+    if (is_new)
+    {
+        emit_scene_changed();
+    }
+    return std::make_unique<BasicScreenLockHandle>(shared);
+}
+
+auto ms::SurfaceStack::screen_is_locked() const -> bool
+{
+    RecursiveReadLock lk(guard);
+    return !screen_lock_handle.expired();
+}
+
 void ms::Observers::surface_added(std::shared_ptr<Surface> const& surface)
 {
     for_each([&](std::shared_ptr<Observer> const& observer)
