@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,99 +12,152 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored By: Alan Griffiths <alan@octopull.co.uk>
  */
 
-#define MIR_LOG_COMPONENT "Server"
 #include "mir/server.h"
 
 #include "mir/emergency_cleanup.h"
 #include "mir/fd.h"
 #include "mir/frontend/connector.h"
 #include "mir/graphics/graphic_buffer_allocator.h"
+#include "mir/graphics/display_buffer.h"
+#include "mir/input/composite_event_filter.h"
+#include "mir/input/event_filter.h"
 #include "mir/options/default_configuration.h"
+#include "mir/renderer/gl/render_target.h"
 #include "mir/default_server_configuration.h"
 #include "mir/logging/logger.h"
 #include "mir/log.h"
 #include "mir/main_loop.h"
 #include "mir/report_exception.h"
 #include "mir/run_mir.h"
+#include "mir/cookie/authority.h"
 
 // TODO these are used to frig a stub renderer when running headless
-#include "mir/compositor/renderer.h"
+#include "mir/renderer/renderer.h"
 #include "mir/graphics/renderable.h"
-#include "mir/compositor/renderer_factory.h"
+#include "mir/renderer/renderer_factory.h"
+
+#include "frontend_wayland/wayland_connector.h"
 
 #include <iostream>
+#include <mir/server.h>
+
 
 namespace mo = mir::options;
+namespace mi = mir::input;
+
+namespace
+{
+struct TemporaryCompositeEventFilter : public mi::CompositeEventFilter
+{
+    bool handle(MirEvent const&) override { return false; }
+    void append(std::weak_ptr<mi::EventFilter> const& filter) override
+    {
+        append_event_filters.push_back(filter);
+    }
+    void prepend(std::weak_ptr<mi::EventFilter> const& filter) override
+    {
+        prepend_event_filters.push_back(filter);
+    }
+
+    void move_filters(std::shared_ptr<mi::CompositeEventFilter> const& composite_event_filter)
+    {
+        for (auto const& filter : prepend_event_filters)
+            composite_event_filter->prepend(filter);
+
+        for (auto const& filter : append_event_filters)
+            composite_event_filter->append(filter);
+
+        append_event_filters.clear();
+        prepend_event_filters.clear();
+    }
+    std::vector<std::weak_ptr<mi::EventFilter>> prepend_event_filters;
+    std::vector<std::weak_ptr<mi::EventFilter>> append_event_filters;
+};
+}
 
 #define FOREACH_WRAPPER(MACRO)\
+    MACRO(cursor)\
     MACRO(cursor_listener)\
     MACRO(display_buffer_compositor_factory)\
     MACRO(display_configuration_policy)\
-    MACRO(shell)
+    MACRO(shell)\
+    MACRO(surface_stack)\
+    MACRO(application_not_responding_detector)
 
 #define FOREACH_OVERRIDE(MACRO)\
     MACRO(compositor)\
+    MACRO(cursor_images)\
     MACRO(display_buffer_compositor_factory)\
     MACRO(gl_config)\
-    MACRO(host_lifecycle_event_listener)\
     MACRO(input_dispatcher)\
+    MACRO(input_targeter)\
     MACRO(logger)\
-    MACRO(placement_strategy)\
     MACRO(prompt_session_listener)\
     MACRO(prompt_session_manager)\
     MACRO(server_status_listener)\
     MACRO(session_authorizer)\
     MACRO(session_listener)\
-    MACRO(session_mediator_report)\
     MACRO(shell)\
-    MACRO(surface_configurator)
+    MACRO(application_not_responding_detector)\
+    MACRO(cookie_authority)\
+    MACRO(persistent_surface_store)
 
 #define FOREACH_ACCESSOR(MACRO)\
+    MACRO(the_buffer_stream_factory)\
     MACRO(the_compositor)\
-    MACRO(the_composite_event_filter)\
+    MACRO(the_compositor_report)\
     MACRO(the_cursor_listener)\
+    MACRO(the_cursor)\
     MACRO(the_display)\
+    MACRO(the_display_configuration_controller)\
     MACRO(the_focus_controller)\
     MACRO(the_gl_config)\
-    MACRO(the_graphics_platform)\
-    MACRO(the_input_targeter)\
+    MACRO(the_display_platforms)    \
+    MACRO(the_rendering_platforms)  \
+    MACRO(the_input_targeter)  \
     MACRO(the_logger)\
     MACRO(the_main_loop)\
     MACRO(the_prompt_session_listener)\
     MACRO(the_session_authorizer)\
     MACRO(the_session_coordinator)\
     MACRO(the_session_listener)\
+    MACRO(the_surface_factory)\
     MACRO(the_prompt_session_manager)\
     MACRO(the_shell)\
     MACRO(the_shell_display_layout)\
-    MACRO(the_surface_configurator)\
-    MACRO(the_surface_coordinator)\
-    MACRO(the_touch_visualizer)
+    MACRO(the_surface_stack)\
+    MACRO(the_touch_visualizer)\
+    MACRO(the_input_device_hub)\
+    MACRO(the_application_not_responding_detector)\
+    MACRO(the_persistent_surface_store)\
+    MACRO(the_display_configuration_observer_registrar)\
+    MACRO(the_seat_observer_registrar)
 
 #define MIR_SERVER_BUILDER(name)\
-    std::function<std::result_of<decltype(&mir::DefaultServerConfiguration::the_##name)(mir::DefaultServerConfiguration*)>::type()> name##_builder;
+    std::function<std::invoke_result_t<decltype(&mir::DefaultServerConfiguration::the_##name),mir::DefaultServerConfiguration*>()> name##_builder;
 
 #define MIR_SERVER_WRAPPER(name)\
-    std::function<std::result_of<decltype(&mir::DefaultServerConfiguration::the_##name)(mir::DefaultServerConfiguration*)>::type\
-        (std::result_of<decltype(&mir::DefaultServerConfiguration::the_##name)(mir::DefaultServerConfiguration*)>::type const&)> name##_wrapper;
+    std::function<std::invoke_result_t<decltype(&mir::DefaultServerConfiguration::the_##name),mir::DefaultServerConfiguration*>\
+        (std::invoke_result_t<decltype(&mir::DefaultServerConfiguration::the_##name),mir::DefaultServerConfiguration*> const&)> name##_wrapper;
 
 struct mir::Server::Self
 {
     bool exit_status{false};
-    std::weak_ptr<options::Option> options;
     std::string config_file;
     std::shared_ptr<ServerConfiguration> server_config;
 
+    std::function<void()> pre_init_callback{[]{}};
     std::function<void()> init_callback{[]{}};
+    std::function<void()> stop_callback{[]{}};
     int argc{0};
     char const** argv{nullptr};
     std::function<void()> exception_handler{};
     Terminator terminator{};
     EmergencyCleanupHandler emergency_cleanup_handler;
+    std::shared_ptr<TemporaryCompositeEventFilter> temporary_event_filter{
+        std::make_shared<TemporaryCompositeEventFilter>()};
 
     std::function<void(int argc, char const* const* argv)> command_line_hander{};
 
@@ -118,6 +171,8 @@ struct mir::Server::Self
 
     FOREACH_OVERRIDE(MIR_SERVER_BUILDER)
 
+    shell::WindowManagerBuilder wmb;
+
     FOREACH_WRAPPER(MIR_SERVER_WRAPPER)
 };
 
@@ -148,36 +203,6 @@ auto wrap_##name(decltype(Self::name##_wrapper)::result_type const& wrapped)\
     return mir::DefaultServerConfiguration::wrap_##name(wrapped);\
 }
 
-// TODO these are used to frig a stub renderer when running headless
-namespace
-{
-class StubRenderer : public mir::compositor::Renderer
-{
-public:
-    void set_viewport(mir::geometry::Rectangle const&) override {}
-
-    void set_rotation(float) override {}
-
-    void render(mir::graphics::RenderableList const& renderables) const override
-    {
-        for (auto const& r : renderables)
-            r->buffer(); // We need to consume a buffer to unblock client tests
-    }
-
-    void suspend() override {}
-};
-
-class StubRendererFactory : public mir::compositor::RendererFactory
-{
-public:
-    auto create_renderer_for(mir::geometry::Rectangle const&, mir::compositor::DestinationAlpha)
-    -> std::unique_ptr<mir::compositor::Renderer>
-    {
-        return std::unique_ptr<mir::compositor::Renderer>(new StubRenderer());
-    }
-};
-}
-
 struct mir::Server::ServerConfiguration : mir::DefaultServerConfiguration
 {
     ServerConfiguration(
@@ -188,18 +213,9 @@ struct mir::Server::ServerConfiguration : mir::DefaultServerConfiguration
     {
     }
 
-    // TODO this is an ugly frig to avoid exposing the render factory to end users and tests running headless
-    auto the_renderer_factory() -> std::shared_ptr<compositor::RendererFactory> override
+    std::function<void()> the_stop_callback() override
     {
-        auto const& options = the_options();
-        if (options->is_set(options::platform_graphics_lib))
-        {
-            auto const graphics_lib = options->get<std::string>(options::platform_graphics_lib);
-
-            if (graphics_lib.find("graphics-dummy.so") != std::string::npos)
-                return std::make_shared<StubRendererFactory>();
-        }
-        return mir::DefaultServerConfiguration::the_renderer_factory();
+        return self->stop_callback;
     }
 
     using mir::DefaultServerConfiguration::the_options;
@@ -208,6 +224,12 @@ struct mir::Server::ServerConfiguration : mir::DefaultServerConfiguration
 
     FOREACH_WRAPPER(MIR_SERVER_CONFIG_WRAP)
 
+    auto the_window_manager_builder() -> shell::WindowManagerBuilder override
+    {
+        if (self->wmb) return self->wmb;
+        return mir::DefaultServerConfiguration::the_window_manager_builder();
+    }
+
     Self* const self;
 };
 
@@ -216,36 +238,18 @@ struct mir::Server::ServerConfiguration : mir::DefaultServerConfiguration
 
 namespace
 {
-class ConfigurationOptions : public mo::DefaultConfiguration
-{
-public:
-    using mo::DefaultConfiguration::DefaultConfiguration;
-
-    std::string config_file;
-
-    void parse_config_file(
-        boost::program_options::options_description& options_description,
-        mo::ProgramOption& options) const override
-    {
-        if (!config_file.empty())
-            options.parse_file(options_description, config_file);
-    }
-};
-
 std::shared_ptr<mo::DefaultConfiguration> configuration_options(
     int argc,
     char const** argv,
     std::function<void(int argc, char const* const* argv)> const& command_line_hander,
     std::string const& config_file)
 {
-    std::shared_ptr<ConfigurationOptions> result;
+    std::shared_ptr<mo::DefaultConfiguration> result;
 
     if (command_line_hander)
-        result = std::make_shared<ConfigurationOptions>(argc, argv, command_line_hander);
+        result = std::make_shared<mo::DefaultConfiguration>(argc, argv, command_line_hander, config_file);
     else
-        result = std::make_shared<ConfigurationOptions>(argc, argv);
-
-    result->config_file = config_file;
+        result = std::make_shared<mo::DefaultConfiguration>(argc, argv, config_file);
 
     return result;
 }
@@ -284,6 +288,19 @@ void mir::Server::set_command_line(int argc, char const* argv[])
     self->argv = argv;
 }
 
+void mir::Server::add_pre_init_callback(std::function<void()> const& pre_init_callback)
+{
+    auto const& existing = self->pre_init_callback;
+
+    auto const updated = [=]
+        {
+            existing();
+            pre_init_callback();
+        };
+
+    self->pre_init_callback = updated;
+}
+
 void mir::Server::add_init_callback(std::function<void()> const& init_callback)
 {
     auto const& existing = self->init_callback;
@@ -297,6 +314,19 @@ void mir::Server::add_init_callback(std::function<void()> const& init_callback)
     self->init_callback = updated;
 }
 
+void mir::Server::add_stop_callback(std::function<void()> const& stop_callback)
+{
+    auto const& existing = self->stop_callback;
+
+    auto const updated = [=]
+        {
+            stop_callback();
+            existing();
+        };
+
+    self->stop_callback = updated;
+}
+
 void mir::Server::set_command_line_handler(
     std::function<void(int argc, char const* const* argv)> const& command_line_hander)
 {
@@ -313,7 +343,7 @@ void mir::Server::set_config_filename(std::string const& config_file)
 auto mir::Server::get_options() const -> std::shared_ptr<options::Option>
 {
     verify_accessing_allowed(self->server_config);
-    return self->options.lock();
+    return self->server_config->the_options();
 }
 
 void mir::Server::set_exception_handler(std::function<void()> const& exception_handler)
@@ -351,40 +381,57 @@ void mir::Server::apply_settings()
 
     auto const config = std::make_shared<ServerConfiguration>(options, self);
     self->server_config = config;
-    self->options = config->the_options();
 
     mir::logging::set_logger(config->the_logger());
 }
 
 void mir::Server::run()
-try
 {
-    mir::log_info("Starting");
-    verify_accessing_allowed(self->server_config);
+    try
+    {
+        mir::log_info("Starting");
+        verify_accessing_allowed(self->server_config);
 
-    auto const emergency_cleanup = self->server_config->the_emergency_cleanup();
+        auto const emergency_cleanup = self->server_config->the_emergency_cleanup();
+        auto const composite_event_filter = self->server_config->the_composite_event_filter();
 
-    if (self->emergency_cleanup_handler)
-        emergency_cleanup->add(self->emergency_cleanup_handler);
+        // keep the default_reports alive while the server is running
+        auto const default_reports = self->server_config->default_reports();
 
-    run_mir(
-        *self->server_config,
-        [&](DisplayServer&) { self->init_callback(); },
-        self->terminator);
+        self->temporary_event_filter->move_filters(composite_event_filter);
 
-    self->exit_status = true;
+        if (self->emergency_cleanup_handler)
+            emergency_cleanup->add(self->emergency_cleanup_handler);
+
+        self->pre_init_callback();
+
+        run_mir(
+            *self->server_config,
+            [&](DisplayServer&)
+                { self->init_callback(); self->init_callback = []{}; },
+            self->terminator);
+
+        self->exit_status = true;
+    }
+    catch (...)
+    {
+        if (self->exception_handler)
+            self->exception_handler();
+        else
+            mir::report_exception(std::cerr);
+    }
+    /*
+     * The server_config *must* outlive exception processing as it holds references to
+     * the loaded platforms, and those references are keeping the DSOs loaded.
+     *
+     * If exception has been thrown from a platform DSO then the DSO must not be unloaded
+     * before exception processing has finished.
+     *
+     * This is safe to have outside a try {} catch {} as shared_ptr<>::reset() is specified
+     * as noexcept.
+     */
     self->server_config.reset();
 }
-catch (...)
-{
-    self->server_config = nullptr;
-
-    if (self->exception_handler)
-        self->exception_handler();
-    else
-        mir::report_exception(std::cerr);
-}
-
 auto mir::Server::supported_pixel_formats() const -> std::vector<MirPixelFormat>
 {
     return self->server_config->the_buffer_allocator()->supported_pixel_formats();
@@ -394,8 +441,12 @@ void mir::Server::stop()
 {
     mir::log_info("Stopping");
     if (self->server_config)
-        if (auto const main_loop = the_main_loop())
+        if (auto const main_loop = the_main_loop().get())
+        {
+            self->stop_callback();
+            self->stop_callback = []{};
             main_loop->stop();
+        }
 }
 
 bool mir::Server::exited_normally()
@@ -403,26 +454,71 @@ bool mir::Server::exited_normally()
     return self->exit_status;
 }
 
-auto mir::Server::open_client_socket() -> Fd
+auto mir::Server::open_wayland_client_socket() -> Fd
 {
     if (auto const config = self->server_config)
-        return Fd{config->the_connector()->client_socket_fd()};
+        return Fd{config->the_wayland_connector()->client_socket_fd()};
 
     BOOST_THROW_EXCEPTION(std::logic_error("Cannot open connection when not running"));
 }
 
-auto mir::Server::open_prompt_socket() -> Fd
+auto mir::Server::wayland_display() const -> optional_value<std::string>
 {
     if (auto const config = self->server_config)
-        return Fd{config->the_prompt_connector()->client_socket_fd()};
+        return config->the_wayland_connector()->socket_name();
 
     BOOST_THROW_EXCEPTION(std::logic_error("Cannot open connection when not running"));
 }
 
-auto mir::Server::open_client_socket(ConnectHandler const& connect_handler) -> Fd
+auto mir::Server::x11_display() const -> mir::optional_value<std::string>
 {
     if (auto const config = self->server_config)
-        return Fd{config->the_connector()->client_socket_fd(connect_handler)};
+        return config->the_xwayland_connector()->socket_name();
+
+    BOOST_THROW_EXCEPTION(std::logic_error("Cannot open connection when not running"));
+}
+
+void mir::Server::run_on_wayland_display(std::function<void(wl_display*)> const& functor)
+{
+    if (auto const config = self->server_config)
+    {
+        std::dynamic_pointer_cast<mir::frontend::WaylandConnector>(config->the_wayland_connector())
+            ->run_on_wayland_display(functor);
+    }
+}
+
+void mir::Server::add_wayland_extension(
+    std::string const& name, std::function<std::shared_ptr<void>(
+        wl_display*,
+        std::function<void(std::function<void()>&& work)> const&)> builder)
+{
+    if (auto const config = self->server_config)
+    {
+        config->add_wayland_extension(name, builder);
+    }
+}
+
+void mir::Server::set_wayland_extension_filter(
+    std::function<bool(std::shared_ptr<scene::Session> const&, char const*)> const& extension_filter)
+{
+    if (auto const config = self->server_config)
+    {
+        config->set_wayland_extension_filter(extension_filter);
+    }
+}
+
+void mir::Server::set_enabled_wayland_extensions(std::vector<std::string> const& extensions)
+{
+    if (auto const config = self->server_config)
+    {
+        config->set_enabled_wayland_extensions(extensions);
+    }
+}
+
+auto mir::Server::open_client_wayland(ConnectHandler const& connect_handler) -> int
+{
+    if (auto const config = self->server_config)
+        return config->the_wayland_connector()->client_socket_fd(connect_handler);
 
     BOOST_THROW_EXCEPTION(std::logic_error("Cannot open connection when not running"));
 }
@@ -449,6 +545,12 @@ FOREACH_OVERRIDE(MIR_SERVER_OVERRIDE)
 
 #undef MIR_SERVER_OVERRIDE
 
+void mir::Server::override_the_window_manager_builder(shell::WindowManagerBuilder const wmb)
+{
+    verify_setting_allowed(self->server_config);
+    self->wmb = wmb;
+}
+
 #define MIR_SERVER_WRAP(name)\
 void mir::Server::wrap_##name(decltype(Self::name##_wrapper) const& value)\
 {\
@@ -460,6 +562,14 @@ FOREACH_WRAPPER(MIR_SERVER_WRAP)
 
 #undef MIR_SERVER_WRAP
 
+auto mir::Server::the_composite_event_filter() const -> decltype(self->server_config->the_composite_event_filter())
+{
+    if (self->server_config)
+        return self->server_config->the_composite_event_filter();
+    else
+        return self->temporary_event_filter;
+}
+
 void mir::Server::add_configuration_option(
     std::string const& option,
     std::string const& description,
@@ -481,6 +591,27 @@ void mir::Server::add_configuration_option(
     self->set_add_configuration_options(option_adder);
 }
 
+void mir::Server::add_configuration_option(
+    std::string const& option,
+    std::string const& description,
+    double default_)
+{
+    verify_setting_allowed(self->server_config);
+    namespace po = boost::program_options;
+
+    auto const& existing = self->add_configuration_options;
+
+    auto const option_adder = [=](options::DefaultConfiguration& config)
+        {
+            existing(config);
+
+            config.add_options()
+            (option.c_str(), po::value<double>()->default_value(default_), description.c_str());
+        };
+
+    self->set_add_configuration_options(option_adder);
+}
+
 void mir::Server::add_configuration_option(
     std::string const& option,
     std::string const& description,
@@ -598,5 +729,19 @@ void mir::Server::add_configuration_option(
             self->set_add_configuration_options(option_adder);
         }
         break;
+
+    case OptionType::strings:
+        {
+            auto const option_adder = [=](options::DefaultConfiguration& config)
+                {
+                    existing(config);
+
+                    config.add_options()
+                    (option.c_str(), po::value<std::vector<std::string>>(), description.c_str());
+                };
+
+            self->set_add_configuration_options(option_adder);
+        }
+        break;
     }
 }
