@@ -362,29 +362,21 @@ void msh::AbstractShell::raise_surface(
     window_manager->handle_raise_surface(session, surface, timestamp);
 }
 
-void msh::AbstractShell::request_drag_and_drop(
-    std::shared_ptr<scene::Session> const& session,
-    std::shared_ptr<scene::Surface> const& surface,
-    uint64_t timestamp)
-{
-    window_manager->handle_request_drag_and_drop(session, surface, timestamp);
-}
-
 void msh::AbstractShell::request_move(
     std::shared_ptr<scene::Session> const& session,
     std::shared_ptr<scene::Surface> const& surface,
-    uint64_t timestamp)
+    MirInputEvent const* event)
 {
-    window_manager->handle_request_move(session, surface, timestamp);
+    window_manager->handle_request_move(session, surface, event);
 }
 
 void msh::AbstractShell::request_resize(
     std::shared_ptr<scene::Session> const& session,
     std::shared_ptr<scene::Surface> const& surface,
-    uint64_t timestamp,
+    MirInputEvent const* event,
     MirResizeEdge edge)
 {
-    window_manager->handle_request_resize(session, surface, timestamp, edge);
+    window_manager->handle_request_resize(session, surface, event, edge);
 }
 
 void msh::AbstractShell::focus_next_session()
@@ -687,12 +679,3 @@ void msh::AbstractShell::raise(SurfaceSet const& surfaces)
     report->surfaces_raised(surfaces);
 }
 
-void msh::AbstractShell::set_drag_and_drop_handle(std::vector<uint8_t> const& handle)
-{
-    input_targeter->set_drag_and_drop_handle(handle);
-}
-
-void msh::AbstractShell::clear_drag_and_drop_handle()
-{
-    input_targeter->clear_drag_and_drop_handle();
-}
