@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
 #ifndef MIR_SHELL_GRAPHICS_DISPLAY_LAYOUT_H_
@@ -37,10 +35,10 @@ class GraphicsDisplayLayout : public DisplayLayout
 public:
     GraphicsDisplayLayout(std::shared_ptr<graphics::Display> const& display);
 
-    void clip_to_output(geometry::Rectangle& rect);
-    void size_to_output(geometry::Rectangle& rect);
-    void place_in_output(graphics::DisplayConfigurationOutputId output_id,
-                         geometry::Rectangle& rect);
+    void clip_to_output(geometry::Rectangle& rect) override;
+    void size_to_output(geometry::Rectangle& rect) override;
+    bool place_in_output(graphics::DisplayConfigurationOutputId output_id,
+                         geometry::Rectangle& rect) override;
 
 private:
     geometry::Rectangle get_output_for(geometry::Rectangle& rect);
