@@ -1,8 +1,8 @@
 /*
- * Copyright © 2015 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,15 +12,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
 #ifndef MIR_TEST_DOUBLES_MOCK_GL_BUFFER_H_
 #define MIR_TEST_DOUBLES_MOCK_GL_BUFFER_H_
 
 #include "mock_buffer.h"
-#include "mir/renderer/gl/texture_source.h"
+#include "mir/graphics/texture.h"
+#include "mir/graphics/program.h"
+#include "mir/graphics/program_factory.h"
 
 namespace mir
 {
@@ -29,15 +29,17 @@ namespace test
 namespace doubles
 {
 
-struct MockGLBuffer : public MockBuffer,
-                      public renderer::gl::TextureSource
+struct MockTextureBuffer : public MockBuffer,
+                           public graphics::gl::Texture
 {
- public:
+public:
     using MockBuffer::MockBuffer;
 
-    MOCK_METHOD0(gl_bind_to_texture, void());
+    MOCK_METHOD(graphics::gl::Program const&, shader, (graphics::gl::ProgramFactory&), (const override));
+    MOCK_METHOD(Layout, layout, (), (const override));
+    MOCK_METHOD(void, bind, (), (override));
+    MOCK_METHOD(void, add_syncpoint, (), (override));
 };
-
 }
 }
 }
