@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Robert Carr <robert.carr@canonical.com>
  */
 
 #ifndef MIR_TEST_DOUBLES_MOCK_INPUT_MANAGER_H_
@@ -33,9 +31,10 @@ namespace doubles
 
 struct MockInputManager : public input::InputManager
 {
-    MOCK_METHOD1(add_platform, void(std::shared_ptr<mir::input::Platform> const&));
     MOCK_METHOD0(start, void());
     MOCK_METHOD0(stop, void());
+    MOCK_METHOD0(pause_for_config, void());
+    MOCK_METHOD0(continue_after_config, void());
 };
 
 }
