@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Josh Arenson <joshua.arenson@canonical.com>
  */
 
 #ifndef MIR_TEST_FRAMEWORK_COMMAND_LINE_SERVER_CONFIGURATION
@@ -27,12 +25,15 @@ namespace options
 {
 class DefaultConfiguration;
 }
+class Server;
 }
 
 namespace mir_test_framework
 {
     auto configuration_from_commandline() 
     -> std::shared_ptr<mir::options::DefaultConfiguration>;
+
+    void configure_from_commandline(mir::Server& server);
 }
 
 #endif /* MIR_TEST_FRAMEWORK_COMMAND_LINE_SERVER_CONFIGURATION */
