@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Robert Carr <robert.carr@canonical.com>
  */
 
 
@@ -31,24 +29,6 @@
 #include <atomic>
 #include <thread>
 
-namespace mir
-{
-namespace input
-{
-namespace android
-{
-class FakeEventHub;
-}
-}
-namespace test
-{
-namespace doubles
-{
-class FakeEventHubInputConfiguration;
-}
-}
-}
-
 namespace mir_test_framework
 {
 
@@ -61,20 +41,13 @@ public:
     void on_start() override;
     void on_exit() override;
 
-    std::shared_ptr<droidinput::EventHubInterface> the_event_hub() override;
     std::shared_ptr<mir::input::InputManager> the_input_manager() override;
     std::shared_ptr<mir::input::InputDispatcher> the_input_dispatcher() override;
     std::shared_ptr<mir::shell::InputTargeter> the_input_targeter() override;
-    std::shared_ptr<mir::input::InputSender> the_input_sender() override;
-
-    std::shared_ptr<mir::input::android::FakeEventHub> fake_event_hub;
 
 protected:
     virtual void inject_input() = 0;
 
-    void wait_until_client_appears(std::string const& surface_name);
-
-private:
     std::thread input_injection_thread;
 };
 
