@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,11 +12,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
-#include "mir_test/cross_process_action.h"
+#include "mir/test/cross_process_action.h"
 
 namespace mt = mir::test;
 
@@ -27,8 +25,8 @@ void mt::CrossProcessAction::exec(std::function<void()> const& f)
     finish_sync.signal_ready();
 }
 
-void mt::CrossProcessAction::operator()()
+void mt::CrossProcessAction::operator()(std::chrono::milliseconds timeout)
 {
     start_sync.signal_ready();
-    finish_sync.wait_for_signal_ready();
+    finish_sync.wait_for_signal_ready_for(timeout);
 }
