@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,12 +12,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alberto Aguirre <alberto.aguirre@canonical.com>
- *              Daniel van Vugt <daniel.van.vugt@canonical.com>
  */
 
-#include <mir_test/popen.h>
+#include <mir/test/popen.h>
 
 #include <system_error>
 
@@ -48,7 +45,7 @@ bool mt::Popen::get_line(std::string& line)
             line.append(got);
     } while (got && !feof(in) && !line.empty() && line.back() != '\n');
 
-    if (line.back() == '\n')
+    if (!line.empty() && line.back() == '\n')
         line.pop_back();
 
     return !line.empty() || !feof(in);
