@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,13 +12,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by:
- * Thomas Voss <thomas.voss@canonical.com>
- * Kevin DuBois <kevin.dubois@canonical.com>
  */
 
-#include "mir_test_doubles/mock_egl.h"
+#include "mir/test/doubles/mock_egl.h"
 #include <gtest/gtest.h>
 
 namespace mtd = mir::test::doubles;
@@ -44,6 +40,30 @@ EGLImageKHR extension_eglCreateImageKHR(EGLDisplay dpy, EGLContext ctx, EGLenum
                                         EGLClientBuffer buffer, const EGLint *attrib_list);
 EGLBoolean extension_eglDestroyImageKHR (EGLDisplay dpy, EGLImageKHR image);
 void extension_glEGLImageTargetTexture2DOES(GLenum target, GLeglImageOES image);
+EGLSyncKHR extension_eglCreateSyncKHR(EGLDisplay dpy, EGLenum type, const EGLint *attrib_list);
+EGLBoolean extension_eglDestroySyncKHR(EGLDisplay dpy, EGLSyncKHR sync);
+EGLint extension_eglClientWaitSyncKHR(EGLDisplay dpy, EGLSyncKHR sync, EGLint flags, EGLTimeKHR timeout);
+EGLBoolean extension_eglGetSyncValuesCHROMIUM(EGLDisplay dpy,
+    EGLSurface surface, int64_t *ust, int64_t *msc, int64_t *sbc);
+EGLBoolean extension_eglBindWaylandDisplayWL(
+    EGLDisplay dpy,
+    struct wl_display *display);
+EGLBoolean extension_eglUnbindWaylandDisplayWL(
+    EGLDisplay dpy,
+    struct wl_display *display);
+EGLBoolean extension_eglQueryWaylandBufferWL(
+    EGLDisplay dpy,
+    struct wl_resource *buffer,
+    EGLint attribute, EGLint *value);
+EGLDisplay extension_eglGetPlatformDisplayEXT(
+    EGLenum platform,
+    void *native_display,
+    const EGLint *attrib_list);
+EGLSurface extension_eglCreatePlatformWindowSurfaceEXT(
+    EGLDisplay dpy,
+    EGLConfig config,
+    void *native_window,
+    const EGLint *attrib_list);
 
 /* EGL{Surface,Display,Config,Context} are all opaque types, so we can put whatever
    we want in them for testing */
@@ -54,7 +74,7 @@ mtd::MockEGL::MockEGL()
       fake_egl_surface((EGLSurface) 0xa034),
       fake_egl_context((EGLContext) 0xbeef),
       fake_egl_image((EGLImageKHR) 0x1234),
-      fake_visual_id(1) //HAL_PIXEL_FORMAT_RGBA on android
+      fake_visual_id(1)
 {
     using namespace testing;
     assert(global_mock_egl == NULL && "Only one mock object per process is allowed");
@@ -63,6 +83,8 @@ mtd::MockEGL::MockEGL()
 
     ON_CALL(*this, eglGetDisplay(_))
     .WillByDefault(Return(fake_egl_display));
+    ON_CALL(*this, eglGetPlatformDisplayEXT(_,_,_))
+        .WillByDefault(Return(fake_egl_display));
     ON_CALL(*this, eglInitialize(_,_,_))
     .WillByDefault(DoAll(
                        SetArgPointee<1>(1),
@@ -82,13 +104,27 @@ mtd::MockEGL::MockEGL()
                        Return(EGL_TRUE)));
 
     ON_CALL(*this, eglChooseConfig(_,_,_,_,_))
-    .WillByDefault(DoAll(
-                       SetArgPointee<2>(&fake_configs),
-                       SetArgPointee<4>(fake_configs_num),
-                       Return(EGL_TRUE)));
+    .WillByDefault(Invoke(
+        [&] (EGLDisplay, const EGLint *, EGLConfig *configs,
+             EGLint config_size, EGLint *num_config) -> EGLBoolean
+        {
+            if (configs != nullptr)
+            {
+                EGLint const min_size = std::min(config_size, fake_configs_num);
+                std::copy(fake_configs, fake_configs + min_size, configs);
+                *num_config = min_size;
+            }
+            else
+            {
+                *num_config = fake_configs_num;
+            }
+            return EGL_TRUE;
+        }));
 
     ON_CALL(*this, eglCreateWindowSurface(_,_,_,_))
-    .WillByDefault(Return(fake_egl_surface));
+        .WillByDefault(Return(fake_egl_surface));
+    ON_CALL(*this, eglCreatePlatformWindowSurfaceEXT(_,_,_,_))
+        .WillByDefault(Return(fake_egl_surface));
 
     ON_CALL(*this, eglCreatePbufferSurface(_,_,_))
     .WillByDefault(Return(fake_egl_surface));
@@ -97,10 +133,23 @@ mtd::MockEGL::MockEGL()
     .WillByDefault(Return(fake_egl_context));
 
     ON_CALL(*this, eglMakeCurrent(_,_,_,_))
-    .WillByDefault(Return(EGL_TRUE));
+    .WillByDefault(Invoke(
+        [this] (EGLDisplay, EGLSurface, EGLSurface, EGLContext context)
+        {
+            std::lock_guard lock{current_contexts_mutex};
+            current_contexts[std::this_thread::get_id()] = context;
+            return EGL_TRUE;
+        }));
+
+    ON_CALL(*this, eglGetCurrentContext())
+    .WillByDefault(Invoke([this]
+        {
+            std::lock_guard lock{current_contexts_mutex};
+            return current_contexts[std::this_thread::get_id()];
+        }));
 
     ON_CALL(*this, eglSwapBuffers(_,_))
-    .WillByDefault(Return(EGL_TRUE));
+        .WillByDefault(Return(EGL_TRUE));                              
 
     ON_CALL(*this, eglGetCurrentDisplay())
     .WillByDefault(Return(fake_egl_display));
@@ -115,6 +164,57 @@ mtd::MockEGL::MockEGL()
         .WillByDefault(Return(reinterpret_cast<func_ptr_t>(extension_eglDestroyImageKHR)));
     ON_CALL(*this, eglGetProcAddress(StrEq("glEGLImageTargetTexture2DOES")))
         .WillByDefault(Return(reinterpret_cast<func_ptr_t>(extension_glEGLImageTargetTexture2DOES)));
+    ON_CALL(*this, eglGetProcAddress(StrEq("eglCreateSyncKHR")))
+        .WillByDefault(Return(reinterpret_cast<func_ptr_t>(extension_eglCreateSyncKHR)));
+    ON_CALL(*this, eglGetProcAddress(StrEq("eglDestroySyncKHR")))
+        .WillByDefault(Return(reinterpret_cast<func_ptr_t>(extension_eglDestroySyncKHR)));
+    ON_CALL(*this, eglGetProcAddress(StrEq("eglClientWaitSyncKHR")))
+        .WillByDefault(Return(reinterpret_cast<func_ptr_t>(extension_eglClientWaitSyncKHR)));
+    ON_CALL(*this, eglGetProcAddress(StrEq("eglGetSyncValuesCHROMIUM")))
+        .WillByDefault(Return(
+            reinterpret_cast<func_ptr_t>(extension_eglGetSyncValuesCHROMIUM)
+            ));
+    ON_CALL(*this, eglGetProcAddress(StrEq("eglQueryWaylandBufferWL")))
+        .WillByDefault(Return(reinterpret_cast<func_ptr_t>(&extension_eglQueryWaylandBufferWL)));
+    ON_CALL(*this, eglGetProcAddress(StrEq("eglBindWaylandDisplayWL")))
+        .WillByDefault(Return(reinterpret_cast<func_ptr_t>(&extension_eglBindWaylandDisplayWL)));
+    ON_CALL(*this, eglGetProcAddress(StrEq("eglUnbindWaylandDisplayWL")))
+        .WillByDefault(Return(reinterpret_cast<func_ptr_t>(&extension_eglUnbindWaylandDisplayWL)));
+    ON_CALL(*this, eglGetProcAddress(StrEq("eglGetPlatformDisplayEXT")))
+        .WillByDefault(Return(reinterpret_cast<func_ptr_t>(&extension_eglGetPlatformDisplayEXT)));
+    ON_CALL(*this, eglGetProcAddress(StrEq("eglCreatePlatformWindowSurfaceEXT")))
+        .WillByDefault(Return(reinterpret_cast<func_ptr_t>(&extension_eglCreatePlatformWindowSurfaceEXT)));
+}
+
+void mtd::MockEGL::provide_egl_extensions()
+{
+    using namespace testing;
+
+    const char* egl_exts =
+        "EGL_KHR_image "
+        "EGL_KHR_image_base "
+        "EGL_KHR_image_pixmap "
+        "GL_OES_EGL_image "
+        "EGL_EXT_image_dma_buf_import "
+        "EGL_WL_bind_wayland_display "
+        "EGL_EXT_platform_base";
+    ON_CALL(*this, eglQueryString(_,EGL_EXTENSIONS))
+        .WillByDefault(Return(egl_exts));
+}
+
+void mtd::MockEGL::provide_stub_platform_buffer_swapping()
+{
+    using namespace ::testing;
+    // TODO: Comment
+    ON_CALL(*this, eglCreateWindowSurface(_,_,_,_))
+        .WillByDefault(Invoke(
+            [&] (EGLDisplay,EGLConfig,AnyNativeType nw, EGLint const*) -> EGLSurface
+            {
+                return reinterpret_cast<EGLSurface>(nw);
+            }));
+
+    ON_CALL(*this, eglSwapBuffers(_,_))
+        .WillByDefault(Return(EGL_TRUE));
 }
 
 mtd::MockEGL::~MockEGL()
@@ -148,7 +248,7 @@ EGLint eglGetError (void)
 EGLDisplay eglGetDisplay (NativeDisplayType display)
 {
     CHECK_GLOBAL_MOCK(EGLDisplay);
-    return global_mock_egl->eglGetDisplay(display);
+    return global_mock_egl->eglGetDisplay(reinterpret_cast<mtd::MockEGL::AnyNativeType>(display));
 }
 
 EGLBoolean eglInitialize (EGLDisplay dpy, EGLint *major, EGLint *minor)
@@ -202,13 +302,13 @@ EGLBoolean eglGetConfigAttrib (EGLDisplay dpy, EGLConfig config, EGLint attribut
 EGLSurface eglCreateWindowSurface (EGLDisplay dpy, EGLConfig config, NativeWindowType window, const EGLint *attrib_list)
 {
     CHECK_GLOBAL_MOCK(EGLSurface)
-    return global_mock_egl->eglCreateWindowSurface(dpy, config, window, attrib_list);
+    return global_mock_egl->eglCreateWindowSurface(dpy, config, reinterpret_cast<mtd::MockEGL::AnyNativeType>(window), attrib_list);
 }
 
 EGLSurface eglCreatePixmapSurface (EGLDisplay dpy, EGLConfig config, NativePixmapType pixmap, const EGLint *attrib_list)
 {
     CHECK_GLOBAL_MOCK(EGLSurface)
-    return global_mock_egl->eglCreatePixmapSurface(dpy, config, pixmap, attrib_list);
+    return global_mock_egl->eglCreatePixmapSurface(dpy, config, reinterpret_cast<mtd::MockEGL::AnyNativeType>(pixmap), attrib_list);
 }
 
 EGLSurface eglCreatePbufferSurface (EGLDisplay dpy, EGLConfig config, const EGLint *attrib_list)
@@ -318,7 +418,7 @@ EGLBoolean eglSwapBuffers (EGLDisplay dpy, EGLSurface draw)
 EGLBoolean eglCopyBuffers (EGLDisplay dpy, EGLSurface surface, NativePixmapType target)
 {
     CHECK_GLOBAL_MOCK(EGLBoolean)
-    return global_mock_egl->eglCopyBuffers(dpy, surface, target);
+    return global_mock_egl->eglCopyBuffers(dpy, surface, reinterpret_cast<mtd::MockEGL::AnyNativeType>(target));
 }
 
 /* extensions */
@@ -339,3 +439,82 @@ void extension_glEGLImageTargetTexture2DOES(GLenum target, GLeglImageOES image)
     CHECK_GLOBAL_VOID_MOCK();
     global_mock_egl->glEGLImageTargetTexture2DOES(target, image);
 }
+
+EGLSyncKHR extension_eglCreateSyncKHR(EGLDisplay dpy, EGLenum type, const EGLint *attrib_list)
+{
+    CHECK_GLOBAL_MOCK(EGLSyncKHR);
+    return global_mock_egl->eglCreateSyncKHR(dpy, type, attrib_list);
+}
+
+EGLBoolean extension_eglDestroySyncKHR(EGLDisplay dpy, EGLSyncKHR sync)
+{
+    CHECK_GLOBAL_MOCK(EGLBoolean);
+    return global_mock_egl->eglDestroySyncKHR(dpy, sync);
+}
+
+EGLint extension_eglClientWaitSyncKHR(EGLDisplay dpy, EGLSyncKHR sync, EGLint flags, EGLTimeKHR timeout)
+{
+    CHECK_GLOBAL_MOCK(EGLint);
+    return global_mock_egl->eglClientWaitSyncKHR(dpy, sync, flags, timeout);
+}
+
+EGLBoolean extension_eglGetSyncValuesCHROMIUM(EGLDisplay dpy,
+              EGLSurface surface, int64_t *ust, int64_t *msc, int64_t *sbc)
+{
+    CHECK_GLOBAL_MOCK(EGLBoolean);
+    return global_mock_egl->eglGetSyncValuesCHROMIUM(dpy, surface,
+                                                     ust, msc, sbc);
+}
+
+EGLBoolean extension_eglBindWaylandDisplayWL(
+    EGLDisplay dpy,
+    struct wl_display *display)
+{
+    CHECK_GLOBAL_MOCK(EGLBoolean);
+    return global_mock_egl->eglBindWaylandDisplayWL(dpy, display);
+}
+
+EGLBoolean extension_eglUnbindWaylandDisplayWL(
+    EGLDisplay dpy,
+    struct wl_display *display)
+{
+    CHECK_GLOBAL_MOCK(EGLBoolean);
+    return global_mock_egl->eglUnbindWaylandDisplayWL(dpy, display);
+}
+
+EGLBoolean extension_eglQueryWaylandBufferWL(
+    EGLDisplay dpy,
+    struct wl_resource* buffer,
+    EGLint attribute, EGLint* value)
+{
+    CHECK_GLOBAL_MOCK(EGLBoolean);
+    return global_mock_egl->eglQueryWaylandBufferWL(
+        dpy, buffer, attribute, value);
+}
+
+
+EGLDisplay extension_eglGetPlatformDisplayEXT(
+    EGLenum platform,
+    void *native_display,
+    const EGLint *attrib_list)
+{
+    CHECK_GLOBAL_MOCK(EGLDisplay);
+    return global_mock_egl->eglGetPlatformDisplayEXT(
+        platform,
+        native_display,
+        attrib_list);
+}
+
+EGLSurface extension_eglCreatePlatformWindowSurfaceEXT(
+    EGLDisplay dpy,
+    EGLConfig config,
+    void *native_window,
+    const EGLint *attrib_list)
+{
+    CHECK_GLOBAL_MOCK(EGLSurface);
+    return global_mock_egl->eglCreatePlatformWindowSurfaceEXT(
+        dpy,
+        config,
+        native_window,
+        attrib_list);
+}
