@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,14 +12,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by:
- * Kevin DuBois <kevin.dubois@canonical.com>
  */
 
 #include "mir/test/doubles/mock_gl.h"
 #include <gtest/gtest.h>
 
+#include <GLES2/gl2.h>
+
 #include <cstring>
 
 namespace mtd = mir::test::doubles;
@@ -89,7 +88,8 @@ GLenum glGetError()
 
 const GLubyte* glGetString(GLenum name)
 {
-    CHECK_GLOBAL_MOCK(const GLubyte*);
+    if (!global_mock_gl)
+        return nullptr;
     return global_mock_gl->glGetString(name);
 }
 
@@ -129,12 +129,25 @@ void glDisable(GLenum func)
     global_mock_gl->glDisable(func);
 }
 
+void glBlendColor(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha)
+{
+    CHECK_GLOBAL_VOID_MOCK();
+    global_mock_gl->glBlendColor(red, green, blue, alpha);
+}
+
 void glBlendFunc(GLenum src, GLenum dst)
 {
     CHECK_GLOBAL_VOID_MOCK();
     global_mock_gl->glBlendFunc (src, dst);
 }
 
+void glBlendFuncSeparate(GLenum src_rgb, GLenum dst_rgb,
+                         GLenum src_alpha, GLenum dst_alpha)
+{
+    CHECK_GLOBAL_VOID_MOCK();
+    global_mock_gl->glBlendFuncSeparate(src_rgb, dst_rgb, src_alpha, dst_alpha);
+}
+
 void glActiveTexture(GLenum unit)
 {
     CHECK_GLOBAL_VOID_MOCK();
@@ -448,3 +461,15 @@ void glPixelStorei(GLenum pname, GLint param)
     CHECK_GLOBAL_VOID_MOCK();
     global_mock_gl->glPixelStorei(pname, param);
 }
+
+void glDrawElements(GLenum mode, GLsizei count, GLenum type, const void* indicies)
+{
+    CHECK_GLOBAL_VOID_MOCK();
+    global_mock_gl->glDrawElements(mode, count, type, indicies);
+}
+
+void glScissor(GLint x, GLint y, GLsizei width, GLsizei height)
+{
+    CHECK_GLOBAL_VOID_MOCK();
+    global_mock_gl->glScissor(x, y, width, height);
+}
