@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,6 +16,8 @@
  */
 
 #include "mir_test_framework/command_line_server_configuration.h"
+#include "mir_test_framework/main.h"
+
 #include "mir/options/default_configuration.h"
 #include "mir/server.h"
 
@@ -28,8 +30,9 @@ namespace mo=mir::options;
 
 namespace
 {
+char const* args[] = {nullptr};
 int argc;
-char const** argv;
+char const** argv = args;
 }
 
 auto mir_test_framework::configuration_from_commandline()
@@ -43,14 +46,21 @@ void mir_test_framework::configure_from_commandline(mir::Server& server)
     server.set_command_line(::argc, ::argv);
 }
 
-int main(int argc, char** argv)
+void mir_test_framework::set_commandline(int argc, char* argv[])
 {
-    // Override this standard gtest message
-    std::cout << "Running main() from " << basename(__FILE__) << std::endl;
-    ::testing::InitGoogleTest(&argc, argv);
-
     ::argv = const_cast<char const**>(argv);
     ::argc = argc;
+}
 
+int mir_test_framework::main(int argc, char* argv[])
+{
+    ::testing::InitGoogleTest(&argc, argv);
+    set_commandline(argc, argv);
     return RUN_ALL_TESTS();
 }
+
+void mir_test_framework::get_commandline(int* argc, char const* const** argv)
+{
+    *argc = ::argc;
+    *argv = ::argv;
+}
