@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored By: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #ifndef MIR_TEST_FRAMEWORK_STUBBED_GRAPHICS_PLATFORM_H_
@@ -23,16 +21,17 @@
 
 namespace mir_test_framework
 {
-class StubGraphicPlatform : public mir::test::doubles::NullPlatform
+class StubGraphicPlatform :
+    public mir::test::doubles::NullDisplayPlatform,
+    public mir::test::doubles::NullRenderingPlatform
 {
 public:
     StubGraphicPlatform(std::vector<mir::geometry::Rectangle> const& display_rects);
 
-    std::shared_ptr<mir::graphics::GraphicBufferAllocator> create_buffer_allocator() override;
+    mir::UniqueModulePtr<mir::graphics::GraphicBufferAllocator> create_buffer_allocator(
+        mir::graphics::Display const& output) override;
 
-    std::shared_ptr<mir::graphics::PlatformIpcOperations> make_ipc_operations() const override;
-
-    std::shared_ptr<mir::graphics::Display> create_display(
+    mir::UniqueModulePtr<mir::graphics::Display> create_display(
         std::shared_ptr<mir::graphics::DisplayConfigurationPolicy> const&,
         std::shared_ptr<mir::graphics::GLConfig> const&) override;
 
