@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013-2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "mir_test_framework/stubbed_server_configuration.h"
@@ -24,12 +22,9 @@
 #include "mir/options/default_configuration.h"
 #include "mir/graphics/cursor.h"
 
-#include "mir/test/doubles/stub_display_buffer.h"
 #include "mir/test/doubles/stub_renderer.h"
-#include "mir/test/doubles/stub_input_sender.h"
-#include "mir/test/doubles/stub_legacy_input_dispatchable.h"
 
-#include "mir/compositor/renderer_factory.h"
+#include "mir/renderer/renderer_factory.h"
 #include "src/server/input/null_input_manager.h"
 #include "src/server/input/null_input_dispatcher.h"
 #include "src/server/input/null_input_targeter.h"
@@ -48,12 +43,12 @@ namespace mtf = mir_test_framework;
 
 namespace
 {
-class StubRendererFactory : public mc::RendererFactory
+class StubRendererFactory : public mir::renderer::RendererFactory
 {
 public:
-    std::unique_ptr<mc::Renderer> create_renderer_for(mg::DisplayBuffer&)
+    std::unique_ptr<mir::renderer::Renderer> create_renderer_for(mir::renderer::gl::RenderTarget&)
     {
-        return std::unique_ptr<mc::Renderer>(new mtd::StubRenderer());
+        return std::unique_ptr<mir::renderer::Renderer>(new mtd::StubRenderer());
     }
 };
 
@@ -82,17 +77,29 @@ mtf::StubbedServerConfiguration::StubbedServerConfiguration(
 {
 }
 
-std::shared_ptr<mg::Platform> mtf::StubbedServerConfiguration::the_graphics_platform()
-{
-    if (!graphics_platform)
-    {
-        graphics_platform = mtf::make_stubbed_server_graphics_platform(display_rects);
-    }
+mtf::StubbedServerConfiguration::~StubbedServerConfiguration() = default;
 
-    return graphics_platform;
+auto mtf::StubbedServerConfiguration::the_display_platforms()
+-> std::vector<std::shared_ptr<graphics::DisplayPlatform>> const&
+{
+    if (display_platform.empty())
+    {
+        display_platform.push_back(mtf::make_stubbed_display_platform(display_rects));
+    }
+    return display_platform;
 }
 
-std::shared_ptr<mc::RendererFactory> mtf::StubbedServerConfiguration::the_renderer_factory()
+auto mtf::StubbedServerConfiguration::the_rendering_platforms()
+    -> std::vector<std::shared_ptr<graphics::RenderingPlatform>> const&
+{
+    if (rendering_platform.empty())
+    {
+        rendering_platform.push_back(mtf::make_stubbed_rendering_platform());
+    }
+    return rendering_platform;
+}
+
+std::shared_ptr<mir::renderer::RendererFactory> mtf::StubbedServerConfiguration::the_renderer_factory()
 {
     auto options = the_options();
 
@@ -123,26 +130,6 @@ std::shared_ptr<msh::InputTargeter> mtf::StubbedServerConfiguration::the_input_t
         return std::make_shared<mi::NullInputTargeter>();
 }
 
-std::shared_ptr<mi::InputSender> mtf::StubbedServerConfiguration::the_input_sender()
-{
-    auto options = the_options();
-
-    if (options->get<bool>("tests-use-real-input"))
-        return DefaultServerConfiguration::the_input_sender();
-    else
-        return std::make_shared<mtd::StubInputSender>();
-}
-
-std::shared_ptr<mi::LegacyInputDispatchable> mtf::StubbedServerConfiguration::the_legacy_input_dispatchable()
-{
-    auto options = the_options();
-
-    if (options->get<bool>("tests-use-real-input"))
-        return DefaultServerConfiguration::the_legacy_input_dispatchable();
-    else
-        return std::make_shared<mtd::StubLegacyInputDispatchable>();
-}
-
 std::shared_ptr<mg::Cursor> mtf::StubbedServerConfiguration::the_cursor()
 {
     return std::make_shared<mtd::StubCursor>();
