@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "mir/geometry/size.h"
@@ -41,3 +39,25 @@ TEST(geometry, size)
     EXPECT_EQ(Height(0), defaultValue.height);
     EXPECT_NE(size2x4, defaultValue);
 }
+
+TEST(geometry, size_can_be_converted_from_int_to_float)
+{
+    using namespace geom;
+
+    Size const i{1, 3};
+    SizeF const f{i};
+
+    EXPECT_EQ(WidthF(1.0), f.width);
+    EXPECT_EQ(HeightF(3.0), f.height);
+}
+
+TEST(geometry, size_can_be_converted_from_float_to_int)
+{
+    using namespace geom;
+
+    SizeF const f{1.2, 3.0};
+    Size const i{f};
+
+    EXPECT_EQ(Width(1), i.width);
+    EXPECT_EQ(Height(3), i.height);
+}
