@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,14 +12,14 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
 #include "mir/graphics/default_display_configuration_policy.h"
 #include "mir/graphics/display_configuration.h"
 #include "mir/geometry/displacement.h"
 
+#include "mir/test/doubles/stub_display_configuration.h"
+
 #include <gtest/gtest.h>
 #include <gmock/gmock.h>
 
@@ -29,59 +29,27 @@ using namespace mir::geometry;
 namespace
 {
 
-class MockDisplayConfiguration : public DisplayConfiguration
+class StubDisplayConfiguration : public mir::test::doubles::StubDisplayConfig
 {
 public:
-    MockDisplayConfiguration(MockDisplayConfiguration && m)
-        : max_simultaneous_outputs{m.max_simultaneous_outputs},
-        outputs{std::move(m.outputs)}
+    using mir::test::doubles::StubDisplayConfig::StubDisplayConfig;
+
+    StubDisplayConfiguration(
+        std::vector<DisplayConfigurationOutput> const& outputs)
+        : StubDisplayConfig{outputs}
     {
     }
-
-    MockDisplayConfiguration(size_t max_simultaneous_outputs, std::vector<DisplayConfigurationOutput> && config)
-        : max_simultaneous_outputs{max_simultaneous_outputs},
-        outputs{config}
-    {
-        if (max_simultaneous_outputs == max_simultaneous_outputs_all)
-            max_simultaneous_outputs = outputs.size();
-    }
-
-    MockDisplayConfiguration(std::vector<DisplayConfigurationOutput> && config)
-        : MockDisplayConfiguration(max_simultaneous_outputs_all, std::move(config))
-    {
-    }
-
-    void for_each_card(std::function<void(DisplayConfigurationCard const&)> f) const
-    {
-        f({DisplayConfigurationCardId{1}, max_simultaneous_outputs});
-    }
-
-    void for_each_output(std::function<void(DisplayConfigurationOutput const&)> f) const override
-    {
-        for (auto const& output : outputs)
-            f(output);
-    }
-
-    void for_each_output(std::function<void(UserDisplayConfigurationOutput&)> f)
-    {
-        for (auto& output : outputs)
-        {
-            UserDisplayConfigurationOutput user(output);
-            f(user);
-        }
-    }
-
-    static const size_t max_simultaneous_outputs_all{std::numeric_limits<size_t>::max()};
-private:
-    size_t max_simultaneous_outputs;
-    std::vector<DisplayConfigurationOutput> outputs;
 };
 
 }
 
 DisplayConfigurationOutput default_output(DisplayConfigurationOutputId id)
 {
-    return { id, DisplayConfigurationCardId{1},
+    // We name the return value just to work around an apparent clang bug/quirk
+    DisplayConfigurationOutput ret{
+        id,
+        DisplayConfigurationCardId{1},
+        DisplayConfigurationLogicalGroupId{0},
         DisplayConfigurationOutputType::vga,
         {mir_pixel_format_abgr_8888},
         { {Size{523, 555}, 60.0} },
@@ -95,8 +63,14 @@ DisplayConfigurationOutput default_output(DisplayConfigurationOutputId id)
         mir_power_mode_on,
         mir_orientation_normal,
         1.0f,
-        mir_form_factor_monitor
+        mir_form_factor_monitor,
+        mir_subpixel_arrangement_unknown,
+        {},
+        mir_output_gamma_unsupported,
+        {},
+        {}
     };
+    return ret;
 }
 
 DisplayConfigurationOutput connected_with_modes()
@@ -150,11 +124,10 @@ DisplayConfigurationOutput connected_with_xrgb_bgr()
     return output;
 }
 
-MockDisplayConfiguration create_default_configuration(size_t max_outputs = MockDisplayConfiguration::max_simultaneous_outputs_all)
+StubDisplayConfiguration create_default_configuration()
 {
-    return MockDisplayConfiguration
+    return StubDisplayConfiguration
     {
-        max_outputs,
         {
             connected_with_modes(),
             connected_without_modes(),
@@ -169,11 +142,11 @@ TEST(CloneDisplayConfigurationPolicyTest, uses_all_connected_valid_outputs)
     using namespace ::testing;
 
     CloneDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration()};
+    StubDisplayConfiguration conf{create_default_configuration()};
 
     policy.apply_to(conf);
 
-    conf.for_each_output([&conf](DisplayConfigurationOutput const& output)
+    conf.for_each_output([](DisplayConfigurationOutput const& output)
     {
         if (output.connected && output.modes.size() > 0)
         {
@@ -193,7 +166,7 @@ TEST(CloneDisplayConfigurationPolicyTest, default_policy_is_power_mode_on)
     using namespace ::testing;
 
     CloneDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration()};
+    StubDisplayConfiguration conf{create_default_configuration()};
 
     policy.apply_to(conf);
 
@@ -208,40 +181,20 @@ TEST(CloneDisplayConfigurationPolicyTest, default_orientation_is_normal)
     using namespace ::testing;
 
     CloneDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration()};
+    StubDisplayConfiguration conf{create_default_configuration()};
 
-    conf.for_each_output([&conf](DisplayConfigurationOutput const& output)
+    conf.for_each_output([](DisplayConfigurationOutput const& output)
     {
         EXPECT_EQ(mir_orientation_normal, output.orientation);
     });
 }
 
-TEST(CloneDisplayConfigurationPolicyTest, does_not_enable_more_outputs_than_supported)
-{
-    using namespace ::testing;
-
-    size_t const max_simultaneous_outputs{1};
-    CloneDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration(max_simultaneous_outputs)};
-
-    policy.apply_to(conf);
-
-    size_t used_count{0};
-    conf.for_each_output([&used_count](DisplayConfigurationOutput const& output)
-    {
-        if (output.used)
-            ++used_count;
-    });
-
-    EXPECT_GE(max_simultaneous_outputs, used_count);
-}
-
 TEST(CloneDisplayConfigurationPolicyTest, prefer_opaque_over_alpha)
 {
     using namespace ::testing;
 
     CloneDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration pick_xrgb{ { connected_with_rgba_and_xrgb() } };
+    StubDisplayConfiguration pick_xrgb{ { connected_with_rgba_and_xrgb() } };
 
     policy.apply_to(pick_xrgb);
 
@@ -256,7 +209,7 @@ TEST(CloneDisplayConfigurationPolicyTest, preserve_opaque_selection)
     using namespace ::testing;
 
     CloneDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration keep_bgr{ { connected_with_xrgb_bgr() } };
+    StubDisplayConfiguration keep_bgr{ { connected_with_xrgb_bgr() } };
 
     policy.apply_to(keep_bgr);
 
@@ -271,7 +224,7 @@ TEST(CloneDisplayConfigurationPolicyTest, accept_transparency_when_only_option)
     using namespace ::testing;
 
     CloneDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration pick_rgba{ { default_output(DisplayConfigurationOutputId{15}) } };
+    StubDisplayConfiguration pick_rgba{ { default_output(DisplayConfigurationOutputId{15}) } };
 
     policy.apply_to(pick_rgba);
 
@@ -286,13 +239,13 @@ TEST(SingleDisplayConfigurationPolicyTest, uses_first_of_connected_valid_outputs
     using namespace ::testing;
 
     SingleDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration()};
+    StubDisplayConfiguration conf{create_default_configuration()};
 
     policy.apply_to(conf);
 
     bool is_first{true};
 
-    conf.for_each_output([&conf, &is_first](DisplayConfigurationOutput const& output)
+    conf.for_each_output([&is_first](DisplayConfigurationOutput const& output)
     {
         if (output.connected && output.modes.size() > 0 && is_first)
         {
@@ -313,7 +266,7 @@ TEST(SingleDisplayConfigurationPolicyTest, default_policy_is_power_mode_on)
     using namespace ::testing;
 
     SingleDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration()};
+    StubDisplayConfiguration conf{create_default_configuration()};
 
     policy.apply_to(conf);
 
@@ -334,40 +287,21 @@ TEST(SingleDisplayConfigurationPolicyTest, default_orientation_is_normal)
     using namespace ::testing;
 
     SingleDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration()};
+    auto conf = create_default_configuration();
+    //StubDisplayConfiguration conf{create_default_configuration()};
 
-    conf.for_each_output([&conf](DisplayConfigurationOutput const& output)
+    conf.for_each_output([](DisplayConfigurationOutput const& output)
     {
         EXPECT_EQ(mir_orientation_normal, output.orientation);
     });
 }
 
-TEST(SingleDisplayConfigurationPolicyTest, does_not_enable_more_outputs_than_supported)
-{
-    using namespace ::testing;
-
-    size_t const max_simultaneous_outputs{1};
-    SingleDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration(max_simultaneous_outputs)};
-
-    policy.apply_to(conf);
-
-    size_t used_count{0};
-    conf.for_each_output([&used_count](DisplayConfigurationOutput const& output)
-    {
-        if (output.used)
-            ++used_count;
-    });
-
-    EXPECT_GE(max_simultaneous_outputs, used_count);
-}
-
 TEST(SideBySideDisplayConfigurationPolicyTest, uses_all_connected_valid_outputs)
 {
     using namespace ::testing;
 
     SideBySideDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration()};
+    StubDisplayConfiguration conf{create_default_configuration()};
 
     policy.apply_to(conf);
 
@@ -390,12 +324,43 @@ TEST(SideBySideDisplayConfigurationPolicyTest, uses_all_connected_valid_outputs)
     });
 }
 
+TEST(SideBySideDisplayConfigurationPolicyTest, placement_respects_scale)
+{
+    using namespace ::testing;
+
+    SideBySideDisplayConfigurationPolicy policy;
+    StubDisplayConfiguration conf{create_default_configuration()};
+
+    conf.for_each_output([&](UserDisplayConfigurationOutput const& output)
+    {
+        output.scale = 2.0f;
+    });
+
+    policy.apply_to(conf);
+
+    Point expected_position;
+
+    conf.for_each_output([&](DisplayConfigurationOutput const& output)
+    {
+        if (output.connected && output.modes.size() > 0)
+        {
+            EXPECT_EQ(expected_position, output.top_left);
+
+            expected_position.x += DeltaX{round(output.modes[output.current_mode_index].size.width.as_int()/2.0f)};
+        }
+        else
+        {
+            EXPECT_FALSE(output.used);
+        }
+    });
+}
+
 TEST(SideBySideDisplayConfigurationPolicyTest, default_policy_is_power_mode_on)
 {
     using namespace ::testing;
 
     SideBySideDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration()};
+    StubDisplayConfiguration conf{create_default_configuration()};
 
     policy.apply_to(conf);
 
@@ -413,32 +378,10 @@ TEST(SideBySideDisplayConfigurationPolicyTest, default_orientation_is_normal)
     using namespace ::testing;
 
     SideBySideDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration()};
+    StubDisplayConfiguration conf{create_default_configuration()};
 
-    conf.for_each_output([&conf](DisplayConfigurationOutput const& output)
+    conf.for_each_output([](DisplayConfigurationOutput const& output)
     {
         EXPECT_EQ(mir_orientation_normal, output.orientation);
     });
 }
-
-TEST(SideBySideDisplayConfigurationPolicyTest, does_not_enable_more_outputs_than_supported)
-{
-    using namespace ::testing;
-
-    size_t const max_simultaneous_outputs{1};
-    SideBySideDisplayConfigurationPolicy policy;
-    MockDisplayConfiguration conf{create_default_configuration(max_simultaneous_outputs)};
-
-    policy.apply_to(conf);
-
-    size_t used_count{0};
-    conf.for_each_output([&used_count](DisplayConfigurationOutput const& output)
-    {
-        if (output.used)
-            ++used_count;
-    });
-
-    EXPECT_GE(max_simultaneous_outputs, used_count);
-}
-
-
