@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,9 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by:
- *   Kevin DuBois <kevin.dubois@canonical.com>
  */
 
 #include "mir/graphics/egl_extensions.h"
@@ -26,50 +23,53 @@
 
 namespace mg = mir::graphics;
 namespace mtd = mir::test::doubles;
+using namespace testing;
 
-class EGLExtensionsTest  : public ::testing::Test
+typedef mtd::MockEGL::generic_function_pointer_t func_ptr_t;
+class EGLExtensions  : public ::testing::Test
 {
 protected:
     virtual void SetUp()
     {
+        mock_egl.provide_egl_extensions();
     }
 
     testing::NiceMock<mtd::MockEGL> mock_egl;
 };
 
-TEST_F(EGLExtensionsTest, constructor_throws_if_egl_image_not_supported)
+TEST_F(EGLExtensions, constructor_throws_if_egl_image_not_supported)
 {
-    using namespace testing;
-    typedef mtd::MockEGL::generic_function_pointer_t func_ptr_t;
-
     ON_CALL(mock_egl, eglGetProcAddress(StrEq("eglCreateImageKHR")))
         .WillByDefault(Return(reinterpret_cast<func_ptr_t>(0)));
     ON_CALL(mock_egl, eglGetProcAddress(StrEq("eglDestroyImageKHR")))
         .WillByDefault(Return(reinterpret_cast<func_ptr_t>(0)));
 
+    mg::EGLExtensions extensions;
+    EGLDisplay dpy = eglGetDisplay(nullptr);
+
     EXPECT_THROW({
-        mg::EGLExtensions extensions;
+        extensions.base(dpy);
     }, std::runtime_error);
 }
 
-TEST_F(EGLExtensionsTest, constructor_throws_if_gl_oes_egl_image_not_supported)
+TEST_F(EGLExtensions, base_throws_if_gl_oes_egl_image_not_supported)
 {
-    using namespace testing;
-    typedef mtd::MockEGL::generic_function_pointer_t func_ptr_t;
-
     ON_CALL(mock_egl, eglGetProcAddress(StrEq("glEGLImageTargetTexture2DOES")))
         .WillByDefault(Return(reinterpret_cast<func_ptr_t>(0)));
 
+    mg::EGLExtensions extensions;
+    EGLDisplay dpy = eglGetDisplay(nullptr);
+
     EXPECT_THROW({
-        mg::EGLExtensions extensions;
+        extensions.base(dpy);
     }, std::runtime_error);
 }
 
-TEST_F(EGLExtensionsTest, success_has_sane_function_hooks)
+TEST_F(EGLExtensions, success_has_sane_function_hooks)
 {
     mg::EGLExtensions extensions;
-    EXPECT_NE(nullptr, extensions.eglCreateImageKHR);
-    EXPECT_NE(nullptr, extensions.eglDestroyImageKHR);
-    EXPECT_NE(nullptr, extensions.glEGLImageTargetTexture2DOES);
+    EGLDisplay dpy = eglGetDisplay(nullptr);
+    EXPECT_NE(nullptr, extensions.base(dpy).eglCreateImageKHR);
+    EXPECT_NE(nullptr, extensions.base(dpy).eglDestroyImageKHR);
+    EXPECT_NE(nullptr, extensions.base(dpy).glEGLImageTargetTexture2DOES);
 }
-
