@@ -88,12 +88,18 @@ struct StubInputScene : public mtd::StubInputScene
     {
         return add_surface({{0, 0}, {1, 1}});
     }
-    
-    void for_each(std::function<void(std::shared_ptr<mi::Surface> const&)> const& exec) override
+
+    auto input_surface_at(geom::Point point) const -> std::shared_ptr<mi::Surface> override
     {
-	surfaces.for_each([&exec](std::shared_ptr<mi::Surface> const& surface) {
-            exec(surface);
-        });
+        std::shared_ptr<mi::Surface> result;
+        surfaces.for_each([&](auto surface)
+            {
+                if (surface->input_area_contains(point))
+                {
+                    result = surface;
+                }
+            });
+        return result;
     }
 
     void add_observer(std::shared_ptr<ms::Observer> const& new_observer) override
@@ -111,10 +117,16 @@ struct StubInputScene : public mtd::StubInputScene
         observer->end_observation();
         observer.reset();
     }
+
+    bool screen_is_locked() const override
+    {
+        return is_locked;
+    }
     
-    mir::ThreadSafeList<std::shared_ptr<ms::Surface>> surfaces;
+    mir::ThreadSafeList<std::shared_ptr<ms::Surface>> mutable surfaces;
 
     std::shared_ptr<ms::Observer> observer;
+    bool is_locked = false;
 };
 
 struct SurfaceInputDispatcher : public testing::Test
@@ -331,6 +343,68 @@ TEST_F(SurfaceInputDispatcher, keyboard_focus_clear_delivered_to_keyboard_observ
     executor.execute();
 }
 
+TEST_F(SurfaceInputDispatcher, keyboard_focus_clear_delivered_to_keyboard_observer_when_screen_is_locked)
+{
+    auto surface = scene.add_surface();
+    mtd::ExplicitExecutor executor;
+
+    auto const kb_observer = std::make_shared<NiceMock<MockKeyboardObserver>>();
+
+    dispatcher.start();
+    dispatcher.register_interest(kb_observer, executor);
+    dispatcher.set_focus(surface);
+    executor.execute();
+
+    EXPECT_CALL(*kb_observer, keyboard_focus_set(Eq(nullptr)));
+    scene.is_locked = true;
+    scene.observer->scene_changed();
+    executor.execute();
+}
+
+TEST_F(SurfaceInputDispatcher, keyboard_focus_not_set_when_screen_is_locked)
+{
+    auto surface = scene.add_surface();
+    mtd::ExplicitExecutor executor;
+
+    auto const kb_observer = std::make_shared<NiceMock<MockKeyboardObserver>>();
+
+    dispatcher.start();
+    dispatcher.register_interest(kb_observer, executor);
+    executor.execute();
+
+    scene.is_locked = true;
+    scene.observer->scene_changed();
+    executor.execute();
+
+    EXPECT_CALL(*kb_observer, keyboard_focus_set(_)).Times(0);
+    dispatcher.set_focus(surface);
+    executor.execute();
+}
+
+TEST_F(SurfaceInputDispatcher, keyboard_focus_can_be_set_after_screen_is_unlocked)
+{
+    auto surface = scene.add_surface();
+    mtd::ExplicitExecutor executor;
+
+    auto const kb_observer = std::make_shared<NiceMock<MockKeyboardObserver>>();
+
+    dispatcher.start();
+    dispatcher.register_interest(kb_observer, executor);
+    executor.execute();
+
+    scene.is_locked = true;
+    scene.observer->scene_changed();
+    executor.execute();
+
+    scene.is_locked = false;
+    scene.observer->scene_changed();
+    executor.execute();
+
+    EXPECT_CALL(*kb_observer, keyboard_focus_set(Eq(surface)));
+    dispatcher.set_focus(surface);
+    executor.execute();
+}
+
 TEST_F(SurfaceInputDispatcher, pointer_motion_delivered_to_client_under_pointer)
 {
     auto surface = scene.add_surface({{0, 0}, {5, 5}});
@@ -475,8 +549,8 @@ TEST_F(SurfaceInputDispatcher, touch_delivered_to_surface)
     auto surface = scene.add_surface({{1, 1}, {1, 1}});
 
     InSequence seq;
-    EXPECT_CALL(*surface, consume(mt::TouchEvent(0,0))).Times(1);
-    EXPECT_CALL(*surface, consume(mt::TouchUpEvent(0,0))).Times(1);
+    EXPECT_CALL(*surface, consume(mt::TouchEvent(1,1))).Times(1);
+    EXPECT_CALL(*surface, consume(mt::TouchUpEvent(1,1))).Times(1);
 
     dispatcher.start();
 
@@ -491,16 +565,16 @@ TEST_F(SurfaceInputDispatcher, touch_delivered_only_to_top_surface)
     auto surface = scene.add_surface({{1, 1}, {3, 3}});
 
     InSequence seq;
-    EXPECT_CALL(*surface, consume(mt::TouchEvent(0,0))).Times(1);
+    EXPECT_CALL(*surface, consume(mt::TouchEvent(1,1))).Times(1);
     EXPECT_CALL(*surface, consume(mt::TouchUpEvent(1,1))).Times(1);
-    EXPECT_CALL(*bottom_surface, consume(mt::TouchEvent(0,0))).Times(0);
-    EXPECT_CALL(*bottom_surface, consume(mt::TouchUpEvent(0,0))).Times(0);
+    EXPECT_CALL(*bottom_surface, consume(mt::TouchEvent(1,1))).Times(0);
+    EXPECT_CALL(*bottom_surface, consume(mt::TouchUpEvent(1,1))).Times(0);
 
     dispatcher.start();
 
     FakeToucher toucher;
     EXPECT_TRUE(dispatcher.dispatch(toucher.touch_at({1,1})));
-    EXPECT_TRUE(dispatcher.dispatch(toucher.release_at({2,2})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.release_at({1,1})));
 }
 
 TEST_F(SurfaceInputDispatcher, gestures_persist_over_touch_down)
