@@ -149,7 +149,7 @@ public:
     {
         return std::make_shared<mgg::Platform>(
                mir::report::null_display_report(),
-               std::make_shared<mtd::StubConsoleServices>(),
+               *std::make_shared<mtd::StubConsoleServices>(),
                *std::make_shared<mtd::NullEmergencyCleanup>(),
                mgg::BypassOption::allowed,
                std::make_unique<mgg::Quirks>(mir::options::ProgramOption{}));
