@@ -31,6 +31,7 @@
 #include "mir/test/doubles/fake_alarm_factory.h"
 #include "mir/test/doubles/mock_display_configuration_observer.h"
 
+#include "gmock/gmock.h"
 #include <mutex>
 #include <boost/throw_exception.hpp>
 
@@ -203,6 +204,55 @@ TEST_F(MediatingDisplayChangerTest, does_not_pause_system_when_applying_new_conf
                        mt::fake_shared(conf));
 }
 
+TEST_F(MediatingDisplayChangerTest, does_full_stop_start_configuration_if_preserving_db_applies_incomplete_configuration)
+{
+    mtd::NullDisplayConfiguration conf;
+    auto session = std::make_shared<mtd::StubSession>();
+
+    InSequence seq;
+
+    EXPECT_CALL(mock_display, apply_if_configuration_preserves_display_buffers(Ref(conf)))
+        .WillOnce(Throw(mg::Display::IncompleteConfigurationApplied{"Quack!"}));
+
+    EXPECT_CALL(mock_compositor, stop());
+    EXPECT_CALL(mock_display, configure(Ref(conf)));
+    EXPECT_CALL(mock_compositor, start());
+
+    session_event_sink.handle_focus_change(session);
+    changer->configure(session,
+                       mt::fake_shared(conf));
+}
+
+TEST_F(MediatingDisplayChangerTest, sends_error_when_incomplete_fallback_configuration_fails)
+{
+    mtd::NullDisplayConfiguration conf;
+    auto session = std::make_shared<mtd::StubSession>();
+
+    InSequence seq;
+
+    // First we try pause-less configuration, which fails awkwardly…
+    EXPECT_CALL(mock_display, apply_if_configuration_preserves_display_buffers(Ref(conf)))
+        .WillOnce(Throw(mg::Display::IncompleteConfigurationApplied{"Quack!"}));
+
+    // …then we go through the full tear-down-and-rebuild configuration path…
+    EXPECT_CALL(mock_compositor, stop());
+    EXPECT_CALL(mock_display, configure(Ref(conf)))
+        .WillOnce(Throw(std::runtime_error{"Oooof"}));    //… which also fails! Awkward!
+    EXPECT_CALL(mock_compositor, start());
+
+    // So we notify of error, and then…
+    EXPECT_CALL(display_configuration_observer, configuration_failed(Pointee(Ref(conf)), _));
+
+    // …we revert to the previous configuration
+    EXPECT_CALL(mock_compositor, stop());
+    EXPECT_CALL(mock_display, configure(_));
+    EXPECT_CALL(mock_compositor, start());
+
+    session_event_sink.handle_focus_change(session);
+    changer->configure(session,
+                       mt::fake_shared(conf));
+}
+
 TEST_F(MediatingDisplayChangerTest, sends_error_when_applying_new_configuration_for_focused_session_fails)
 {
     mtd::NullDisplayConfiguration conf;
@@ -532,7 +582,7 @@ TEST_F(MediatingDisplayChangerTest, focusing_a_session_without_attached_config_a
     session_event_sink.handle_focus_change(session2);
 }
 
-TEST_F(MediatingDisplayChangerTest, focusing_a_session_without_attached_config_applies_base_config_not_pausing_if_db_preserved)
+TEST_F(MediatingDisplayChangerTest, focusing_a_session_without_attached_config_applies_base_config_pausing_if_db_content_not_preserved)
 {
     std::shared_ptr<mg::DisplayConfiguration> conf = base_config.clone();
     conf->for_each_output(
@@ -556,6 +606,42 @@ TEST_F(MediatingDisplayChangerTest, focusing_a_session_without_attached_config_a
     Mock::VerifyAndClearExpectations(&mock_compositor);
     Mock::VerifyAndClearExpectations(&mock_display);
 
+    EXPECT_CALL(
+        mock_display,
+        apply_if_configuration_preserves_display_buffers(mt::DisplayConfigMatches(std::cref(base_config))))
+            .WillOnce(Return(true));
+
+    EXPECT_CALL(mock_compositor, stop()).Times(1);
+    EXPECT_CALL(mock_display, configure(_)).Times(0);
+    EXPECT_CALL(mock_compositor, start()).Times(1);
+
+    session_event_sink.handle_focus_change(session2);
+}
+
+TEST_F(MediatingDisplayChangerTest, focusing_a_session_without_attached_config_applies_base_config_not_pausing_if_db_content_preserved)
+{
+    std::shared_ptr<mg::DisplayConfiguration> conf = base_config.clone();
+    conf->for_each_output(
+        [](mg::UserDisplayConfigurationOutput& output)
+        {
+            if (output.used)
+            {
+                output.form_factor =
+                    output.form_factor == mir_form_factor_monitor ? mir_form_factor_tv : mir_form_factor_monitor;
+            }
+        });
+
+    auto session1 = std::make_shared<mtd::StubSession>();
+    auto session2 = std::make_shared<mtd::StubSession>();
+
+    session_container.insert_session(session1);
+    changer->configure(session1, conf);
+
+    session_event_sink.handle_focus_change(session1);
+
+    Mock::VerifyAndClearExpectations(&mock_compositor);
+    Mock::VerifyAndClearExpectations(&mock_display);
+
     EXPECT_CALL(
         mock_display,
         apply_if_configuration_preserves_display_buffers(mt::DisplayConfigMatches(std::cref(base_config))))
