@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,20 +12,17 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored By: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "src/server/scene/prompt_session_manager_impl.h"
 
-#include "src/server/scene/session_container.h"
-
+#include "mir/scene/session_container.h"
 #include "mir/scene/prompt_session.h"
 #include "mir/scene/prompt_session_creation_parameters.h"
 
-#include "mir_test_doubles/mock_prompt_session_listener.h"
-#include "mir_test_doubles/stub_scene_session.h"
-#include "mir_test/fake_shared.h"
+#include "mir/test/doubles/mock_prompt_session_listener.h"
+#include "mir/test/doubles/stub_session.h"
+#include "mir/test/fake_shared.h"
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
@@ -38,42 +35,17 @@ using namespace ::testing;
 
 namespace
 {
-struct StubSessionContainer : ms::SessionContainer
-{
-    void insert_session(std::shared_ptr<ms::Session> const& session)
-    {
-        sessions.push_back(session);
-    }
-
-    void remove_session(std::shared_ptr<ms::Session> const&)
-    {
-    }
-
-    void for_each(std::function<void(std::shared_ptr<ms::Session> const&)> f) const
-    {
-        for (auto const& session : sessions)
-            f(session);
-    }
-
-    std::shared_ptr<ms::Session> successor_of(std::shared_ptr<ms::Session> const&) const
-    {
-        return {};
-    }
-
-    std::vector<std::shared_ptr<ms::Session>> sessions;
-};
-
 struct PromptSessionManager : public testing::Test
 {
     pid_t const helper_pid = __LINE__;
     pid_t const application_pid = __LINE__;
     pid_t const prompt_provider_pid = __LINE__;
-    std::shared_ptr<ms::Session> const helper{std::make_shared<mtd::StubSceneSession>(helper_pid)};
-    std::shared_ptr<ms::Session> const application_session{std::make_shared<mtd::StubSceneSession>(application_pid)};
-    std::shared_ptr<ms::Session> const provider_session{std::make_shared<mtd::StubSceneSession>(prompt_provider_pid)};
-    std::shared_ptr<ms::Session> const another_prompt_provider{std::make_shared<mtd::StubSceneSession>(__LINE__)};
+    std::shared_ptr<ms::Session> const helper{std::make_shared<mtd::StubSession>(helper_pid)};
+    std::shared_ptr<ms::Session> const application_session{std::make_shared<mtd::StubSession>(application_pid)};
+    std::shared_ptr<ms::Session> const provider_session{std::make_shared<mtd::StubSession>(prompt_provider_pid)};
+    std::shared_ptr<ms::Session> const another_prompt_provider{std::make_shared<mtd::StubSession>(__LINE__)};
     ms::PromptSessionCreationParameters parameters;
-    StubSessionContainer existing_sessions;
+    ms::SessionContainer existing_sessions;
 
     NiceMock<mtd::MockPromptSessionListener> prompt_session_listener;
 
