@@ -94,7 +94,7 @@ struct StubSurface : public ms::BasicSurface
             {},
             {},
             "stub",
-            {{},{}},
+            {{-1, -1}, {2, 2}},
             mir_pointer_unconfined,
             std::list<ms::StreamInfo> { { stream, {}, {} } },
             {},
@@ -144,7 +144,9 @@ struct SurfaceStack : public ::testing::Test
     std::shared_ptr<ms::Surface> invisible_stub_surface;
 
     std::shared_ptr<ms::SceneReport> const report = mr::null_scene_report();
-    ms::SurfaceStack stack{report};
+    // The surface stack must be a shared pointer so shared_from_this() works
+    std::shared_ptr<ms::SurfaceStack> shared_stack = std::make_shared<ms::SurfaceStack>(report);
+    ms::SurfaceStack& stack = *shared_stack;
     void const* compositor_id{&stack};
     mtd::ExplicitExecutor executor;
 };
@@ -531,9 +533,7 @@ TEST_F(SurfaceStack, scene_observer_can_query_scene_within_surface_exists_notifi
     MockSceneObserver observer;
 
     auto const scene_query = [&]{
-        stack.for_each([&](std::shared_ptr<mi::Surface> const& surface){
-            EXPECT_THAT(surface.get(), Eq(stub_surface1.get()));
-        });
+        EXPECT_THAT(stack.input_surface_at({}).get(), Eq(stub_surface1.get()));
     };
     EXPECT_CALL(observer, surface_exists(stub_surface1)).Times(1)
         .WillOnce(InvokeWithoutArgs(scene_query));
@@ -549,9 +549,7 @@ TEST_F(SurfaceStack, scene_observer_can_async_query_scene_within_surface_exists_
     MockSceneObserver observer;
 
     auto const scene_query = [&]{
-        stack.for_each([&](std::shared_ptr<mi::Surface> const& surface){
-            EXPECT_THAT(surface.get(), Eq(stub_surface1.get()));
-        });
+        EXPECT_THAT(stack.input_surface_at({}).get(), Eq(stub_surface1.get()));
     };
 
     auto const async_scene_query = [&]{
@@ -874,8 +872,10 @@ TEST_F(SurfaceStack, scene_observer_notified_of_add_and_remove_input_visualizati
     stack.remove_input_visualization(mt::fake_shared(r));
 }
 
-TEST_F(SurfaceStack, overlays_do_not_appear_in_input_enumeration)
+TEST_F(SurfaceStack, overlays_do_not_interfere_with_finding_input_surface)
 {
+    using namespace ::testing;
+
     mtd::StubRenderable r;
 
     stack.add_surface(stub_surface1, mi::InputReceptionMode::normal);
@@ -887,12 +887,20 @@ TEST_F(SurfaceStack, overlays_do_not_appear_in_input_enumeration)
 
     stack.add_input_visualization(mt::fake_shared(r));
 
-    unsigned int observed_input_targets = 0;
-    stack.for_each([&observed_input_targets](std::shared_ptr<mi::Surface> const&)
-        {
-            observed_input_targets++;
-        });
-    EXPECT_EQ(2, observed_input_targets);
+    EXPECT_THAT(stack.input_surface_at({}).get(), Eq(stub_surface2.get()));
+}
+
+TEST_F(SurfaceStack, overlays_do_not_interfere_with_surface_at)
+{
+    using namespace ::testing;
+
+    mtd::StubRenderable r{{{-1, -1}, {2, 2}}};
+
+    stack.add_surface(stub_surface1, mi::InputReceptionMode::normal);
+    stack.add_input_visualization(mt::fake_shared(r));
+
+    EXPECT_THAT(stack.surface_at({}).get(), Eq(stub_surface1.get()));
+    EXPECT_THAT(stack.input_surface_at({}).get(), Eq(stub_surface1.get()));
 }
 
 TEST_F(SurfaceStack, overlays_appear_at_top_of_renderlist)
@@ -952,7 +960,7 @@ TEST_F(SurfaceStack, scene_observers_notified_of_generic_scene_change)
     stack.emit_scene_changed();
 }
 
-TEST_F(SurfaceStack, for_each_enumerates_all_input_surfaces)
+TEST_F(SurfaceStack, input_surface_at_finds_top_surface)
 {
     using namespace ::testing;
 
@@ -960,22 +968,12 @@ TEST_F(SurfaceStack, for_each_enumerates_all_input_surfaces)
     stack.add_surface(stub_surface2, mi::InputReceptionMode::normal);
     stack.add_surface(stub_surface3, mi::InputReceptionMode::normal);
 
-    stub_surface1->configure(mir_window_attrib_visibility, MirWindowVisibility::mir_window_visibility_exposed);
-    stub_surface2->configure(mir_window_attrib_visibility, MirWindowVisibility::mir_window_visibility_occluded);
-    stub_surface3->configure(mir_window_attrib_visibility, MirWindowVisibility::mir_window_visibility_occluded);
-
-    int num_exposed_surfaces = 0;
-    auto const count_exposed_surfaces = [&num_exposed_surfaces](std::shared_ptr<mi::Surface> const&){
-        num_exposed_surfaces++;
-    };
-
-    stack.for_each(count_exposed_surfaces);
-    EXPECT_THAT(num_exposed_surfaces, Eq(3));
+    EXPECT_THAT(stack.input_surface_at({}).get(), Eq(stub_surface3.get()));
 }
 
 using namespace ::testing;
 
-TEST_F(SurfaceStack, returns_top_surface_under_cursor)
+TEST_F(SurfaceStack, surface_at_returns_top_surface_under_cursor)
 {
     geom::Point const cursor_over_all {100, 100};
     geom::Point const cursor_over_12  {200, 100};
@@ -996,7 +994,51 @@ TEST_F(SurfaceStack, returns_top_surface_under_cursor)
     EXPECT_THAT(stack.surface_at(cursor_over_none).get(), IsNull());
 }
 
-TEST_F(SurfaceStack, returns_top_visible_surface_under_cursor)
+TEST_F(SurfaceStack, input_surface_at_returns_top_surface_under_cursor)
+{
+    geom::Point const cursor_over_all {100, 100};
+    geom::Point const cursor_over_12  {200, 100};
+    geom::Point const cursor_over_1   {600, 600};
+    geom::Point const cursor_over_none{999, 999};
+
+    stack.add_surface(stub_surface1, mi::InputReceptionMode::normal);
+    stack.add_surface(stub_surface2, mi::InputReceptionMode::normal);
+    stack.add_surface(stub_surface3, mi::InputReceptionMode::normal);
+
+    stub_surface1->resize({900, 900});
+    stub_surface2->resize({500, 200});
+    stub_surface3->resize({200, 500});
+
+    EXPECT_THAT(stack.surface_at(cursor_over_all),  Eq(stub_surface3));
+    EXPECT_THAT(stack.surface_at(cursor_over_12),   Eq(stub_surface2));
+    EXPECT_THAT(stack.surface_at(cursor_over_1),    Eq(stub_surface1));
+    EXPECT_THAT(stack.surface_at(cursor_over_none).get(), IsNull());
+}
+
+TEST_F(SurfaceStack, surface_at_returns_top_visible_surface_under_cursor)
+{
+    geom::Point const cursor_over_all {100, 100};
+    geom::Point const cursor_over_12  {200, 100};
+    geom::Point const cursor_over_1   {600, 600};
+    geom::Point const cursor_over_none{999, 999};
+
+    stack.add_surface(stub_surface1, mi::InputReceptionMode::normal);
+    stack.add_surface(stub_surface2, mi::InputReceptionMode::normal);
+    stack.add_surface(stub_surface3, mi::InputReceptionMode::normal);
+    stack.add_surface(invisible_stub_surface, mi::InputReceptionMode::normal);
+
+    stub_surface1->resize({900, 900});
+    stub_surface2->resize({500, 200});
+    stub_surface3->resize({200, 500});
+    invisible_stub_surface->resize({999, 999});
+
+    EXPECT_THAT(stack.surface_at(cursor_over_all),  Eq(stub_surface3));
+    EXPECT_THAT(stack.surface_at(cursor_over_12),   Eq(stub_surface2));
+    EXPECT_THAT(stack.surface_at(cursor_over_1),    Eq(stub_surface1));
+    EXPECT_THAT(stack.surface_at(cursor_over_none).get(), IsNull());
+}
+
+TEST_F(SurfaceStack, input_surface_at_returns_top_visible_surface_under_cursor)
 {
     geom::Point const cursor_over_all {100, 100};
     geom::Point const cursor_over_12  {200, 100};
@@ -1265,5 +1307,70 @@ TEST_F(SurfaceStack, all_depth_layers_are_handled)
                 SceneElementForStream(stub_buffer_stream2)))
             << "A surface at " << depth_layer_names[i] << " could not be raised over another on the same layer";
     }
+}
 
+TEST_F(SurfaceStack, screen_can_be_locked)
+{
+    NiceMock<MockSceneObserver> observer;
+    stack.add_observer(mt::fake_shared(observer));
+    EXPECT_THAT(stack.screen_is_locked(), Eq(false));
+    EXPECT_CALL(observer, scene_changed());
+
+    auto handle = stack.lock_screen();
+    EXPECT_THAT(stack.screen_is_locked(), Eq(true));
+    Mock::VerifyAndClearExpectations(&observer);
+    handle->allow_to_be_dropped();
+}
+
+TEST_F(SurfaceStack, screen_can_be_unlocked)
+{
+    auto handle = stack.lock_screen();
+    EXPECT_THAT(stack.screen_is_locked(), Eq(true));
+
+    NiceMock<MockSceneObserver> observer;
+    stack.add_observer(mt::fake_shared(observer));
+    EXPECT_CALL(observer, scene_changed());
+
+    handle->allow_to_be_dropped();
+    handle.reset();
+    EXPECT_THAT(stack.screen_is_locked(), Eq(false));
+    Mock::VerifyAndClearExpectations(&observer);
+}
+
+TEST_F(SurfaceStack, screen_is_not_unlocked_until_all_handles_are_dropped)
+{
+    NiceMock<MockSceneObserver> observer;
+    stack.add_observer(mt::fake_shared(observer));
+    EXPECT_CALL(observer, scene_changed());
+
+    auto handle1 = stack.lock_screen();
+    Mock::VerifyAndClearExpectations(&observer);
+    EXPECT_CALL(observer, scene_changed()).Times(0);
+
+    auto handle2 = stack.lock_screen();
+    auto handle3 = stack.lock_screen();
+    EXPECT_THAT(stack.screen_is_locked(), Eq(true));
+    handle2->allow_to_be_dropped();
+    handle2.reset();
+    EXPECT_THAT(stack.screen_is_locked(), Eq(true));
+    handle1->allow_to_be_dropped();
+    handle1.reset();
+    EXPECT_THAT(stack.screen_is_locked(), Eq(true));
+    EXPECT_CALL(observer, scene_changed());
+
+    handle3->allow_to_be_dropped();
+    handle3.reset();
+    EXPECT_THAT(stack.screen_is_locked(), Eq(false));
+}
+
+TEST_F(SurfaceStack, when_screen_is_locked_surface_ignores_surface)
+{
+    geom::Point const cursor_position{100, 100};
+    stack.add_surface(stub_surface1, mi::InputReceptionMode::normal);
+    stub_surface1->resize({200, 200});
+    EXPECT_THAT(stack.surface_at(cursor_position), Eq(stub_surface1));
+
+    auto handle = stack.lock_screen();
+    EXPECT_THAT(stack.surface_at(cursor_position).get(), IsNull());
+    handle->allow_to_be_dropped();
 }
