@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,19 +12,19 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Christopher James Halse Rogers <christopher.halse.rogers@canonical.com>
  */
 
 #include "mir_test_framework/udev_environment.h"
 #include "mir/udev/wrapper.h"
+#include "mir/test/death.h"
 
 #include <gtest/gtest.h>
+#include <gmock/gmock.h>
 #include <memory>
 #include <stdexcept>
-#include <umockdev.h>
 #include <libudev.h>
 #include <poll.h>
+#include <sys/sysmacros.h>
 
 namespace mtf=mir_test_framework;
 
@@ -119,6 +119,52 @@ TEST_F(UdevWrapperTest, UdevDeviceHasCorrectDevNode)
     ASSERT_STREQ("/dev/dri/card0", dev->devnode());
 }
 
+TEST_F(UdevWrapperTest, UdevDeviceHasCorrectProperties)
+{
+    auto sysfs_path = udev_environment.add_device("drm", "card0", NULL, {},
+        {"AWESOME", "yes", "REALLY", "absolutely"});
+
+    mir::udev::Context ctx;
+    auto dev = ctx.device_from_syspath(sysfs_path);
+
+    ASSERT_STREQ("yes", dev->property("AWESOME"));
+    ASSERT_STREQ("absolutely", dev->property("REALLY"));
+    ASSERT_EQ(NULL, dev->property("SOMETHING_ELSE"));
+}
+
+TEST_F(UdevWrapperTest, UdevDeviceHasCorrectMajorMinorNumbers)
+{
+    using namespace testing;
+    auto sysfs_path = udev_environment.add_device("drm", "card0", NULL, {},
+                                                  {"MAJOR", "22", "MINOR", "42"});
+
+    mir::udev::Context ctx;
+    auto dev = ctx.device_from_syspath(sysfs_path);
+
+    dev_t devnum = dev->devnum();
+    EXPECT_THAT(major(devnum), Eq(22));
+    EXPECT_THAT(minor(devnum), Eq(42));
+}
+
+TEST_F(UdevWrapperTest, UdevCharDeviceFromDevnumWorks)
+{
+    // Workaround spurious failures seen with UMOCKDEV_VERSION=0.14.1 (on Focal)
+    if (UMOCKDEV_VERSION < 1.17)
+        return;
+
+    using namespace testing;
+    udev_environment.add_standard_device("standard-drm-devices");
+
+    mir::udev::Context ctx;
+    // DRM devices have major 226, minors starting from 0
+    auto dev = ctx.char_device_from_devnum(makedev(226, 0));
+
+    dev_t devnum = dev->devnum();
+    EXPECT_THAT(major(devnum), Eq(226));
+    EXPECT_THAT(minor(devnum), Eq(0));
+    EXPECT_THAT(dev->devnode(), StrEq("/dev/dri/card0"));
+}
+
 TEST_F(UdevWrapperTest, UdevDeviceComparisonIsReflexive)
 {
     auto sysfs_path = udev_environment.add_device("drm", "card0", NULL, {}, {});
@@ -280,7 +326,7 @@ TEST_F(UdevWrapperDeathTest, DereferencingEndReturnsInvalidObject)
 
     devices.scan_devices();
 
-    EXPECT_EXIT((*devices.end()).subsystem(), KilledByInvalidMemoryAccess, "");
+    MIR_EXPECT_EXIT((*devices.end()).subsystem(), KilledByInvalidMemoryAccess, "");
 
     auto iter = devices.begin();
 
@@ -288,7 +334,7 @@ TEST_F(UdevWrapperDeathTest, DereferencingEndReturnsInvalidObject)
     {
         iter++;
     }
-    EXPECT_EXIT((*iter).subsystem(), KilledByInvalidMemoryAccess, "");
+    MIR_EXPECT_EXIT((*iter).subsystem(), KilledByInvalidMemoryAccess, "");
 }
 
 TEST_F(UdevWrapperTest, MemberDereferenceWorks)
@@ -315,7 +361,7 @@ TEST_F(UdevWrapperDeathTest, MemberDereferenceOfEndDies)
 
     devices.scan_devices();
 
-    EXPECT_EXIT(devices.end()->subsystem(), KilledByInvalidMemoryAccess, "");
+    MIR_EXPECT_EXIT(devices.end()->subsystem(), KilledByInvalidMemoryAccess, "");
 
     auto iter = devices.begin();
 
@@ -323,7 +369,7 @@ TEST_F(UdevWrapperDeathTest, MemberDereferenceOfEndDies)
     {
         iter++;
     }
-    EXPECT_EXIT(iter->subsystem(), KilledByInvalidMemoryAccess, "");
+    MIR_EXPECT_EXIT(iter->subsystem(), KilledByInvalidMemoryAccess, "");
 }
 
 TEST_F(UdevWrapperTest, UdevMonitorDoesNotTriggerBeforeEnabling)
@@ -404,7 +450,7 @@ TEST_F(UdevWrapperTest, UdevMonitorEventHasCorrectDeviceDetails)
     monitor.enable();
 
     auto sysfs_path = udev_environment.add_device("drm", "control64D", NULL, {}, {});
-    auto device = ctx.device_from_syspath(sysfs_path);
+    std::shared_ptr<mir::udev::Device> device = ctx.device_from_syspath(sysfs_path);
 
     monitor.process_events(
         [&event_handler_called, device](mir::udev::Monitor::EventType, mir::udev::Device const& dev)
@@ -467,7 +513,7 @@ TEST_F(UdevWrapperTest, UdevMonitorFiltersByPathAndType)
     monitor.enable();
 
     auto test_sysfspath = udev_environment.add_device("drm", "control64D", NULL, {}, {"DEVTYPE", "drm_minor"});
-    auto minor_device = ctx.device_from_syspath(test_sysfspath);
+    std::shared_ptr<mir::udev::Device> minor_device = ctx.device_from_syspath(test_sysfspath);
     udev_environment.add_device("drm", "card0-LVDS1", test_sysfspath.c_str(), {}, {});
     udev_environment.add_device("usb", "mightymouse", NULL, {}, {});
 
@@ -494,10 +540,10 @@ TEST_F(UdevWrapperTest, UdevMonitorFiltersAreAdditive)
     monitor.enable();
 
     auto drm_sysfspath = udev_environment.add_device("drm", "control64D", NULL, {}, {"DEVTYPE", "drm_minor"});
-    auto drm_device = ctx.device_from_syspath(drm_sysfspath);
+    std::shared_ptr<mir::udev::Device> drm_device = ctx.device_from_syspath(drm_sysfspath);
     udev_environment.add_device("drm", "card0-LVDS1", drm_sysfspath.c_str(), {}, {});
     auto usb_sysfspath = udev_environment.add_device("usb", "mightymouse", NULL, {}, {"DEVTYPE", "hid"});
-    auto usb_device = ctx.device_from_syspath(usb_sysfspath);
+    std::shared_ptr<mir::udev::Device> usb_device = ctx.device_from_syspath(usb_sysfspath);
 
     monitor.process_events([&drm_event_recieved, drm_device, &usb_event_received, usb_device]
         (mir::udev::Monitor::EventType, mir::udev::Device const& dev)
@@ -524,7 +570,7 @@ TEST_F(UdevWrapperTest, UdevMonitorFiltersApplyAfterEnable)
     monitor.filter_by_subsystem_and_type("drm", "drm_minor");
 
     auto test_sysfspath = udev_environment.add_device("drm", "control64D", NULL, {}, {"DEVTYPE", "drm_minor"});
-    auto minor_device = ctx.device_from_syspath(test_sysfspath);
+    std::shared_ptr<mir::udev::Device> minor_device = ctx.device_from_syspath(test_sysfspath);
     udev_environment.add_device("drm", "card0-LVDS1", test_sysfspath.c_str(), {}, {});
     udev_environment.add_device("usb", "mightymouse", NULL, {}, {});
 
